@@ -78,6 +78,21 @@ extern const char* System_stringReplace(const char* str, const char* source, con
   return res;
 }
 
+extern const char* System_makeC89Identifier(const char* str)
+{
+  int i=0, len=strlen(str);
+  char *res = omc_alloc_interface.malloc_strdup(str);
+  if (!((res[0]>='a' && res[0]<='z') || (res[0]>='A' && res[0]<='Z'))) {
+    res[0] = '_';
+  }
+  for (i=1; i<len; i++) {
+    if (!((res[i]>='a' && res[i]<='z') || (res[i]>='A' && res[i]<='Z') || (res[i]>='0' && res[i]<='9'))) {
+      res[i] = '_';
+    }
+  }
+  return res;
+}
+
 extern int System_stringFind(const char* str, const char* searchStr)
 {
   const char *found = strstr(str, searchStr);
@@ -274,6 +289,16 @@ extern void System_setHasExpandableConnectors(int b)
   hasExpandableConnectors = b;
 }
 
+extern int System_getHasOverconstrainedConnectors()
+{
+  return hasOverconstrainedConnectors;
+}
+
+extern void System_setHasOverconstrainedConnectors(int b)
+{
+  hasOverconstrainedConnectors = b;
+}
+
 extern int System_getPartialInstantiation()
 {
   return isPartialInstantiation;
@@ -460,6 +485,47 @@ void* System_moFiles(const char *directory)
 }
 #endif
 
+#if defined(__MINGW32__) || defined(_MSC_VER)
+void* System_mocFiles(const char *directory)
+{
+  void *res;
+  WIN32_FIND_DATA FileData;
+  BOOL more = TRUE;
+  char pattern[1024];
+  HANDLE sh;
+  sprintf(pattern, "%s\\*.moc", directory);
+  res = mmc_mk_nil();
+  sh = FindFirstFile(pattern, &FileData);
+  if (sh != INVALID_HANDLE_VALUE) {
+    while(more) {
+      if (strcmp(FileData.cFileName,"package.moc") != 0)
+      {
+        res = mmc_mk_cons(mmc_mk_scon(FileData.cFileName),res);
+      }
+      more = FindNextFile(sh, &FileData);
+    }
+    if (sh != INVALID_HANDLE_VALUE) FindClose(sh);
+  }
+  return res;
+}
+#else
+void* System_mocFiles(const char *directory)
+{
+  int i,count;
+  void *res;
+  struct dirent **files;
+  select_from_dir = directory;
+  count = scandir(directory, &files, file_select_moc, NULL);
+  res = mmc_mk_nil();
+  for (i=0; i<count; i++)
+  {
+    res = mmc_mk_cons(mmc_mk_scon(files[i]->d_name),res);
+    free(files[i]);
+  }
+  return res;
+}
+#endif
+
 extern int System_lookupFunction(int _inLibHandle, const char* _inFunc)
 {
   int res = SystemImpl__lookupFunction(_inLibHandle, _inFunc);
@@ -482,6 +548,15 @@ extern int System_userIsRoot()
   return CONFIG_USER_IS_ROOT;
 }
 
+extern int System_getuid()
+{
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  return 0;
+#else
+  return getuid();
+#endif
+}
+
 extern const char* System_readEnv(const char *envname)
 {
   char *envvalue = getenv(envname);
@@ -721,6 +796,26 @@ extern const char* System_snprintff(const char *fmt, int len, double d)
   return buf;
 }
 
+extern const char* System_sprintff(const char *fmt, double d)
+{
+  char *buf;
+  const int buf_size = 20;
+  buf = ModelicaAllocateString(buf_size);
+
+  int len = snprintf(buf, buf_size, fmt, d);
+
+  if (len < 0) {
+    MMC_THROW();
+  }
+
+  if (len >= buf_size) {
+    buf = ModelicaAllocateString(len + 1);
+    snprintf(buf, len, fmt, d);
+  }
+
+  return buf;
+}
+
 extern const char* System_realpath(const char *path)
 {
   char buf[PATH_MAX];
@@ -790,26 +885,54 @@ extern void* System_launchParallelTasks(threadData_t *threadData, int numThreads
 {
   int len = listLength(dataLst), i;
   void *result = mmc_mk_nil();
-  thread_data data;
+  thread_data data = {0};
 #if !defined(_MSC_VER)
   void *commands[len];
   void *status[len];
-  int ids[len];
   pthread_t th[numThreads];
   int isInteger = 0;
-  memset(th, 0, numThreads*sizeof(pthread_t)); /* Make sure we get nothing unexpected here */
 
 #if defined(__MINGW32__)
-  /* adrpo: set thread stack size on Windows to 2MB */
+  /* adrpo: set thread stack size on Windows to 4MB */
   pthread_attr_t attr;
-  pthread_attr_init(&attr);
-  pthread_attr_setstacksize(&attr, 2097152);
+  if (pthread_attr_init(&attr))
+  {
+    const char *tok[1] = {strerror(errno)};
+    data.fail = 1;
+    c_add_message(NULL,5999,
+      ErrorType_scripting,
+      ErrorLevel_internal,
+      gettext("System.launchParallelTasks: failed to initialize the pthread attributes: %s"),
+      NULL,
+      0);
+    MMC_THROW_INTERNAL();
+  }
+  /* try to set a stack size of 4MB */
+  if (pthread_attr_setstacksize(&attr, 4194304))
+  {
+    /* did not work, try half 2MB */
+    if (pthread_attr_setstacksize(&attr, 2097152))
+    {
+      /* did not work, try half 1MB */
+      if (pthread_attr_setstacksize(&attr, 1048576))
+      {
+        const char *tok[1] = {strerror(errno)};
+        data.fail = 1;
+        c_add_message(NULL,5999,
+          ErrorType_scripting,
+          ErrorLevel_internal,
+          gettext("System.launchParallelTasks: failed to set the pthread stack size to 1MB: %s"),
+          NULL,
+          0);
+        MMC_THROW_INTERNAL();
+      }
+    }
+  }
 #endif
 
-#else
+#else /* MSVC */
   void **commands = (void**) omc_alloc_interface.malloc(sizeof(void*)*len);
   void **status = (void**) omc_alloc_interface.malloc(sizeof(void*)*len);
-  int *ids = (int*) omc_alloc_interface.malloc_atomic(sizeof(int)*len);
   pthread_t *th = (pthread_t*) omc_alloc_interface.malloc(sizeof(pthread_t)*numThreads);
 #endif
   if (len == 0) {
@@ -817,6 +940,12 @@ extern void* System_launchParallelTasks(threadData_t *threadData, int numThreads
   } else if (numThreads == 1 || len == 1) {
     return System_launchParallelTasksSerial(threadData,dataLst,fn);
   }
+
+  /* Make sure we get nothing unexpected here */
+  memset(commands, 0, len*sizeof(void*));
+  memset(status, 0, len*sizeof(void*));
+  memset(th, 0, numThreads*sizeof(pthread_t));
+
   pthread_mutex_init(&data.mutex,NULL);
   data.fn = fn;
   data.current = 0;
