@@ -106,16 +106,6 @@ typedef void* iconv_t;
 #define S_IFLNK  0120000  /* symbolic link */
 
 #include <sys/types.h>
-#if defined(_MSC_VER)
-  #include <win32_dirent.h>
-  #if !defined(PATH_MAX)
-  #define PATH_MAX MAX_PATH
-  #endif
-#else
-  #include <dirent.h>
-#endif
-
-char *realpath(const char *path, char resolved_path[PATH_MAX]);
 
 #else
 /* includes/defines specific for LINUX/OS X */
@@ -170,7 +160,11 @@ static char *linker = (char *)def_linker;
 static char *cflags = (char *)def_cflags;
 static char *ldflags= (char *)def_ldflags;
 
+/* TODO! FIXME!
+ * we need to move these to threadData if we are to run things in parallel in OMC!
+ */
 static int hasExpandableConnectors = 0;
+static int hasOverconstrainedConnectors = 0;
 static int hasInnerOuterDefinitions = 0;
 static int hasStreamConnectors = 0;
 static int isPartialInstantiation = 0;
@@ -554,7 +548,7 @@ int runProcess(const char* cmd)
   STARTUPINFO si;
   PROCESS_INFORMATION pi;
   char *c = "cmd /c";
-  char *command = (char *)omc_alloc_interface.malloc_atomic(strlen(cmd) + strlen(c) + 2);
+  char *command = (char *)omc_alloc_interface.malloc_atomic(strlen(cmd) + strlen(c) + 4);
   DWORD exitCode = 1;
 
   ZeroMemory(&si, sizeof(si));
@@ -562,7 +556,7 @@ int runProcess(const char* cmd)
   ZeroMemory(&pi, sizeof(pi));
 
 
-  sprintf(command, "%s %s", c, cmd);
+  sprintf(command, "%s \"%s\"", c, cmd);
 
   /* fprintf(stderr, "%s\n", command); fflush(NULL); */
 
@@ -603,6 +597,9 @@ int SystemImpl__systemCall(const char* str, const char* outFile)
     if (*outFile) {
       /* redirect stdout, stderr in the fork'ed process */
       int fd = open(outFile, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+      if (fd < 0) {
+        _exit(1);
+      }
       dup2(fd, 1);
       dup2(fd, 2);
 #if defined(__APPLE_CC__)
@@ -896,6 +893,36 @@ extern int SystemImpl__createDirectory(const char *str)
   }
 }
 
+extern int SystemImpl__copyFile(const char *str_1, const char *str_2)
+{
+  int rv = 1;
+  char ch;
+  FILE *source, *target;
+
+  if (!SystemImpl__directoryExists(str_2))
+  {
+      rv = SystemImpl__createDirectory(str_2);
+  }
+
+  if (strcmp(str_1, "") == 0)
+    rv = 0;
+
+  char targetFile[100];
+  strcpy(targetFile,str_2);
+  strcat(targetFile,"/");
+  strcat(targetFile,str_1);
+
+  source = fopen(str_1, "r");
+  target = fopen(targetFile, "w");
+
+  while( ( ch = fgetc(source) ) != EOF )
+     rv=rv && fputc(ch, target);
+
+  fclose(source);
+  fclose(target);
+  return rv;
+}
+
 static char * SystemImpl__NextDir(const char * path)
 {
   char * res = NULL;
@@ -1118,7 +1145,7 @@ extern int SystemImpl__removeDirectory(const char *path)
   return retval==0;
 }
 
-extern char* SystemImpl__readFileNoNumeric(const char* filename)
+extern const char* SystemImpl__readFileNoNumeric(const char* filename)
 {
   char* buf, *bufRes;
   int res,numCount;
@@ -1180,6 +1207,24 @@ static int file_select_mo(direntry entry)
   }
 }
 
+static int file_select_moc(direntry entry)
+{
+  char* ptr;
+  if ((strcmp(entry->d_name, ".") == 0) ||
+      (strcmp(entry->d_name, "..") == 0) ||
+      (strcmp(entry->d_name, "package.moc") == 0)) {
+    return (0);
+  } else {
+    ptr = (char*)rindex(entry->d_name, '.');
+    if ((ptr != NULL) &&
+  ((strcmp(ptr, ".moc") == 0))) {
+      return (1);
+    } else {
+      return (0);
+    }
+  }
+}
+
 #endif
 
 #if defined(__MINGW32__) || defined(_MSC_VER)
@@ -1916,13 +1961,13 @@ static modelicaPathEntry* getAllModelicaPaths(const char *name, size_t nlen, voi
 #else
         mightbedir = 1;
 #endif
-        if (mightbedir && regularFileExistsInDirectory(mp,ent->d_name,"package.mo")) {
+        if (mightbedir && (regularFileExistsInDirectory(mp,ent->d_name,"package.mo") || regularFileExistsInDirectory(mp,ent->d_name,"package.moc"))) {
           /* fprintf(stderr, "found match %d %s\n", *numMatches, ent->d_name); */
           (*numMatches)++;
           continue;
         }
         entlen = strlen(ent->d_name);
-        if (entlen > 3 && 0==strcmp(ent->d_name+entlen-3,".mo") && regularFileExistsInDirectory(mp,"",ent->d_name)) {
+        if (((entlen > 3 && 0==strcmp(ent->d_name+entlen-3,".mo")) || (entlen > 4 && 0==strcmp(ent->d_name+entlen-4,".moc"))) && regularFileExistsInDirectory(mp,"",ent->d_name)) {
           /* fprintf(stderr, "found match %d %s\n", *numMatches, ent->d_name); */
           (*numMatches)++;
         }
@@ -1948,13 +1993,13 @@ static modelicaPathEntry* getAllModelicaPaths(const char *name, size_t nlen, voi
 #else
         maybeDir = 1;
 #endif
-        if (maybeDir && regularFileExistsInDirectory(mp,ent->d_name,"package.mo")) {
+        if (maybeDir && (regularFileExistsInDirectory(mp,ent->d_name,"package.mo") || regularFileExistsInDirectory(mp,ent->d_name,"package.moc"))) {
           ok=1;
           res[i].fileIsDir=1;
           /* fprintf(stderr, "found dir match: %ld %s - ok=%d\n", i, ent->d_name, ok); */
         }
         entlen = strlen(ent->d_name);
-        if (!ok && entlen > 3 && 0==strcmp(ent->d_name+entlen-3,".mo") && regularFileExistsInDirectory(mp,"",ent->d_name)) {
+        if (!ok && ((entlen > 3 && 0==strcmp(ent->d_name+entlen-3,".mo")) || entlen > 4 && 0==strcmp(ent->d_name+entlen-4,".moc")) && regularFileExistsInDirectory(mp,"",ent->d_name)) {
           /* fprintf(stderr, "found match file: %ld %s - ok=%d\n", i, ent->d_name, ok); */
           res[i].fileIsDir=0;
           ok=1;
@@ -2229,7 +2274,7 @@ extern int SystemImpl__reopenStandardStream(int id,const char *filename)
   return 1;
 }
 
-char* SystemImpl__iconv__ascii(const char * str)
+const char* SystemImpl__iconv__ascii(const char * str)
 {
   char *buf = 0;
   size_t sz;
@@ -2247,7 +2292,7 @@ static int isUtf8Encoding(const char *str)
   return strcasecmp(str, "UTF-8") || strcasecmp(str, "UTF8");
 }
 
-extern char* SystemImpl__iconv(const char * str, const char *from, const char *to, int printError)
+extern const char* SystemImpl__iconv(const char * str, const char *from, const char *to, int printError)
 {
   char *in_str,*res=NULL;
   size_t sz,out_sz,buflen;
@@ -2257,31 +2302,31 @@ extern char* SystemImpl__iconv(const char * str, const char *from, const char *t
   sz = strlen(str);
   if (isUtf8Encoding(from) && isUtf8Encoding(to))
   {
-    is_utf8(str, sz, &res, &count);
+    is_utf8((unsigned char*)str, sz, &res, &count);
     if (res==NULL) {
       /* Converting from UTF-8 to UTF-8 and the sequence is already UTF-8... */
       return str;
     }
     /* Converting from UTF-8, but is not valid UTF-8. Just quit early. */
     if (printError) {
-      char *ignore = SystemImpl__iconv__ascii(str);
+      const char *ignore = SystemImpl__iconv__ascii(str);
       const char *tokens[4] = {res,from,to,ignore};
       c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(\"%s\",to=\"%s\",from=\"%s\") failed: %s"),tokens,4);
-      omc_alloc_interface.free_uncollectable(ignore);
+      omc_alloc_interface.free_uncollectable((char*)ignore);
     }
-    return (char*) "";
+    return (const char*) "";
   }
   buflen = sz*4;
   /* fprintf(stderr,"iconv(%s,to=%s,%s) of size %d, buflen %d\n",str,to,from,sz,buflen); */
   ic = iconv_open(to, from);
   if (ic == (iconv_t) -1) {
     if (printError) {
-      char *ignore = SystemImpl__iconv__ascii(str);
+      const char *ignore = SystemImpl__iconv__ascii(str);
       const char *tokens[4] = {strerror(errno),from,to,ignore};
       c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(\"%s\",to=\"%s\",from=\"%s\") failed: %s"),tokens,4);
-      omc_alloc_interface.free_uncollectable(ignore);
+      omc_alloc_interface.free_uncollectable((char*)ignore);
     }
-    return (char*) "";
+    return (const char*) "";
   }
   buf = (char*) omc_alloc_interface.malloc_atomic(buflen);
   if (0 == buf) {
@@ -2289,7 +2334,7 @@ extern char* SystemImpl__iconv(const char * str, const char *from, const char *t
       /* Make the error message small so we perhaps have a chance to recover */
       c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv() ran out of memory"),NULL,0);
     }
-    return (char*) "";
+    return (const char*) "";
   }
   *buf = 0;
   in_str = (char*) str;
@@ -2299,24 +2344,24 @@ extern char* SystemImpl__iconv(const char * str, const char *from, const char *t
   iconv_close(ic);
   if (count == -1) {
     if (printError) {
-      char *ignore = SystemImpl__iconv__ascii(str);
+      const char *ignore = SystemImpl__iconv__ascii(str);
       const char *tokens[4] = {strerror(errno),from,to,ignore};
       c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(\"%s\",to=\"%s\",from=\"%s\") failed: %s"),tokens,4);
-      omc_alloc_interface.free_uncollectable(ignore);
+      omc_alloc_interface.free_uncollectable((char*)ignore);
     }
     omc_alloc_interface.free_uncollectable(buf);
-    return (char*) "";
+    return (const char*) "";
   }
   buf[(buflen-1)-out_sz] = 0;
   if (strlen(buf) != (buflen-1)-out_sz) {
     if (printError) c_add_message(NULL,-1,ErrorType_scripting,ErrorLevel_error,gettext("iconv(to=%s) failed because the character set output null bytes in the middle of the string."),&to,1);
     omc_alloc_interface.free_uncollectable(buf);
-    return (char*) "";
+    return (const char*) "";
   }
   if (!strcmp(from, to) && !strcmp(str, buf))
   {
     omc_alloc_interface.free_uncollectable(buf);
-    return (char*)str;
+    return (const char*)str;
   }
   return buf;
 }
@@ -2452,184 +2497,6 @@ const char* SystemImpl__gettext(const char *msgid)
 }
 
 
-#if defined(__MINGW32__) || defined(_MSC_VER)
-
-#ifdef __MINGW32__
-
-char *realpath(const char *path, char resolved_path[PATH_MAX])
-{
-  if (!_fullpath(resolved_path, path, PATH_MAX))
-  {
-    const char *c_tokens[0]={};
-    const char* fmt = "System.realpath failed on %s with errno: %d";
-    char* msg = (char*)omc_alloc_interface.malloc_atomic(strlen(path) + strlen(fmt) + 10);
-    sprintf(msg, fmt, path, errno);
-    c_add_message(NULL,6000,
-      ErrorType_scripting,
-      ErrorLevel_warning,
-      msg,
-      c_tokens,
-      0);
-    resolved_path = (char*)path;
-  }
-  return resolved_path;
-}
-
-#else
-
-/*
-realpath() Win32 implementation, supports non standard glibc extension
-This file has no copyright assigned and is placed in the Public Domain.
-Written by Nach M. S. September 8, 2005
-*/
-
-#include <windows.h>
-#include <stdlib.h>
-#include <limits.h>
-#include <errno.h>
-#include <sys/stat.h>
-
-char *realpath(const char *path, char resolved_path[PATH_MAX])
-{
-  char *return_path = 0;
-
-  if (path) //Else EINVAL
-  {
-    if (resolved_path)
-    {
-      return_path = resolved_path;
-    }
-    else
-    {
-      //Non standard extension that glibc uses
-      return_path = (char*)malloc(PATH_MAX);
-    }
-
-    if (return_path) //Else EINVAL
-    {
-      //This is a Win32 API function similar to what realpath() is supposed to do
-      size_t size = GetFullPathNameA(path, PATH_MAX, return_path, 0);
-
-      //GetFullPathNameA() returns a size larger than buffer if buffer is too small
-      if (size > PATH_MAX)
-      {
-        if (return_path != resolved_path) //Malloc'd buffer - Unstandard extension retry
-        {
-          size_t new_size;
-
-          free(return_path);
-          return_path = (char*)malloc(size);
-
-          if (return_path)
-          {
-            new_size = GetFullPathNameA(path, size, return_path, 0); //Try again
-
-            if (new_size > size) //If it's still too large, we have a problem, don't try again
-            {
-              free(return_path);
-              return_path = 0;
-              errno = ENAMETOOLONG;
-            }
-            else
-            {
-              size = new_size;
-            }
-          }
-          else
-          {
-            //I wasn't sure what to return here, but the standard does say to return EINVAL
-            //if resolved_path is null, and in this case we couldn't malloc large enough buffer
-            errno = EINVAL;
-          }
-        }
-        else //resolved_path buffer isn't big enough
-        {
-          return_path = 0;
-          errno = ENAMETOOLONG;
-        }
-      }
-
-      //GetFullPathNameA() returns 0 if some path resolve problem occured
-      if (!size)
-      {
-        if (return_path != resolved_path) //Malloc'd buffer
-        {
-          free(return_path);
-        }
-
-        return_path = 0;
-
-        //Convert MS errors into standard errors
-        switch (GetLastError())
-        {
-          case ERROR_FILE_NOT_FOUND:
-            errno = ENOENT;
-            break;
-
-          case ERROR_PATH_NOT_FOUND: case ERROR_INVALID_DRIVE:
-            errno = ENOTDIR;
-            break;
-
-          case ERROR_ACCESS_DENIED:
-            errno = EACCES;
-            break;
-
-          default: //Unknown Error
-            errno = EIO;
-            break;
-        }
-      }
-
-      //If we get to here with a valid return_path, we're still doing good
-      if (return_path)
-      {
-        struct stat stat_buffer;
-
-        //Make sure path exists, stat() returns 0 on success
-        if (stat(return_path, &stat_buffer))
-        {
-          if (return_path != resolved_path)
-          {
-            free(return_path);
-          }
-
-          return_path = 0;
-          //stat() will set the correct errno for us
-        }
-        //else we succeeded!
-      }
-    }
-    else
-    {
-      errno = EINVAL;
-    }
-  }
-  else
-  {
-    errno = EINVAL;
-  }
-
-  if (return_path == NULL)
-  {
-    const char* fmt = "System.realpath failed on %s with errno: %d";
-    char* msg = (char*)malloc(strlen(path) + strlen(fmt) + 10);
-    sprintf(msg, fmt, path, errno);
-    c_add_message(NULL,6000,
-      ErrorType_scripting,
-      ErrorLevel_warning,
-      msg,
-      NULL,
-      0);
-    resolved_path = (char*)path;
-    return_path = (char*)path;
-  }
-
-  return return_path;
-}
-#endif /* mingw */
-
-#endif /* mingw and msvc */
-
 int System_getTerminalWidth(void)
 {
 #if defined(__MINGW32__) || defined(_MSC_VER)
@@ -2676,22 +2543,35 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
       }
 
       switch(i) {
+
+      case FLAG_IDA_LS:
+        numExtraFlags = IDA_LS_MAX;
+        flagName = IDA_LS_METHOD;
+        flagDesc = IDA_LS_METHOD_DESC;
+        break;
+
+      case FLAG_IIM:
+        numExtraFlags = IIM_MAX;
+        flagName = INIT_METHOD_NAME;
+        flagDesc = INIT_METHOD_DESC;
+        break;
+
+      case FLAG_JACOBIAN:
+        numExtraFlags = JAC_MAX;
+        flagName = JACOBIAN_METHOD;
+        flagDesc = JACOBIAN_METHOD_DESC;
+        break;
+
       case FLAG_LS:
         numExtraFlags = LS_MAX;
         flagName = LS_NAME;
         flagDesc = LS_DESC;
         break;
 
-      case FLAG_NLS:
-        numExtraFlags = NLS_MAX;
-        flagName = NLS_NAME;
-        flagDesc = NLS_DESC;
-        break;
-
-      case FLAG_NEWTON_STRATEGY:
-        numExtraFlags = NEWTON_MAX;
-        flagName = NEWTONSTRATEGY_NAME;
-        flagDesc = NEWTONSTRATEGY_DESC;
+      case FLAG_LSS:
+        numExtraFlags = LSS_MAX;
+        flagName = LSS_NAME;
+        flagDesc = LSS_DESC;
         break;
 
       case FLAG_LV:
@@ -2701,12 +2581,25 @@ char* System_getSimulationHelpTextSphinx(int detailed, int sphinx)
         flagDesc = LOG_STREAM_DESC;
         break;
 
-      case FLAG_IIM:
-        numExtraFlags = IIM_MAX;
-        flagName = INIT_METHOD_NAME;
-        flagDesc = INIT_METHOD_DESC;
+      case FLAG_NEWTON_STRATEGY:
+        numExtraFlags = NEWTON_MAX;
+        flagName = NEWTONSTRATEGY_NAME;
+        flagDesc = NEWTONSTRATEGY_DESC;
         break;
 
+      case FLAG_NLS:
+        numExtraFlags = NLS_MAX;
+        flagName = NLS_NAME;
+        flagDesc = NLS_DESC;
+        break;
+
+      case FLAG_NLS_LS:
+        numExtraFlags = NLS_LS_MAX;
+        flagName = NLS_LS_METHOD;
+        flagDesc = NLS_LS_METHOD_DESC;
+        break;
+
+
       case FLAG_S:
         numExtraFlags = S_MAX;
         flagName = NULL;
