@@ -332,19 +332,6 @@ modelica_metatype stringDelimitList(modelica_metatype lst, metamodelica_string_c
   return p;
 }
 
-modelica_integer mmc_stringCompare(const void *str1, const void *str2)
-{
-  int res;
-  MMC_CHECK_STRING(str1);
-  MMC_CHECK_STRING(str2);
-  res = strcmp(MMC_STRINGDATA(str1),MMC_STRINGDATA(str2));
-  if (res < 0)
-    return -1;
-  if (res > 0)
-    return 1;
-  return 0;
-}
-
 modelica_metatype boxptr_stringGetStringChar(threadData_t *threadData,metamodelica_string str, modelica_metatype iix)
 {
   modelica_metatype res;
@@ -394,13 +381,15 @@ modelica_metatype boxptr_stringUpdateStringChar(threadData_t *threadData,metamod
 modelica_metatype listReverse(modelica_metatype lst)
 {
   modelica_metatype res = NULL;
-
+  if (MMC_NILTEST(lst) || MMC_NILTEST(MMC_CDR(lst))) {
+    // 0/1 elements are already reversed
+    return lst;
+  }
   res = mmc_mk_nil();
-  while (!MMC_NILTEST(lst))
-  {
+  do {
     res = mmc_mk_cons(MMC_CAR(lst),res);
     lst = MMC_CDR(lst);
-  }
+  } while (!MMC_NILTEST(lst));
   return res;
 }
 
@@ -425,29 +414,37 @@ void boxptr_listSetRest(threadData_t *threadData, modelica_metatype cellToDestro
   MMC_CDR(cellToDestroy) = newRest;
 }
 
-modelica_metatype listAppend(modelica_metatype lst1,modelica_metatype lst2)
+void boxptr_listSetFirst(threadData_t *threadData, modelica_metatype cellToDestroy, modelica_metatype newContent)
+{
+  if (MMC_NILTEST(cellToDestroy)) {
+    MMC_THROW_INTERNAL();
+  }
+  MMC_CAR(cellToDestroy) = newContent;
+}
+
+modelica_metatype listAppend(modelica_metatype l1,modelica_metatype l2)
 {
   int length = 0, i = 0;
   struct mmc_cons_struct *res = NULL;
   struct mmc_cons_struct *p = NULL;
-  if (MMC_NILTEST(lst2)) /* If lst2 is empty, simply return lst1; huge performance gain for some uses of listAppend */
-    return lst1;
-  length = listLength(lst1);
-  if (length == 0) /* We need to check for empty lst1 */
-    return lst2;
+  if (MMC_NILTEST(l2)) /* If l2 is empty, simply return l1; huge performance gain for some uses of listAppend */
+    return l1;
+  length = listLength(l1);
+  if (length == 0) /* We need to check for empty l1 */
+    return l2;
   res = (struct mmc_cons_struct*)mmc_alloc_words( length * 3 /*(sizeof(struct mmc_cons_struct)/sizeof(void*))*/ ); /* Do one single big alloc. It's cheaper */
   for (i=0; i<length-1; i++) { /* Write all except the last element... */
     struct mmc_cons_struct *p = res+i;
     p->header = MMC_STRUCTHDR(2, MMC_CONS_CTOR);
-    p->data[0] = MMC_CAR(lst1);
+    p->data[0] = MMC_CAR(l1);
     p->data[1] = MMC_TAGPTR(res+i+1);
-    lst1 = MMC_CDR(lst1);
+    l1 = MMC_CDR(l1);
   }
-  /* The last element is a bit special. It points to lst2. */
+  /* The last element is a bit special. It points to l2. */
   p = res+length-1;
   p->header = MMC_STRUCTHDR(2, MMC_CONS_CTOR);
-  p->data[0] = MMC_CAR(lst1);
-  p->data[1] = lst2;
+  p->data[0] = MMC_CAR(l1);
+  p->data[1] = l2;
   return MMC_TAGPTR(res);
 }
 
@@ -507,7 +504,7 @@ modelica_metatype boxptr_listDelete(threadData_t *threadData, modelica_metatype
     MMC_THROW_INTERNAL();
   }
 
-  tmpArr = (modelica_metatype *) GC_malloc(sizeof(modelica_metatype)*(ix-1)); /* We know the size of the first part of the list */
+  tmpArr = (modelica_metatype *) mmc_alloc_words(ix-1); /* We know the size of the first part of the list */
   if (tmpArr == NULL) {
     fprintf(stderr, "%s:%d: malloc failed", __FILE__, __LINE__);
     EXIT(1);
@@ -571,7 +568,7 @@ modelica_metatype arrayList(modelica_metatype arr)
 modelica_metatype listArray(modelica_metatype lst)
 {
   int nelts = listLength(lst);
-  void* arr = (struct mmc_struct*)mmc_mk_box_no_assign(nelts, MMC_ARRAY_TAG);
+  void* arr = (struct mmc_struct*)mmc_mk_box_no_assign(nelts, MMC_ARRAY_TAG, MMC_IS_IMMEDIATE(MMC_CAR(lst)));
   void **arrp = MMC_STRUCTDATA(arr);
   int i = 0;
   for(i=0; i<nelts; i++) {
@@ -581,10 +578,15 @@ modelica_metatype listArray(modelica_metatype lst)
   return arr;
 }
 
+modelica_metatype listArrayLiteral(modelica_metatype lst)
+{
+  return listArray(lst);
+}
+
 modelica_metatype arrayCopy(modelica_metatype arr)
 {
   int nelts = MMC_HDRSLOTS(MMC_GETHDR(arr));
-  void* res = (struct mmc_struct*)mmc_mk_box_no_assign(nelts, MMC_ARRAY_TAG);
+  void* res = (struct mmc_struct*)mmc_mk_box_no_assign(nelts, MMC_ARRAY_TAG, MMC_IS_IMMEDIATE(MMC_STRUCTDATA(arr)[0]));
   void **arrp = MMC_STRUCTDATA(arr);
   void **resp = MMC_STRUCTDATA(res);
   memcpy(resp, arrp, sizeof(modelica_metatype)*nelts);
@@ -595,7 +597,7 @@ modelica_metatype arrayAppend(modelica_metatype arr1, modelica_metatype arr2)
 {
   int nelts1 = MMC_HDRSLOTS(MMC_GETHDR(arr1));
   int nelts2 = MMC_HDRSLOTS(MMC_GETHDR(arr2));
-  void* res = (struct mmc_struct*)mmc_mk_box_no_assign(nelts1 + nelts2, MMC_ARRAY_TAG);
+  void* res = (struct mmc_struct*)mmc_mk_box_no_assign(nelts1 + nelts2, MMC_ARRAY_TAG, MMC_IS_IMMEDIATE(MMC_STRUCTDATA(arr1)[0]));
   void **arr1p = MMC_STRUCTDATA(arr1);
   void **arr2p = MMC_STRUCTDATA(arr2);
   void **resp = MMC_STRUCTDATA(res);
