@@ -42,24 +42,19 @@
 #include "simulation_data.h"
 
 #include "util/rtclock.h"
+#include "simulation/omc_simulation_util.h"
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef __cplusplus
-#include <string>
-
 #include "linearization/linearize.h"
 #include "simulation/results/simulation_result.h"
 
+#ifdef __cplusplus
 extern "C" {
-
-extern int callSolver(DATA* simData, threadData_t *threadData, std::string init_initMethod,
-    std::string init_file, double init_time, int lambda_steps, std::string outputVariablesAtEnd, int cpuTime);
+#endif /* cplusplus */
 
 extern int initializeResultData(DATA* simData, threadData_t *threadData, int cpuTime);
 
-#endif /* cplusplus */
-
 extern int modelTermination;     /* Becomes non-zero when simulation terminates. */
 extern int terminationTerminate; /* Becomes non-zero when user terminates simulation. */
 extern int terminationAssert;    /* Becomes non-zero when model call assert simulation. */
@@ -77,7 +72,7 @@ extern const char* getNameString(const char** ptr);
 extern double getSimulationStepSize();
 extern void printSimulationStepSize(double in_stepSize, double time);
 
-extern void communicateStatus(const char *phase, double completionPercent);
+extern void communicateStatus(const char *phase, double completionPercent, double currentTime, double currentStepSize);
 extern void communicateMsg(char id, unsigned int size, const char *data);
 
 /* the main function of the simulation runtime!
@@ -85,8 +80,23 @@ extern void communicateMsg(char id, unsigned int size, const char *data);
  */
 extern int _main_SimulationRuntime(int argc, char**argv, DATA *data, threadData_t *threadData);
 
+#if !defined(OMC_MINIMAL_RUNTIME)
+const char* prettyPrintNanoSec(int64_t ns, int *v);
+#endif
+
+void setStreamPrintXML(int isXML);
+
 #ifdef __cplusplus
 }
 #endif
 
+/*
+ * adrpo: weird msvc _C2 defined inside stdint.h via yvals.h interferes with some generated Modelica code
+ */
+#if defined(_MSC_VER)
+#if defined(_C2)
+#undef _C2
+#endif
+#endif
+
 #endif
