@@ -37,22 +37,45 @@
 #include "util/omc_error.h"
 #include "openmodelica.h"
 #include "openmodelica_func.h"
+#include "simulation/options.h"
 #include "simulation/solver/model_help.h"
+#if !defined(OMC_MINIMAL_RUNTIME)
 #include "util/read_matlab4.h"
+#endif
 #include "simulation/solver/events.h"
 #include "simulation/solver/stateset.h"
 #include "meta/meta_modelica.h"
 
+#if defined(OMC_NUM_MIXED_SYSTEMS) && OMC_NUM_MIXED_SYSTEMS==0
+#define check_mixed_solutions(X,Y) 0
+#else
 #include "simulation/solver/mixedSystem.h"
+#endif
+
+#if defined(OMC_NUM_LINEAR_SYSTEMS) && OMC_NUM_LINEAR_SYSTEMS==0
+#define check_linear_solutions(X,Y) 0
+#define updateStaticDataOfLinearSystems(X,Y)
+#else
 #include "simulation/solver/linearSystem.h"
+#endif
+
+#if defined(OMC_NUM_NONLINEAR_SYSTEMS) && OMC_NUM_NONLINEAR_SYSTEMS==0
+#define check_nonlinear_solutions(X,Y) 0
+#define updateStaticDataOfNonlinearSystems(X,Y)
+#else
 #include "simulation/solver/nonlinearSystem.h"
+#endif
+
 #include "simulation/solver/delay.h"
+#include "simulation/solver/synchronous.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
 
+int init_lambda_steps = 4;
+
 /*! \fn void dumpInitializationStatus(DATA *data)
  *
  *  \param [in]  [data]
@@ -63,8 +86,8 @@ void dumpInitialSolution(DATA *simData)
 {
   long i, j;
 
-  const MODEL_DATA      *mData = &(simData->modelData);
-  const SIMULATION_INFO *sInfo = &(simData->simulationInfo);
+  const MODEL_DATA      *mData = simData->modelData;
+  const SIMULATION_INFO *sInfo = simData->simulationInfo;
 
   if (ACTIVE_STREAM(LOG_INIT))
     printParameters(simData, LOG_INIT);
@@ -148,115 +171,175 @@ void dumpInitialSolution(DATA *simData)
   messageClose(LOG_SOTI);
 }
 
-/*! \fn static int symbolic_initialization(DATA *data)
+/*! \fn static int symbolic_initialization(DATA *data, threadData_t *threadData)
  *
  *  \param [ref] [data]
+ *  \param [ref] [threadData]
  *
  *  \author lochel
+ *  \author ptaeuber
  */
-static int symbolic_initialization(DATA *data, threadData_t *threadData, long numLambdaSteps)
+static int symbolic_initialization(DATA *data, threadData_t *threadData)
 {
   TRACE_PUSH
-  long step;
   int retVal;
+  FILE *pFile = NULL;
+  long i;
+  MODEL_DATA *mData = data->modelData;
+  int homotopySupport = 0;
+  int solveWithGlobalHomotopy;
+  int adaptiveGlobal;
+  int kinsol = 0;
 
+#if !defined(OMC_MINIMAL_RUNTIME)
+  kinsol = (data->simulationInfo->nlsMethod == NLS_KINSOL);
+#endif
+
+#if !defined(OMC_NUM_NONLINEAR_SYSTEMS) || OMC_NUM_NONLINEAR_SYSTEMS>0
+  for(i=0; i<mData->nNonLinearSystems; i++) {
+    if (data->simulationInfo->nonlinearSystemData[i].homotopySupport) {
+      homotopySupport = 1;
+      break;
+    }
+  }
+#endif
+  adaptiveGlobal = data->callback->useHomotopy == 2;
+  solveWithGlobalHomotopy = homotopySupport
+                            && ((data->callback->useHomotopy == 1 && init_lambda_steps > 1) || adaptiveGlobal);
+
+#if !defined(OMC_NDELAY_EXPRESSIONS) || OMC_NDELAY_EXPRESSIONS>0
   /* initial sample and delay before initial the system */
-  initDelay(data, data->simulationInfo.startTime);
-
+  initDelay(data, data->simulationInfo->startTime);
+#endif
   /* initialize all relations that are ZeroCrossings */
   storePreValues(data);
   overwriteOldSimulationData(data);
 
-  if (data->callback->useHomotopy && numLambdaSteps > 1)
+  /* If there is no homotopy in the model or local homotopy is activated
+     or homotopy is disabled by runtime flag '-ils=<lambda_steps>',
+     solve WITHOUT HOMOTOPY or LOCAL HOMOTOPY. */
+  if (!solveWithGlobalHomotopy){
+    data->simulationInfo->lambda = 1.0;
+    data->callback->functionInitialEquations(data, threadData);
+
+  /* If there is homotopy in the model and global homotopy is activated
+     and homotopy on first try is deactivated,
+     TRY TO SOLVE WITHOUT HOMOTOPY FIRST.
+     TO-DO: For the adaptive global approach, provide a separate DAE with
+     the original unmanipulated systems for trying without homotopy */
+  } else if (!omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY]) {
+    /* try */
+#ifndef OMC_EMCC
+  MMC_TRY_INTERNAL(simulationJumpBuffer)
+#endif
+    if (adaptiveGlobal && kinsol) {
+      infoStreamPrint(LOG_INIT, 0, "Automatically set -homotopyOnFirstTry, because trying without homotopy first is not supported for the adaptive global approach in combination with KINSOL.");
+    } else {
+      if (adaptiveGlobal)
+        data->callback->useHomotopy = 1;
+      data->simulationInfo->lambda = 1.0;
+      infoStreamPrint(LOG_INIT, 0, "Try to solve the initialization problem without homotopy first.");
+      data->callback->functionInitialEquations(data, threadData);
+      solveWithGlobalHomotopy = 0;
+    }
+
+    /* catch */
+#ifndef OMC_EMCC
+  MMC_CATCH_INTERNAL(simulationJumpBuffer)
+#endif
+    if (adaptiveGlobal)
+      data->callback->useHomotopy = 2;
+    if(solveWithGlobalHomotopy) {
+      if (!kinsol)
+        warningStreamPrint(LOG_ASSERT, 0, "Failed to solve the initialization problem without homotopy method. If homotopy is available the homotopy method is used now.");
+      omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY] = 1;
+      setAllParamsToStart(data);
+      setAllVarsToStart(data);
+      data->callback->updateBoundParameters(data, threadData);
+      data->callback->updateBoundVariableAttributes(data, threadData);
+    }
+  }
+
+  /* If there is homotopy in the model and the equidistant global homotopy approach is activated
+     and solving without homotopy failed or is not wanted,
+     use EQUIDISTANT GLOBAL HOMOTOPY METHOD. */
+  if (data->callback->useHomotopy == 1 && solveWithGlobalHomotopy)
   {
-    long i;
+    long step;
     char buffer[4096];
-    FILE *pFile = NULL;
 
-    modelica_real* realVars = (modelica_real*)calloc(data->modelData.nVariablesReal, sizeof(modelica_real));
-    modelica_integer* integerVars = (modelica_integer*)calloc(data->modelData.nVariablesInteger, sizeof(modelica_integer));
-    modelica_boolean* booleanVars = (modelica_boolean*)calloc(data->modelData.nVariablesBoolean, sizeof(modelica_boolean));
-    modelica_string* stringVars = (modelica_string*) GC_malloc_uncollectable(data->modelData.nVariablesString * sizeof(modelica_string));
-    MODEL_DATA *mData = &(data->modelData);
-
-    assertStreamPrint(threadData, 0 != realVars, "out of memory");
-    assertStreamPrint(threadData, 0 != integerVars, "out of memory");
-    assertStreamPrint(threadData, 0 != booleanVars, "out of memory");
-    assertStreamPrint(threadData, 0 != stringVars, "out of memory");
-
-    for(i=0; i<mData->nVariablesReal; ++i) {
-      realVars[i] = mData->realVarsData[i].attribute.start;
-    }
-    for(i=0; i<mData->nVariablesInteger; ++i) {
-      integerVars[i] = mData->integerVarsData[i].attribute.start;
-    }
-    for(i=0; i<mData->nVariablesBoolean; ++i) {
-      booleanVars[i] = mData->booleanVarsData[i].attribute.start;
-    }
-    for(i=0; i<mData->nVariablesString; ++i) {
-      stringVars[i] = mData->stringVarsData[i].attribute.start;
-    }
+    infoStreamPrint(LOG_INIT, 0, "Global homotopy with equidistant step size started.");
 
+#if !defined(OMC_NO_FILESYSTEM)
+    const char sep[] = ",";
     if(ACTIVE_STREAM(LOG_INIT))
     {
-      sprintf(buffer, "%s_homotopy.csv", mData->modelFilePrefix);
+      sprintf(buffer, "%s_equidistant_global_homotopy.csv", mData->modelFilePrefix);
+      infoStreamPrint(LOG_INIT, 0, "The homotopy path will be exported to %s.", buffer);
       pFile = fopen(buffer, "wt");
-      fprintf(pFile, "%s,", "lambda");
+      fprintf(pFile, "\"sep=%s\"\n%s", sep, "\"lambda\"");
       for(i=0; i<mData->nVariablesReal; ++i)
-        fprintf(pFile, "%s,", mData->realVarsData[i].info.name);
+        fprintf(pFile, "%s\"%s\"", sep, mData->realVarsData[i].info.name);
       fprintf(pFile, "\n");
     }
+#endif
 
-    infoStreamPrint(LOG_INIT, 1, "homotopy process");
-    for(step=0; step<numLambdaSteps; ++step)
+    infoStreamPrint(LOG_INIT, 1, "homotopy process\n---------------------------");
+    for(step=0; step<init_lambda_steps; ++step)
     {
-      data->simulationInfo.lambda = ((double)step)/(numLambdaSteps-1);
+      data->simulationInfo->lambda = ((double)step)/(init_lambda_steps-1);
+      infoStreamPrint(LOG_INIT, 0, "homotopy parameter lambda = %g", data->simulationInfo->lambda);
 
-      if(data->simulationInfo.lambda > 1.0) {
-        data->simulationInfo.lambda = 1.0;
+      if(data->simulationInfo->lambda > 1.0) {
+        data->simulationInfo->lambda = 1.0;
       }
 
-      data->callback->functionInitialEquations(data, threadData);
+      if(0 == step)
+        data->callback->functionInitialEquations_lambda0(data, threadData);
+      else
+        data->callback->functionInitialEquations(data, threadData);
 
-      infoStreamPrint(LOG_INIT, 0, "lambda = %g done", data->simulationInfo.lambda);
+      infoStreamPrint(LOG_INIT, 0, "homotopy parameter lambda = %g done\n---------------------------", data->simulationInfo->lambda);
 
+#if !defined(OMC_NO_FILESYSTEM)
       if(ACTIVE_STREAM(LOG_INIT))
       {
-        fprintf(pFile, "%.16g,", data->simulationInfo.lambda);
+        fprintf(pFile, "%.16g", data->simulationInfo->lambda);
         for(i=0; i<mData->nVariablesReal; ++i)
-          fprintf(pFile, "%.16g,", data->localData[0]->realVars[i]);
+          fprintf(pFile, "%s%.16g", sep, data->localData[0]->realVars[i]);
         fprintf(pFile, "\n");
       }
-
-      if(check_nonlinear_solutions(data, 0) ||
-         check_linear_solutions(data, 0) ||
-         check_mixed_solutions(data, 0))
-        break;
-
-      setAllStartToVars(data);
+#endif
     }
+    data->simulationInfo->homotopySteps += init_lambda_steps;
     messageClose(LOG_INIT);
 
+#if !defined(OMC_NO_FILESYSTEM)
     if(ACTIVE_STREAM(LOG_INIT))
       fclose(pFile);
-
-    for(i=0; i<mData->nVariablesReal; ++i)
-      mData->realVarsData[i].attribute.start = realVars[i];
-    for(i=0; i<mData->nVariablesInteger; ++i)
-      mData->integerVarsData[i].attribute.start = integerVars[i];
-    for(i=0; i<mData->nVariablesBoolean; ++i)
-      mData->booleanVarsData[i].attribute.start = booleanVars[i];
-    for(i=0; i<mData->nVariablesString; ++i)
-      mData->stringVarsData[i].attribute.start = stringVars[i];
-
-    free(realVars);
-    free(integerVars);
-    free(booleanVars);
-    GC_free(stringVars);
-  } else {
-    data->simulationInfo.lambda = 1.0;
-    data->callback->functionInitialEquations(data, threadData);
+#endif
   }
+
+  /* If there is homotopy in the model and the adaptive global homotopy approach is activated
+     and solving without homotopy failed or is not wanted,
+     use ADAPTIVE GLOBAL HOMOTOPY APPROACH. */
+  if (adaptiveGlobal && solveWithGlobalHomotopy)
+  {
+    infoStreamPrint(LOG_INIT, 0, "Global homotopy with adaptive step size started.");
+    infoStreamPrint(LOG_INIT, 1, "homotopy process\n---------------------------");
+
+    // Solve lambda0-DAE
+    data->simulationInfo->lambda = 0;
+    infoStreamPrint(LOG_INIT, 0, "solve simplified lambda0-DAE");
+    data->callback->functionInitialEquations_lambda0(data, threadData);
+    infoStreamPrint(LOG_INIT, 0, "solving simplified lambda0-DAE done\n---------------------------");
+
+    // Run along the homotopy path and solve the actual system
+    data->callback->functionInitialEquations(data, threadData);
+
+    messageClose(LOG_INIT);
+  }
+
   storeRelations(data);
 
   /* check for over-determined systems */
@@ -329,6 +412,7 @@ static char *mapToDymolaVars(const char *varname)
   return newVarname;
 }
 
+#if !defined(OMC_MINIMAL_RUNTIME)
 /*! \fn int importStartValues(DATA *data, const char *pInitFile, const double initTime)
  *
  *  \param [ref] [data]
@@ -345,93 +429,75 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
   const char *pError = NULL;
   char* newVarname = NULL;
 
-  MODEL_DATA *mData = &(data->modelData);
+  MODEL_DATA *mData = data->modelData;
   long i;
 
   infoStreamPrint(LOG_INIT, 0, "import start values\nfile: %s\ntime: %g", pInitFile, initTime);
 
-  if(!strcmp(data->modelData.resultFileName, pInitFile))
+  if(!strcmp(data->modelData->resultFileName, pInitFile))
   {
     errorStreamPrint(LOG_INIT, 0, "Cannot import a result file for initialization that is also the current output file <%s>.\nConsider redirecting the output result file (-r=<new_res.mat>) or renaming the result file that is used for initialization import.", pInitFile);
     return 1;
   }
 
   pError = omc_new_matlab4_reader(pInitFile, &reader);
-  if(pError)
-  {
+  if(pError) {
     throwStreamPrint(threadData, "unable to read input-file <%s> [%s]", pInitFile, pError);
     return 1;
-  }
-  else
-  {
+  } else {
     infoStreamPrint(LOG_INIT, 0, "import real variables");
-    for(i=0; i<mData->nVariablesReal; ++i)
-    {
+    for(i=0; i<mData->nVariablesReal; ++i) {
       pVar = omc_matlab4_find_var(&reader, mData->realVarsData[i].info.name);
 
-      if(!pVar)
-      {
+      if(!pVar) {
         newVarname = mapToDymolaVars(mData->realVarsData[i].info.name);
         pVar = omc_matlab4_find_var(&reader, newVarname);
         free(newVarname);
       }
-
-      if(pVar)
-      {
+      if(pVar) {
         omc_matlab4_val(&(mData->realVarsData[i].attribute.start), &reader, pVar, initTime);
         infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realVarsData[i].info.name, mData->realVarsData[i].attribute.start);
-      }
-      else if((strlen(mData->realVarsData[i].info.name) > 0) &&
+      } else if((strlen(mData->realVarsData[i].info.name) > 0) &&
               (mData->realVarsData[i].info.name[0] != '$') &&
-              (strncmp(mData->realVarsData[i].info.name, "der($", 5) != 0))
-      {
+              (strncmp(mData->realVarsData[i].info.name, "der($", 5) != 0)) {
         /* skip warnings about self-generated variables */
         warningStreamPrint(LOG_INIT, 0, "unable to import real variable %s from given file", mData->realVarsData[i].info.name);
       }
     }
 
     infoStreamPrint(LOG_INIT, 0, "import real parameters");
-    for(i=0; i<mData->nParametersReal; ++i)
-    {
+    for(i=0; i<mData->nParametersReal; ++i) {
       pVar = omc_matlab4_find_var(&reader, mData->realParameterData[i].info.name);
 
-      if(!pVar)
-      {
+      if(!pVar) {
         newVarname = mapToDymolaVars(mData->realParameterData[i].info.name);
         pVar = omc_matlab4_find_var(&reader, newVarname);
         free(newVarname);
       }
 
-      if(pVar)
-      {
+      if(pVar) {
         omc_matlab4_val(&(mData->realParameterData[i].attribute.start), &reader, pVar, initTime);
+        data->simulationInfo->realParameter[i] = mData->realParameterData[i].attribute.start;
         infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
-      }
-      else
-      {
+      } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import real parameter %s from given file", mData->realParameterData[i].info.name);
       }
     }
 
     infoStreamPrint(LOG_INIT, 0, "import real discrete");
-    for(i=mData->nVariablesReal-mData->nDiscreteReal; i<mData->nDiscreteReal; ++i)
-    {
+    for(i=mData->nVariablesReal-mData->nDiscreteReal; i<mData->nDiscreteReal; ++i) {
       pVar = omc_matlab4_find_var(&reader, mData->realParameterData[i].info.name);
 
-      if(!pVar)
-      {
+      if(!pVar) {
         newVarname = mapToDymolaVars(mData->realParameterData[i].info.name);
         pVar = omc_matlab4_find_var(&reader, newVarname);
         free(newVarname);
       }
 
-      if(pVar)
-      {
+      if(pVar) {
         omc_matlab4_val(&(mData->realParameterData[i].attribute.start), &reader, pVar, initTime);
         infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
-      }
-      else
-      {
+      } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import real parameter %s from given file", mData->realParameterData[i].info.name);
       }
     }
@@ -441,45 +507,38 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
     {
       pVar = omc_matlab4_find_var(&reader, mData->integerParameterData[i].info.name);
 
-      if(!pVar)
-      {
+      if (!pVar) {
         newVarname = mapToDymolaVars(mData->integerParameterData[i].info.name);
         pVar = omc_matlab4_find_var(&reader, newVarname);
         free(newVarname);
       }
 
-      if(pVar)
-      {
+      if (pVar) {
         omc_matlab4_val(&value, &reader, pVar, initTime);
         mData->integerParameterData[i].attribute.start = (modelica_integer)value;
+        data->simulationInfo->integerParameter[i] = (modelica_integer)value;
         infoStreamPrint(LOG_INIT, 0, "| %s(start=%ld)", mData->integerParameterData[i].info.name, mData->integerParameterData[i].attribute.start);
-      }
-      else
-      {
+      } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import integer parameter %s from given file", mData->integerParameterData[i].info.name);
       }
     }
 
     infoStreamPrint(LOG_INIT, 0, "import boolean parameters");
-    for(i=0; i<mData->nParametersBoolean; ++i)
-    {
+    for(i=0; i<mData->nParametersBoolean; ++i) {
       pVar = omc_matlab4_find_var(&reader, mData->booleanParameterData[i].info.name);
 
-      if(!pVar)
-      {
+      if(!pVar) {
         newVarname = mapToDymolaVars(mData->booleanParameterData[i].info.name);
         pVar = omc_matlab4_find_var(&reader, newVarname);
         free(newVarname);
       }
 
-      if(pVar)
-      {
+      if(pVar) {
         omc_matlab4_val(&value, &reader, pVar, initTime);
         mData->booleanParameterData[i].attribute.start = (modelica_boolean)value;
+        data->simulationInfo->booleanParameter[i] = (modelica_boolean)value;
         infoStreamPrint(LOG_INIT, 0, "| %s(start=%s)", mData->booleanParameterData[i].info.name, mData->booleanParameterData[i].attribute.start ? "true" : "false");
-      }
-      else
-      {
+      } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import boolean parameter %s from given file", mData->booleanParameterData[i].info.name);
       }
     }
@@ -488,6 +547,43 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
 
   return 0;
 }
+#endif
+
+/*! \fn initSample
+ *
+ *  \param [ref] [data]
+ *  \param [in]  [startTime]
+ *  \param [in]  [stopTime]
+ *
+ *  This function initializes sample-events.
+ */
+void initSample(DATA* data, threadData_t *threadData, double startTime, double stopTime)
+{
+  TRACE_PUSH
+  long i;
+
+  data->callback->function_initSample(data, threadData);              /* set-up sample */
+  data->simulationInfo->nextSampleEvent = NAN;  /* should never be reached */
+  for(i=0; i<data->modelData->nSamples; ++i) {
+    if(startTime < data->modelData->samplesInfo[i].start) {
+      data->simulationInfo->nextSampleTimes[i] = data->modelData->samplesInfo[i].start;
+    } else {
+      data->simulationInfo->nextSampleTimes[i] = data->modelData->samplesInfo[i].start + ceil((startTime-data->modelData->samplesInfo[i].start) / data->modelData->samplesInfo[i].interval) * data->modelData->samplesInfo[i].interval;
+    }
+
+    if((i == 0) || (data->simulationInfo->nextSampleTimes[i] < data->simulationInfo->nextSampleEvent)) {
+      data->simulationInfo->nextSampleEvent = data->simulationInfo->nextSampleTimes[i];
+    }
+  }
+
+  if(stopTime < data->simulationInfo->nextSampleEvent) {
+    debugStreamPrint(LOG_EVENTS, 0, "there are no sample-events");
+  } else {
+    debugStreamPrint(LOG_EVENTS, 0, "first sample-event at t = %g", data->simulationInfo->nextSampleEvent);
+  }
+
+  TRACE_POP
+}
 
 /*! \fn int initialization(DATA *data, const char* pInitMethod, const char* pOptiMethod, const char* pInitFile, double initTime)
  *
@@ -498,38 +594,38 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
  *
  *  \author lochel
  */
-int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod, const char* pInitFile, double initTime, int lambda_steps)
+int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod, const char* pInitFile, double initTime)
 {
   TRACE_PUSH
   int initMethod = IIM_SYMBOLIC; /* default method */
   int retVal = -1;
   int i;
 
+  data->simulationInfo->homotopySteps = 0;
+
   infoStreamPrint(LOG_INIT, 0, "### START INITIALIZATION ###");
 
   setAllParamsToStart(data);
 
+#if !defined(OMC_MINIMAL_RUNTIME)
   /* import start values from extern mat-file */
   if(pInitFile && strcmp(pInitFile, ""))
   {
     data->callback->updateBoundParameters(data, threadData);
     data->callback->updateBoundVariableAttributes(data, threadData);
 
-    if(importStartValues(data, threadData, pInitFile, initTime))
-    {
+    if(importStartValues(data, threadData, pInitFile, initTime)) {
       TRACE_POP
       return 1;
     }
   }
-
+#endif
   /* set up all variables with their start-values */
   setAllVarsToStart(data);
 
-  if(!(pInitFile && strcmp(pInitFile, "")))
-  {
+  if(!(pInitFile && strcmp(pInitFile, ""))) {
     data->callback->updateBoundParameters(data, threadData);
     data->callback->updateBoundVariableAttributes(data, threadData);
-    setAllVarsToStart(data);
   }
 
   /* update static data of linear/non-linear system solvers */
@@ -537,24 +633,19 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   updateStaticDataOfNonlinearSystems(data, threadData);
 
   /* if there are user-specified options, use them! */
-  if(pInitMethod && strcmp(pInitMethod, ""))
-  {
+  if (pInitMethod && strcmp(pInitMethod, "")) {
     initMethod = IIM_UNKNOWN;
 
-    for(i=1; i<IIM_MAX; ++i)
-    {
-      if(!strcmp(pInitMethod, INIT_METHOD_NAME[i]))
-      {
+    for (i=1; i<IIM_MAX; ++i) {
+      if(!strcmp(pInitMethod, INIT_METHOD_NAME[i])) {
         initMethod = i;
       }
     }
 
-    if(initMethod == IIM_UNKNOWN)
-    {
+    if(initMethod == IIM_UNKNOWN) {
       warningStreamPrint(LOG_STDOUT, 0, "unrecognized option -iim %s", pInitMethod);
       warningStreamPrint(LOG_STDOUT, 0, "current options are:");
-      for(i=1; i<IIM_MAX; ++i)
-      {
+      for (i=1; i<IIM_MAX; ++i) {
         warningStreamPrint(LOG_STDOUT, 0, "| %-15s [%s]", INIT_METHOD_NAME[i], INIT_METHOD_DESC[i]);
       }
       throwStreamPrint(threadData, "see last warning");
@@ -564,85 +655,84 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
   infoStreamPrint(LOG_INIT, 0, "initialization method: %-15s [%s]", INIT_METHOD_NAME[initMethod], INIT_METHOD_DESC[initMethod]);
 
   /* start with the real initialization */
-  data->simulationInfo.initial = 1;             /* to evaluate when-equations with initial()-conditions */
+  data->simulationInfo->initial = 1;             /* to evaluate when-equations with initial()-conditions */
 
   /* initialize all (nonlinear|linear|mixed) systems
    * This is a workaround and should be removed as soon as possible.
    */
-  for(i=0; i<data->modelData.nNonLinearSystems; ++i)
-  {
-    data->simulationInfo.nonlinearSystemData[i].solved = 1;
+#if !defined(OMC_NUM_NONLINEAR_SYSTEMS) || OMC_NUM_NONLINEAR_SYSTEMS>0
+  for(i=0; i<data->modelData->nNonLinearSystems; ++i) {
+    data->simulationInfo->nonlinearSystemData[i].solved = 1;
   }
-  for(i=0; i<data->modelData.nLinearSystems; ++i)
-  {
-    data->simulationInfo.linearSystemData[i].solved = 1;
+#endif
+#if !defined(OMC_NUM_LINEAR_SYSTEMS) || OMC_NUM_LINEAR_SYSTEMS>0
+  for(i=0; i<data->modelData->nLinearSystems; ++i) {
+    data->simulationInfo->linearSystemData[i].solved = 1;
   }
-  for(i=0; i<data->modelData.nMixedSystems; ++i)
-  {
-    data->simulationInfo.mixedSystemData[i].solved = 1;
+#endif
+#if !defined(OMC_NUM_MIXED_SYSTEMS) || OMC_NUM_MIXED_SYSTEMS>0
+  for(i=0; i<data->modelData->nMixedSystems; ++i) {
+    data->simulationInfo->mixedSystemData[i].solved = 1;
   }
+#endif
   /* end workaround */
 
   /* select the right initialization-method */
-  if(IIM_NONE == initMethod)
-  {
+  if(IIM_NONE == initMethod) {
     retVal = 0;
-  }
-  else if(IIM_SYMBOLIC == initMethod)
-  {
-    retVal = symbolic_initialization(data, threadData, lambda_steps);
-  }
-  else
-  {
+  } else if(IIM_SYMBOLIC == initMethod) {
+    retVal = symbolic_initialization(data, threadData);
+  } else {
     throwStreamPrint(threadData, "unsupported option -iim");
   }
 
-  /* do pivoting for dynamic state selection if selection changed try again */
-  if(stateSelection(data, threadData, 0, 1) == 1)
-  {
-    if(stateSelection(data, threadData, 1, 1) == 1)
-    {
-      /* report a warning about strange start values */
-      warningStreamPrint(LOG_STDOUT, 0, "Cannot initialize the dynamic state selection in an unique way. Use -lv LOG_DSS to see the switching state set.");
-    }
-  }
-
   /* check for unsolved (nonlinear|linear|mixed) systems
    * This is a workaround and should be removed as soon as possible.
    */
-  if(check_nonlinear_solutions(data, 1))
-  {
+  if(check_nonlinear_solutions(data, 1)) {
     retVal = -2;
-  }
-  else if(check_linear_solutions(data, 1))
-  {
+  } else if(check_linear_solutions(data, 1)) {
     retVal = -3;
-  }
-  else if(check_mixed_solutions(data, 1))
-  {
+  } else if(check_mixed_solutions(data, 1)) {
     retVal = -4;
   }
   /* end workaround */
 
+#if !defined(OMC_MINIMAL_LOGGING)
   dumpInitialSolution(data);
   infoStreamPrint(LOG_INIT, 0, "### END INITIALIZATION ###");
+#endif
 
-  data->simulationInfo.initial = 0;
-  /* initialization is done */
-
-  overwriteOldSimulationData(data);     /* overwrite the whole ring-buffer with initialized values */
-  storePreValues(data);                 /* save pre-values */
-  updateDiscreteSystem(data, threadData);           /* evaluate discrete variables (event iteration) */
+  overwriteOldSimulationData(data);       /* overwrite the whole ring-buffer with initialized values */
+  storePreValues(data);                   /* save pre-values */
+  updateDiscreteSystem(data, threadData); /* evaluate discrete variables (event iteration) */
   saveZeroCrossings(data, threadData);
 
-  initSample(data, threadData, data->simulationInfo.startTime, data->simulationInfo.stopTime);
+  /* do pivoting for dynamic state selection if selection changed try again */
+#if !defined(OMC_NO_STATESELECTION)
+  if(stateSelection(data, threadData, 0, 1) == 1) {
+    if(stateSelection(data, threadData, 1, 1) == 1) {
+      /* report a warning about strange start values */
+      warningStreamPrint(LOG_STDOUT, 0, "Cannot initialize the dynamic state selection in an unique way. Use -lv LOG_DSS to see the switching state set.");
+    }
+  }
+#endif
+
+  data->simulationInfo->initial = 0;
+  /* initialization is done */
+
+  initSample(data, threadData, data->simulationInfo->startTime, data->simulationInfo->stopTime);
   data->callback->function_storeDelayed(data, threadData);
   data->callback->function_updateRelations(data, threadData, 1);
+  initSynchronous(data, threadData, data->simulationInfo->startTime);
 
-  initSynchronous(data, threadData, data->simulationInfo.startTime);
-
+#if !defined(OMC_MINIMAL_LOGGING)
   printRelations(data, LOG_EVENTS);
   printZeroCrossings(data, LOG_EVENTS);
+#endif
+
+  /* Check for warning of variables out of range assert(min<x || x>xmax, ...)*/
+  data->callback->checkForAsserts(data, threadData);
 
   /* valid system for the first time! */
   TRACE_POP
