@@ -49,14 +49,16 @@
 extern int dgesv_(int *n, int *nrhs, double *a, int *lda,
                   int *ipiv, double *b, int *ldb, int *info);
 
+extern int dgetrs_(char* tran, int *n, int *nrhs, double *a, int *lda,
+                  int *ipiv, double *b, int *ldb, int *info);
 /*! \fn allocate memory for linear system solver lapack
  *
  */
 int allocateLapackData(int size, void** voiddata)
 {
-  DATA_LAPACK* data = (DATA_LAPACK*) malloc(sizeof(DATA_LAPACK));
+  DATA_LAPACK* data = (DATA_LAPACK*) calloc(1, sizeof(DATA_LAPACK));
 
-  data->ipiv = (int*) malloc(size*sizeof(int));
+  data->ipiv = (int*) calloc(size, sizeof(int));
   assertStreamPrint(NULL, 0 != data->ipiv, "Could not allocate data for linear solver lapack.");
   data->nrhs = 1;
   data->info = 0;
@@ -84,6 +86,9 @@ int freeLapackData(void **voiddata)
   _omc_destroyVector(data->b);
   _omc_destroyMatrix(data->A);
 
+  free(data);
+  voiddata[0] = 0;
+
   return 0;
 }
 
@@ -103,34 +108,35 @@ int getAnalyticalJacobianLapack(DATA* data, threadData_t *threadData, double* ja
   LINEAR_SYSTEM_DATA* systemData = &(((DATA*)data)->simulationInfo->linearSystemData[currentSys]);
 
   const int index = systemData->jacobianIndex;
+  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[systemData->jacobianIndex]);
 
   memset(jac, 0, (systemData->size)*(systemData->size)*sizeof(double));
 
-  for(i=0; i < data->simulationInfo->analyticJacobians[index].sparsePattern.maxColors; i++)
+  for(i=0; i < jacobian->sparsePattern.maxColors; i++)
   {
     /* activate seed variable for the corresponding color */
-    for(ii=0; ii < data->simulationInfo->analyticJacobians[index].sizeCols; ii++)
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.colorCols[ii]-1 == i)
-        data->simulationInfo->analyticJacobians[index].seedVars[ii] = 1;
+    for(ii=0; ii < jacobian->sizeCols; ii++)
+      if(jacobian->sparsePattern.colorCols[ii]-1 == i)
+        jacobian->seedVars[ii] = 1;
 
-    ((systemData->analyticalJacobianColumn))(data, threadData);
+    ((systemData->analyticalJacobianColumn))(data, threadData, jacobian, systemData->parentJacobian);
 
-    for(j = 0; j < data->simulationInfo->analyticJacobians[index].sizeCols; j++)
+    for(j = 0; j < jacobian->sizeCols; j++)
     {
-      if(data->simulationInfo->analyticJacobians[index].seedVars[j] == 1)
+      if(jacobian->seedVars[j] == 1)
       {
-        ii = data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j];
-        while(ii < data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j+1])
+        ii = jacobian->sparsePattern.leadindex[j];
+        while(ii < jacobian->sparsePattern.leadindex[j+1])
         {
-          l  = data->simulationInfo->analyticJacobians[index].sparsePattern.index[ii];
-          k  = j*data->simulationInfo->analyticJacobians[index].sizeRows + l;
-          jac[k] = -data->simulationInfo->analyticJacobians[index].resultVars[l];
+          l  = jacobian->sparsePattern.index[ii];
+          k  = j*jacobian->sizeRows + l;
+          jac[k] = -jacobian->resultVars[l];
           ii++;
         };
       }
       /* de-activate seed variable for the corresponding color */
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.colorCols[j]-1 == i)
-        data->simulationInfo->analyticJacobians[index].seedVars[j] = 0;
+      if(jacobian->sparsePattern.colorCols[j]-1 == i)
+        jacobian->seedVars[j] = 0;
     }
   }
 
@@ -155,12 +161,12 @@ static int wrapper_fvec_lapack(_omc_vector* x, _omc_vector* f, int* iflag, void*
  *
  *  \author wbraun
  */
-int solveLapack(DATA *data, threadData_t *threadData, int sysNumber)
+int solveLapack(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 {
   void *dataAndThreadData[2] = {data, threadData};
   int i, iflag = 1;
   LINEAR_SYSTEM_DATA* systemData = &(data->simulationInfo->linearSystemData[sysNumber]);
-  DATA_LAPACK* solverData = (DATA_LAPACK*)systemData->solverData;
+  DATA_LAPACK* solverData = (DATA_LAPACK*)systemData->solverData[0];
 
   int success = 1;
 
@@ -170,6 +176,7 @@ int solveLapack(DATA *data, threadData_t *threadData, int sysNumber)
   int indexes[2] = {1,eqSystemNumber};
   _omc_scalar residualNorm = 0;
   double tmpJacEvalTime;
+  int reuseMatrixJac = (data->simulationInfo->currentContext == CONTEXT_SYM_JACOBIAN && data->simulationInfo->currentJacobianEval > 0);
 
   infoStreamPrintWithEquationIndexes(LOG_LS, 0, indexes, "Start solving Linear System %d (size %d) at time %g with Lapack Solver",
          eqSystemNumber, (int) systemData->size,
@@ -177,29 +184,32 @@ int solveLapack(DATA *data, threadData_t *threadData, int sysNumber)
 
 
   /* set data */
-  _omc_setVectorData(solverData->x, systemData->x);
+  _omc_setVectorData(solverData->x, aux_x);
   _omc_setVectorData(solverData->b, systemData->b);
   _omc_setMatrixData(solverData->A, systemData->A);
 
-
   rt_ext_tp_tick(&(solverData->timeClock));
   if (0 == systemData->method) {
 
-    /* reset matrix A */
-    memset(systemData->A, 0, (systemData->size)*(systemData->size)*sizeof(double));
+    if (!reuseMatrixJac){
+      /* reset matrix A */
+      memset(systemData->A, 0, (systemData->size)*(systemData->size)*sizeof(double));
 
-    /* update matrix A */
-    systemData->setA(data, threadData, systemData);
+      /* update matrix A */
+      systemData->setA(data, threadData, systemData);
+    }
 
     /* update vector b (rhs) */
     systemData->setb(data, threadData, systemData);
   } else {
 
-    /* calculate jacobian -> matrix A*/
-    if(systemData->jacobianIndex != -1){
-      getAnalyticalJacobianLapack(data, threadData, solverData->A->data, sysNumber);
-    } else {
-      assertStreamPrint(threadData, 1, "jacobian function pointer is invalid" );
+    if (!reuseMatrixJac){
+      /* calculate jacobian -> matrix A*/
+      if(systemData->jacobianIndex != -1){
+        getAnalyticalJacobianLapack(data, threadData, solverData->A->data, sysNumber);
+      } else {
+        assertStreamPrint(threadData, 1, "jacobian function pointer is invalid" );
+      }
     }
 
     /* calculate vector b (rhs) */
@@ -219,15 +229,35 @@ int solveLapack(DATA *data, threadData_t *threadData, int sysNumber)
 
   rt_ext_tp_tick(&(solverData->timeClock));
 
-  /* Solve system */
-  dgesv_((int*) &systemData->size,
-         (int*) &solverData->nrhs,
-         solverData->A->data,
-         (int*) &systemData->size,
-         solverData->ipiv,
-         solverData->b->data,
-         (int*) &systemData->size,
-         &solverData->info);
+  /* if reuseMatrixJac use also previous factorization */
+  if (!reuseMatrixJac)
+  {
+    /* Solve system */
+    dgesv_((int*) &systemData->size,
+           (int*) &solverData->nrhs,
+           solverData->A->data,
+           (int*) &systemData->size,
+           solverData->ipiv,
+           solverData->b->data,
+           (int*) &systemData->size,
+           &solverData->info);
+
+  } /* further Jacobian evaluations */
+  else
+  {
+    char trans = 'N';
+    /* Solve system */
+    dgetrs_(&trans,
+            (int*) &systemData->size,
+            (int*) &solverData->nrhs,
+            solverData->A->data,
+            (int*) &systemData->size,
+            solverData->ipiv,
+            solverData->b->data,
+            (int*) &systemData->size,
+            &solverData->info);
+  }
+
 
   infoStreamPrint(LOG_LS_V, 0, "Solve System: %f", rt_ext_tp_tock(&(solverData->timeClock)));
 
@@ -278,7 +308,7 @@ int solveLapack(DATA *data, threadData_t *threadData, int sysNumber)
       infoStreamPrint(LOG_LS_V, 0, "System %d numVars %d.", eqSystemNumber, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).numVar);
 
       for(i = 0; i < systemData->size; ++i) {
-        infoStreamPrint(LOG_LS_V, 0, "[%d] %s = %.15g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], systemData->x[i]);
+        infoStreamPrint(LOG_LS_V, 0, "[%d] %s = %.15g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], aux_x[i]);
       }
 
       messageClose(LOG_LS_V);
