@@ -281,7 +281,7 @@ int allocateTotalPivotData(int size, void** voiddata)
   data->indRow =(int*) calloc(size,sizeof(int));
   data->indCol =(int*) calloc(size+1,sizeof(int));
 
-  *voiddata = (void*)data;
+  voiddata[1] = (void*)data;
   return 0;
 }
 
@@ -291,7 +291,7 @@ int allocateTotalPivotData(int size, void** voiddata)
  */
 int freeTotalPivotData(void** voiddata)
 {
-  DATA_TOTALPIVOT* data = (DATA_TOTALPIVOT*) *voiddata;
+  DATA_TOTALPIVOT* data = (DATA_TOTALPIVOT*) voiddata[1];
 
   /* memory for linear system */
   free(data->Ab);
@@ -302,6 +302,9 @@ int freeTotalPivotData(void** voiddata)
   free(data->indRow);
   free(data->indCol);
 
+  free(voiddata[1]);
+  voiddata[1] = 0;
+
   return 0;
 }
 
@@ -321,37 +324,34 @@ int getAnalyticalJacobianTotalPivot(DATA* data, threadData_t *threadData, double
   LINEAR_SYSTEM_DATA* systemData = &(((DATA*)data)->simulationInfo->linearSystemData[currentSys]);
 
   const int index = systemData->jacobianIndex;
+  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[systemData->jacobianIndex]);
 
   memset(jac, 0, (systemData->size)*(systemData->size)*sizeof(double));
 
-  for(i=0; i < data->simulationInfo->analyticJacobians[index].sparsePattern.maxColors; i++)
+  for(i=0; i < jacobian->sparsePattern.maxColors; i++)
   {
     /* activate seed variable for the corresponding color */
-    for(ii=0; ii < data->simulationInfo->analyticJacobians[index].sizeCols; ii++)
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.colorCols[ii]-1 == i)
-        data->simulationInfo->analyticJacobians[index].seedVars[ii] = 1;
+    for(ii=0; ii < jacobian->sizeCols; ii++)
+      if(jacobian->sparsePattern.colorCols[ii]-1 == i)
+        jacobian->seedVars[ii] = 1;
 
-    ((systemData->analyticalJacobianColumn))(data, threadData);
+    ((systemData->analyticalJacobianColumn))(data, threadData, jacobian, systemData->parentJacobian);
 
-    for(j = 0; j < data->simulationInfo->analyticJacobians[index].sizeCols; j++)
+    for(j = 0; j < jacobian->sizeCols; j++)
     {
-      if(data->simulationInfo->analyticJacobians[index].seedVars[j] == 1)
+      if(jacobian->seedVars[j] == 1)
       {
-        if(j==0) {
-          ii = 0;
-        } else {
-          ii = data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j-1];
-        }
-        while(ii < data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j]) {
-          l  = data->simulationInfo->analyticJacobians[index].sparsePattern.index[ii];
-          k  = j*data->simulationInfo->analyticJacobians[index].sizeRows + l;
-          jac[k] = data->simulationInfo->analyticJacobians[index].resultVars[l];
+        ii = jacobian->sparsePattern.leadindex[j];
+        while(ii < jacobian->sparsePattern.leadindex[j+1]) {
+          l  = jacobian->sparsePattern.index[ii];
+          k  = j*jacobian->sizeRows + l;
+          jac[k] = jacobian->resultVars[l];
           ii++;
         }
       }
       /* de-activate seed variable for the corresponding color */
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.colorCols[j]-1 == i) {
-        data->simulationInfo->analyticJacobians[index].seedVars[j] = 0;
+      if(jacobian->sparsePattern.colorCols[j]-1 == i) {
+        jacobian->seedVars[j] = 0;
       }
     }
 
@@ -382,12 +382,12 @@ static int wrapper_fvec_totalpivot(double* x, double* f, void** data, int sysNum
  *
  *  \author bbachmann
  */
-int solveTotalPivot(DATA *data, threadData_t *threadData, int sysNumber)
+int solveTotalPivot(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 {
   void *dataAndThreadData[2] = {data, threadData};
   int i, j;
   LINEAR_SYSTEM_DATA* systemData = &(data->simulationInfo->linearSystemData[sysNumber]);
-  DATA_TOTALPIVOT* solverData = (DATA_TOTALPIVOT*)systemData->solverData;
+  DATA_TOTALPIVOT* solverData = (DATA_TOTALPIVOT*) systemData->solverData[1];
   int n = systemData->size, status;
   double fdeps = 1e-8;
   double xTol = 1e-8;
@@ -399,14 +399,14 @@ int solveTotalPivot(DATA *data, threadData_t *threadData, int sysNumber)
    * We want to look it up among all equations. */
   /* int eqSystemNumber = systemData->equationIndex; */
   int success = 1;
+  double tmpJacEvalTime;
 
   infoStreamPrintWithEquationIndexes(LOG_LS, 0, indexes, "Start solving Linear System %d (size %d) at time %g with Total Pivot Solver",
          eqSystemNumber, (int) systemData->size,
          data->localData[0]->timeValue);
 
-
   debugVectorDoubleLS(LOG_LS_V,"SCALING",systemData->nominal,n);
-  debugVectorDoubleLS(LOG_LS_V,"Old VALUES",systemData->x,n);
+  debugVectorDoubleLS(LOG_LS_V,"Old VALUES",aux_x,n);
 
   rt_ext_tp_tick(&(solverData->timeClock));
   if (0 == systemData->method){
@@ -431,14 +431,16 @@ int solveTotalPivot(DATA *data, threadData_t *threadData, int sysNumber)
       assertStreamPrint(threadData, 1, "jacobian function pointer is invalid" );
     }
     /* calculate vector b (rhs) -> -b is last column of matrix Ab */
-    wrapper_fvec_totalpivot(systemData->x, solverData->Ab + n*n, dataAndThreadData, sysNumber);
+    wrapper_fvec_totalpivot(aux_x, solverData->Ab + n*n, dataAndThreadData, sysNumber);
   }
-  infoStreamPrint(LOG_LS, 0, "###  %f  time to set Matrix A and vector b.", rt_ext_tp_tock(&(solverData->timeClock)));
+  tmpJacEvalTime = rt_ext_tp_tock(&(solverData->timeClock));
+  systemData->jacobianTime += tmpJacEvalTime;
+  infoStreamPrint(LOG_LS_V, 0, "###  %f  time to set Matrix A and vector b.", tmpJacEvalTime);
   debugMatrixDoubleLS(LOG_LS_V,"LGS: matrix Ab",solverData->Ab, n, n+1);
 
   rt_ext_tp_tick(&(solverData->timeClock));
   status = solveSystemWithTotalPivotSearchLS(n, solverData->x, solverData->Ab, solverData->indRow, solverData->indCol, &rank);
-  infoStreamPrint(LOG_LS, 0, "Solve System: %f", rt_ext_tp_tock(&(solverData->timeClock)));
+  infoStreamPrint(LOG_LS_V, 0, "Solve System: %f", rt_ext_tp_tock(&(solverData->timeClock)));
 
   if (status != 0)
   {
@@ -449,11 +451,11 @@ int solveTotalPivot(DATA *data, threadData_t *threadData, int sysNumber)
     debugVectorDoubleLS(LOG_LS_V, "SOLUTION:", solverData->x, n+1);
     if (1 == systemData->method){
       /* add the solution to old solution vector*/
-      vecAddLS(n, systemData->x, solverData->x, systemData->x);
-      wrapper_fvec_totalpivot(systemData->x, solverData->b, dataAndThreadData, sysNumber);
+      vecAddLS(n, aux_x, solverData->x, aux_x);
+      wrapper_fvec_totalpivot(aux_x, solverData->b, dataAndThreadData, sysNumber);
     } else {
        /* take the solution */
-       vecCopyLS(n, solverData->x, systemData->x);
+       vecCopyLS(n, solverData->x, aux_x);
     }
 
     if (ACTIVE_STREAM(LOG_LS_V)){
@@ -461,7 +463,7 @@ int solveTotalPivot(DATA *data, threadData_t *threadData, int sysNumber)
       infoStreamPrint(LOG_LS_V, 0, "System %d numVars %d.", eqSystemNumber, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).numVar);
       for(i=0; i<systemData->size; ++i)
       {
-        infoStreamPrint(LOG_LS_V, 0, "[%d] %s = %g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], systemData->x[i]);
+        infoStreamPrint(LOG_LS_V, 0, "[%d] %s = %g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], aux_x[i]);
       }
       messageClose(LOG_LS_V);
     }
