@@ -43,7 +43,7 @@ extern "C" {
 #include "util/omc_error.h"
 #include "util/varinfo.h"
 #include "model_help.h"
-#include "util/memory_pool.h"
+#include "gc/omc_gc.h"
 #include "meta/meta_modelica.h"
 
 #include "nonlinearSystem.h"
@@ -75,9 +75,9 @@ int allocateHybrdData(int size, void** voiddata)
   data->xScalefactors = (double*) malloc(size*sizeof(double));
 
   data->n = size;
-  data->x = (double*) malloc(size*sizeof(double));
-  data->xSave = (double*) malloc(size*sizeof(double));
-  data->xScaled = (double*) malloc(size*sizeof(double));
+  data->x = (double*) malloc((size+1)*sizeof(double));
+  data->xSave = (double*) malloc((size+1)*sizeof(double));
+  data->xScaled = (double*) malloc((size+1)*sizeof(double));
   data->fvec = (double*) calloc(size, sizeof(double));
   data->fvecSave = (double*) calloc(size, sizeof(double));
   data->xtol = 1e-12;
@@ -93,8 +93,8 @@ int allocateHybrdData(int size, void** voiddata)
   data->info = 0;
   data->nfev = 0;
   data->njev = 0;
-  data->fjac = (double*) calloc((size*size), sizeof(double));
-  data->fjacobian = (double*) calloc((size*size), sizeof(double));
+  data->fjac = (double*) calloc((size*(size+1)), sizeof(double));
+  data->fjacobian = (double*) calloc((size*(size+1)), sizeof(double));
   data->ldfjac = size;
   data->r__ = (double*) malloc(((size*(size+1))/2)*sizeof(double));
   data->lr = (size*(size + 1)) / 2;
@@ -263,38 +263,36 @@ static int getAnalyticalJacobian(struct dataAndSys* dataSys, double* jac)
   NONLINEAR_SYSTEM_DATA* systemData = &(data->simulationInfo->nonlinearSystemData[dataSys->sysNumber]);
   DATA_HYBRD* solverData = (DATA_HYBRD*)(systemData->solverData);
   const int index = systemData->jacobianIndex;
+  ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[systemData->jacobianIndex]);
 
   memset(jac, 0, (solverData->n)*(solverData->n)*sizeof(double));
   memset(solverData->fjacobian, 0, (solverData->n)*(solverData->n)*sizeof(double));
 
-  for(i=0; i < data->simulationInfo->analyticJacobians[index].sparsePattern.maxColors; i++)
+  for(i=0; i < jacobian->sparsePattern.maxColors; i++)
   {
     /* activate seed variable for the corresponding color */
-    for(ii=0; ii < data->simulationInfo->analyticJacobians[index].sizeCols; ii++)
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.colorCols[ii]-1 == i)
-        data->simulationInfo->analyticJacobians[index].seedVars[ii] = 1;
+    for(ii=0; ii < jacobian->sizeCols; ii++)
+      if(jacobian->sparsePattern.colorCols[ii]-1 == i)
+        jacobian->seedVars[ii] = 1;
 
-    ((systemData->analyticalJacobianColumn))(data, threadData);
+    ((systemData->analyticalJacobianColumn))(data, threadData, jacobian, NULL);
 
-    for(j=0; j<data->simulationInfo->analyticJacobians[index].sizeCols; j++)
+    for(j=0; j<jacobian->sizeCols; j++)
     {
-      if(data->simulationInfo->analyticJacobians[index].seedVars[j] == 1)
+      if(jacobian->seedVars[j] == 1)
       {
-        if(j==0)
-          ii = 0;
-        else
-          ii = data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j-1];
-        while(ii < data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j])
+        ii = jacobian->sparsePattern.leadindex[j];
+        while(ii < jacobian->sparsePattern.leadindex[j+1])
         {
-          l  = data->simulationInfo->analyticJacobians[index].sparsePattern.index[ii];
-          k  = j*data->simulationInfo->analyticJacobians[index].sizeRows + l;
-          solverData->fjacobian[k] = jac[k] = data->simulationInfo->analyticJacobians[index].resultVars[l];
+          l  = jacobian->sparsePattern.index[ii];
+          k  = j*jacobian->sizeRows + l;
+          solverData->fjacobian[k] = jac[k] = jacobian->resultVars[l];
           ii++;
         };
       }
       /* de-activate seed variable for the corresponding color */
-      if(data->simulationInfo->analyticJacobians[index].sparsePattern.colorCols[j]-1 == i)
-        data->simulationInfo->analyticJacobians[index].seedVars[j] = 0;
+      if(jacobian->sparsePattern.colorCols[j]-1 == i)
+        jacobian->seedVars[j] = 0;
     }
 
   }
@@ -355,6 +353,9 @@ static int wrapper_fvec_hybrj(const integer* n, const double* x, double* f, doub
     if(ACTIVE_STREAM(LOG_NLS_RES))
       infoStreamPrint(LOG_NLS_RES, 0, "-- begin calculating jacobian --");
 
+    /* performance measurement */
+    rt_ext_tp_tick(&systemData->jacobianTimeClock);
+
     /* call apropreated jacobian function */
     if(systemData->jacobianIndex != -1){
       integer iflagtmp = 1;
@@ -388,6 +389,11 @@ static int wrapper_fvec_hybrj(const integer* n, const double* x, double* f, doub
     /* reset residual function again */
     if(continuous)
       data->simulationInfo->solveContinuous = 1;
+
+    /* performance measurement and statistics */
+    systemData->jacobianTime += rt_ext_tp_tock(&(systemData->jacobianTimeClock));
+    systemData->numberOfJEval++;
+
     break;
 
   default:
@@ -440,11 +446,24 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
   relationsPreBackup = (modelica_boolean*) malloc(data->modelData->nRelations*sizeof(modelica_boolean));
 
   solverData->numberOfFunctionEvaluations = 0;
+
+  // Initialize lambda variable
+  if (data->simulationInfo->nonlinearSystemData[sysNumber].homotopySupport) {
+    solverData->x[solverData->n] = 1.0;
+    solverData->xSave[solverData->n] = 1.0;
+    solverData->xScaled[solverData->n] = 1.0;
+  }
+  else {
+    solverData->x[solverData->n] = 0.0;
+    solverData->xSave[solverData->n] = 0.0;
+    solverData->xScaled[solverData->n] = 0.0;
+  }
+
   /* debug output */
   if(ACTIVE_STREAM(LOG_NLS_V))
   {
     int indexes[2] = {1,eqSystemNumber};
-    infoStreamPrintWithEquationIndexes(LOG_NLS_V, 1, indexes, "start solving non-linear system >>%d<< at time %g", eqSystemNumber, data->localData[0]->timeValue);
+    infoStreamPrintWithEquationIndexes(LOG_NLS_V, 1, indexes, "Start solving non-linear system >>%d<< using Hybrd solver at time %g", eqSystemNumber, data->localData[0]->timeValue);
     for(i=0; i<solverData->n; i++)
     {
       infoStreamPrint(LOG_NLS_V, 1, "%d. %s = %f", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], systemData->nlsx[i]);
@@ -516,7 +535,7 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       success = 1;
       if(assertCalled)
       {
-        infoStreamPrint(LOG_NLS, 0, "After assertions failed, found a solution for which assertions did not fail.");
+        infoStreamPrint(LOG_NLS_V, 0, "After assertions failed, found a solution for which assertions did not fail.");
         /* re-scaling x vector */
         for(i=0; i<solverData->n; i++){
           if(solverData->useXScaling)
@@ -544,8 +563,8 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
               warningStreamPrint(LOG_STDOUT, 1, "While solving non-linear system an assertion failed at time %g.", data->localData[0]->timeValue);
             warningStreamPrint(LOG_STDOUT, 0, "The non-linear solver tries to solve the problem that could take some time.");
             warningStreamPrint(LOG_STDOUT, 0, "It could help to provide better start-values for the iteration variables.");
-            if (!ACTIVE_STREAM(LOG_NLS))
-              warningStreamPrint(LOG_STDOUT, 0, "For more information simulate with -lv LOG_NLS");
+            if (!ACTIVE_STREAM(LOG_NLS_V))
+              warningStreamPrint(LOG_STDOUT, 0, "For more information simulate with -lv LOG_NLS_V");
             messageClose(LOG_STDOUT);
           }
           assertMessage = 1;
@@ -684,16 +703,16 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
 
       success = 1;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS))
+      if(ACTIVE_STREAM(LOG_NLS_V))
       {
         int indexes[2] = {1,eqSystemNumber};
         /* output solution */
-        infoStreamPrintWithEquationIndexes(LOG_NLS, 1, indexes, "solution for NLS %d at t=%g", eqSystemNumber, data->localData[0]->timeValue);
+        infoStreamPrintWithEquationIndexes(LOG_NLS_V, 1, indexes, "solution for NLS %d at t=%g", eqSystemNumber, data->localData[0]->timeValue);
         for(i=0; i<solverData->n; ++i)
         {
-          infoStreamPrint(LOG_NLS, 0, "[%d] %s = %g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i],  solverData->x[i]);
+          infoStreamPrint(LOG_NLS_V, 0, "[%d] %s = %g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i],  solverData->x[i]);
         }
-        messageClose(LOG_NLS);
+        messageClose(LOG_NLS_V);
       }else if (ACTIVE_STREAM(LOG_NLS_V)){
         infoStreamPrint(LOG_NLS_V, 1, "system solved");
         infoStreamPrint(LOG_NLS_V, 0, "%d retries\n%d restarts", retries, retries2+retries3);
@@ -763,9 +782,9 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       giveUp = 0;
       nfunc_evals += solverData->nfev;
       assertRetries++;
-      if(ACTIVE_STREAM(LOG_NLS))
+      if(ACTIVE_STREAM(LOG_NLS_V))
       {
-        infoStreamPrint(LOG_NLS, 0, " - try to handle a problem with a called assert vary initial value a bit. (Retry: %d)",assertRetries);
+        infoStreamPrint(LOG_NLS_V, 0, " - try to handle a problem with a called assert vary initial value a bit. (Retry: %d)",assertRetries);
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     }
@@ -784,9 +803,9 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       retries++;
       giveUp = 0;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS))
+      if(ACTIVE_STREAM(LOG_NLS_V))
       {
-        infoStreamPrint(LOG_NLS, 0, " - iteration making no progress:\t decreasing initial step bound to %f.", solverData->factor);
+        infoStreamPrint(LOG_NLS_V, 0, " - iteration making no progress:\t decreasing initial step bound to %f.", solverData->factor);
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     }
@@ -802,9 +821,9 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       giveUp = 0;
       nfunc_evals += solverData->nfev;
 
-      if(ACTIVE_STREAM(LOG_NLS))
+      if(ACTIVE_STREAM(LOG_NLS_V))
       {
-        infoStreamPrint(LOG_NLS, 0, "iteration making no progress:\t vary solution point by 1%%.");
+        infoStreamPrint(LOG_NLS_V, 0, "iteration making no progress:\t vary solution point by 1%%.");
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     }
@@ -825,9 +844,9 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       retries++;
       giveUp = 0;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS))
+      if(ACTIVE_STREAM(LOG_NLS_V))
       {
-        infoStreamPrint(LOG_NLS, 0, "iteration making no progress:\t try old values as scaling factors.");
+        infoStreamPrint(LOG_NLS_V, 0, "iteration making no progress:\t try old values as scaling factors.");
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     }
@@ -854,9 +873,9 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       giveUp = 0;
       nfunc_evals += solverData->nfev;
 
-      if(ACTIVE_STREAM(LOG_NLS))
+      if(ACTIVE_STREAM(LOG_NLS_V))
       {
-        infoStreamPrint(LOG_NLS, 0, "iteration making no progress:\t try without scaling at all.");
+        infoStreamPrint(LOG_NLS_V, 0, "iteration making no progress:\t try without scaling at all.");
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     }
@@ -878,8 +897,8 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
 
       giveUp = 0;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS)) {
-        infoStreamPrint(LOG_NLS, 0, " - iteration making no progress:\t try to solve a discontinuous system.");
+      if(ACTIVE_STREAM(LOG_NLS_V)) {
+        infoStreamPrint(LOG_NLS_V, 0, " - iteration making no progress:\t try to solve a discontinuous system.");
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     /* Then try with old values (instead of extrapolating )*/
@@ -899,8 +918,8 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       retries2++;
       giveUp = 0;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS)) {
-        infoStreamPrint(LOG_NLS, 0, " - iteration making no progress:\t use old values instead extrapolated.");
+      if(ACTIVE_STREAM(LOG_NLS_V)) {
+        infoStreamPrint(LOG_NLS_V, 0, " - iteration making no progress:\t use old values instead extrapolated.");
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     /* try to vary the initial values */
@@ -918,8 +937,8 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       retries2++;
       giveUp = 0;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS)) {
-        infoStreamPrint(LOG_NLS, 0,
+      if(ACTIVE_STREAM(LOG_NLS_V)) {
+        infoStreamPrint(LOG_NLS_V, 0,
             " - iteration making no progress:\t vary initial point by adding 1%%.");
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
@@ -938,8 +957,8 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       retries2++;
       giveUp = 0;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS)) {
-        infoStreamPrint(LOG_NLS, 0, " - iteration making no progress:\t vary initial point by -1%%.");
+      if(ACTIVE_STREAM(LOG_NLS_V)) {
+        infoStreamPrint(LOG_NLS_V, 0, " - iteration making no progress:\t vary initial point by -1%%.");
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     /* try to vary the initial values */
@@ -950,8 +969,8 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       retries2++;
       giveUp = 0;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS)) {
-        infoStreamPrint(LOG_NLS, 0, " - iteration making no progress:\t try scaling factor as initial point.");
+      if(ACTIVE_STREAM(LOG_NLS_V)) {
+        infoStreamPrint(LOG_NLS_V, 0, " - iteration making no progress:\t try scaling factor as initial point.");
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     /* try own scaling factors */
@@ -972,8 +991,8 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       giveUp = 0;
       solverData->mode = 2;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS)) {
-        infoStreamPrint(LOG_NLS, 0, " - iteration making no progress:\t try with own scaling factors.");
+      if(ACTIVE_STREAM(LOG_NLS_V)) {
+        infoStreamPrint(LOG_NLS_V, 0, " - iteration making no progress:\t try with own scaling factors.");
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     /* try without internal scaling */
@@ -994,8 +1013,8 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       solverData->mode = 2;
       giveUp = 0;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS)) {
-        infoStreamPrint(LOG_NLS, 0, " - iteration making no progress:\t disable solver internal scaling.");
+      if(ACTIVE_STREAM(LOG_NLS_V)) {
+        infoStreamPrint(LOG_NLS_V, 0, " - iteration making no progress:\t disable solver internal scaling.");
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     /* try to reduce the tolerance a bit */
@@ -1018,8 +1037,8 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
 
       giveUp = 0;
       nfunc_evals += solverData->nfev;
-      if(ACTIVE_STREAM(LOG_NLS)) {
-        infoStreamPrint(LOG_NLS, 0, " - iteration making no progress:\t reduce the tolerance slightly to %e.", local_tol);
+      if(ACTIVE_STREAM(LOG_NLS_V)) {
+        infoStreamPrint(LOG_NLS_V, 0, " - iteration making no progress:\t reduce the tolerance slightly to %e.", local_tol);
         printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
     } else if(solverData->info >= 2 && solverData->info <= 5) {
@@ -1028,9 +1047,9 @@ int solveHybrd(DATA *data, threadData_t *threadData, int sysNumber)
       if(!data->simulationInfo->initial){
         printErrorEqSyst(ERROR_AT_TIME, modelInfoGetEquation(&data->modelData->modelDataXml, eqSystemNumber), data->localData[0]->timeValue);
       }
-      if (ACTIVE_STREAM(LOG_NLS)) {
-        infoStreamPrint(LOG_NLS, 0, "### No Solution! ###\n after %d restarts", retries*retries2*retries3);
-        printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS);
+      if (ACTIVE_STREAM(LOG_NLS_V)) {
+        infoStreamPrint(LOG_NLS_V, 0, "### No Solution! ###\n after %d restarts", retries*retries2*retries3);
+        printStatus(data, solverData, eqSystemNumber, &nfunc_evals, &xerror, &xerror_scaled, LOG_NLS_V);
       }
       /* take the best approximation */
       memcpy(systemData->nlsx, solverData->x, solverData->n*(sizeof(double)));
