@@ -53,7 +53,7 @@ typedef struct _FILE_INFO
   int readonly;
 } FILE_INFO;
 
-#define omc_dummyFileInfo {"",-1,-1,-1,-1,1}
+#define omc_dummyFileInfo {"",0,0,0,0,0}
 
 DLLExport extern void printInfo(FILE *stream, FILE_INFO info);
 DLLExport extern void (*omc_assert)(threadData_t*, FILE_INFO, const char*, ...) __attribute__ ((noreturn));
@@ -85,6 +85,7 @@ enum LOG_STREAM
   LOG_DSS,
   LOG_DSS_JAC,
   LOG_DT,
+  LOG_DT_CONS,
   LOG_EVENTS,
   LOG_EVENTS_V,
   LOG_INIT,
@@ -104,12 +105,15 @@ enum LOG_STREAM
   LOG_NLS_RES,
   LOG_NLS_EXTRAPOLATE,
   LOG_RES_INIT,
+  LOG_RT,
   LOG_SIMULATION,
   LOG_SOLVER,
+  LOG_SOLVER_V,
   LOG_SOLVER_CONTEXT,
   LOG_SOTI,
   LOG_STATS,
   LOG_STATS_V,
+  LOG_SUCCESS,
 #ifdef USE_DEBUG_TRACE
   LOG_TRACE,
 #endif
@@ -119,11 +123,6 @@ enum LOG_STREAM
   SIM_LOG_MAX
 };
 
-extern const int firstOMCErrorStream;
-extern const char *LOG_STREAM_NAME[SIM_LOG_MAX];
-extern const char *LOG_STREAM_DESC[SIM_LOG_MAX];
-extern const char *LOG_STREAM_DETAILED_DESC[SIM_LOG_MAX];
-
 enum LOG_TYPE
 {
   LOG_TYPE_UNKNOWN = 0,
@@ -135,6 +134,12 @@ enum LOG_TYPE
   LOG_TYPE_MAX
 };
 
+extern const int firstOMCErrorStream;
+extern const char *LOG_STREAM_NAME[SIM_LOG_MAX];
+extern const char *LOG_STREAM_DESC[SIM_LOG_MAX];
+extern const char *LOG_STREAM_DETAILED_DESC[SIM_LOG_MAX];
+extern const char *LOG_TYPE_DESC[LOG_TYPE_MAX];
+
 extern int useStream[SIM_LOG_MAX];
 extern int level[SIM_LOG_MAX];
 extern int lastType[SIM_LOG_MAX];
@@ -142,8 +147,6 @@ extern int lastStream;
 extern int showAllWarnings;
 extern char logBuffer[2048];
 
-void setStreamPrintXML(int isXML);
-
 #define ACTIVE_STREAM(stream)    (useStream[stream])
 #define ACTIVE_WARNING_STREAM(stream)    (showAllWarnings || useStream[stream])
 
@@ -163,8 +166,11 @@ void setStreamPrintXML(int isXML);
   #define TRACE_POP
 #endif
 
+extern void (*messageFunction)(int type, int stream, int indentNext, char *msg, int subline, const int *indexes);
 extern void (*messageClose)(int stream);
 extern void (*messageCloseWarning)(int stream);
+
+#if !defined(OMC_MINIMAL_LOGGING)
 extern void va_infoStreamPrint(int stream, int indentNext, const char *format, va_list ap);
 extern void infoStreamPrint(int stream, int indentNext, const char *format, ...) __attribute__ ((format (printf, 3, 4)));
 extern void infoStreamPrintWithEquationIndexes(int stream, int indentNext, const int *indexes, const char *format, ...) __attribute__ ((format (printf, 4, 5)));
@@ -175,11 +181,24 @@ extern void va_warningStreamPrintWithEquationIndexes(int stream, int indentNext,
 extern void errorStreamPrint(int stream, int indentNext, const char *format, ...) __attribute__ ((format (printf, 3, 4)));
 extern void va_errorStreamPrint(int stream, int indentNext, const char *format, va_list ap);
 extern void va_errorStreamPrintWithEquationIndexes(int stream, int indentNext, const int *indexes, const char *format,va_list ap);
+#else
+static inline void va_infoStreamPrint(int stream, int indentNext, const char *format, va_list ap) {}
+static inline void infoStreamPrint(int stream, int indentNext, const char *format, ...) {}
+static inline void infoStreamPrintWithEquationIndexes(int stream, int indentNext, const int *indexes, const char *format, ...) {}
+static inline void warningStreamPrint(int stream, int indentNext, const char *format, ...) {}
+static inline void va_warningStreamPrint(int stream, int indentNext, const char *format,va_list ap) {}
+static inline void warningStreamPrintWithEquationIndexes(int stream, int indentNext, const int *indexes, const char *format, ...) {}
+static inline void va_warningStreamPrintWithEquationIndexes(int stream, int indentNext, const int *indexes, const char *format,va_list ap) {}
+static inline void errorStreamPrint(int stream, int indentNext, const char *format, ...) {}
+static inline void va_errorStreamPrint(int stream, int indentNext, const char *format, va_list ap) {}
+static inline void va_errorStreamPrintWithEquationIndexes(int stream, int indentNext, const int *indexes, const char *format,va_list ap) {}
+#endif
+
 extern void va_throwStreamPrint(threadData_t *threadData, const char *format, va_list ap) __attribute__ ((noreturn));
 extern void throwStreamPrint(threadData_t *threadData, const char *format, ...) __attribute__ ((format (printf, 2, 3), noreturn));
 extern void throwStreamPrintWithEquationIndexes(threadData_t *threadData, const int *indexes, const char *format, ...) __attribute__ ((format (printf, 3, 4), noreturn));
 #ifdef HAVE_VA_MACROS
-#define assertStreamPrint(threadData, cond, ...) (cond) ? (void) 0 : throwStreamPrint((threadData), __VA_ARGS__)
+#define assertStreamPrint(threadData, cond, ...) if (!(cond)) {throwStreamPrint((threadData), __VA_ARGS__); assert(0);}
 #else
 static void OMC_INLINE assertStreamPrint(threadData_t *threadData, int cond, const char *format, ...) __attribute__ ((format (printf, 3, 4)));
 static void OMC_INLINE assertStreamPrint(threadData_t *threadData, int cond, const char *format, ...)
