@@ -30,9 +30,10 @@
 
 
 #include "string_array.h"
-#include "memory_pool.h"
+#include "gc/omc_gc.h"
 #include "index_spec.h"
-#include "meta/meta_modelica.h"
+#include "modelica_string.h"
+#include "omc_error.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -243,53 +244,24 @@ void simple_indexed_assign_string_array2(const string_array_t * source,
     string_set(dest, index, string_get(*source, index));
 }
 
-void indexed_assign_string_array(const string_array_t * source,
+void indexed_assign_string_array(const string_array_t source,
                                  string_array_t* dest,
                                  const index_spec_t* dest_spec)
 {
-    _index_t* idx_vec1;
-    _index_t* idx_vec2;
-    _index_t* idx_size;
-    int i,j;
-
-    assert(base_array_ok(source));
-    assert(base_array_ok(dest));
-    assert(index_spec_ok(dest_spec));
-    assert(index_spec_fit_base_array(dest_spec, dest));
-    for(i = 0,j = 0; i < dest_spec->ndims; ++i) {
-        if(dest_spec->dim_size[i] != 0) {
-            ++j;
-        }
-    }
-    assert(j == source->ndims);
-
-    idx_vec1 = size_alloc(dest->ndims);
-    idx_vec2 = size_alloc(source->ndims);
-    idx_size = size_alloc(dest_spec->ndims);
-
-    for(i = 0; i < dest_spec->ndims; ++i) {
-        idx_vec1[i] = 0;
-
-        if(dest_spec->index[i] != NULL) {
-            idx_size[i] = imax(dest_spec->dim_size[i],1);
-        } else {
-            idx_size[i] = dest->dim_size[i];
-        }
-    }
+    _index_t *idx_vec1, *idx_size;
+    int j;
+    indexed_assign_base_array_size_alloc(&source, dest, dest_spec, &idx_vec1, &idx_size);
 
+    j = 0;
     do {
-        for(i = 0, j = 0; i < dest_spec->ndims; ++i) {
-            if(dest_spec->dim_size[i] != 0) {
-                idx_vec2[j] = idx_vec1[i];
-                ++j;
-            }
-        }
-        string_set(dest, calc_base_index_spec(dest->ndims, idx_vec1,
-                                              dest, dest_spec),
-                   string_get(*source, calc_base_index(source->ndims,
-                                                      idx_vec2, source)));
+        string_set(dest,
+                 calc_base_index_spec(dest->ndims, idx_vec1, dest, dest_spec),
+                 string_get(source, j));
+        j++;
 
     } while(0 == next_index(dest_spec->ndims, idx_vec1, idx_size));
+
+    omc_assert_macro(j == base_array_nr_of_elements(source));
 }
 
 /*
@@ -374,37 +346,7 @@ void index_alloc_string_array(const string_array_t * source,
                               const index_spec_t* source_spec,
                               string_array_t* dest)
 {
-    int i;
-    int j;
-    int ndimsdiff;
-
-    assert(base_array_ok(source));
-    assert(index_spec_ok(source_spec));
-    assert(index_spec_fit_base_array(source_spec, source));
-
-    ndimsdiff = 0;
-    for(i = 0; i < source_spec->ndims; ++i) {
-        if((source_spec->index_type[i] == 'W')
-            ||
-            (source_spec->index_type[i] == 'A')) {
-            ndimsdiff--;
-        }
-    }
-
-    dest->ndims = source->ndims + ndimsdiff;
-    dest->dim_size = size_alloc(dest->ndims);
-
-    for(i = 0,j = 0; i < dest->ndims; ++i) {
-        while(source_spec->index_type[i+j] == 'S') { /* Skip scalars */
-            j++;
-        }
-        if(source_spec->index_type[i+j] == 'W') { /*take whole dimension from source*/
-            dest->dim_size[i]=source->dim_size[i+j];
-        } else if(source_spec->index_type[i+j] == 'A') { /* Take dimension size from splice*/
-            dest->dim_size[i]=source_spec->dim_size[i+j];
-        }
-    }
-
+    index_alloc_base_array_size(source, source_spec, dest);
     alloc_string_array_data(dest);
     index_string_array(source, source_spec, dest);
 }
@@ -521,22 +463,6 @@ void array_alloc_string_array(string_array_t* dest, int n,
     free(elts);
 }
 
-void array_scalar_string_array(string_array_t* dest, int n,
-                               modelica_string first, ...)
-{
-    int i;
-    va_list ap;
-    assert(base_array_ok(dest));
-    assert(dest->ndims == 1);
-    assert(dest->dim_size[0] == n);
-    put_string_element(first, 0, dest);
-    va_start(ap,first);
-    for(i = 0; i < n; ++i) {
-        put_string_element(va_arg(ap, modelica_string),i,dest);
-    }
-    va_end(ap);
-}
-
 /* array_alloc_scalar_string_array
  *
  * Creates(incl allocation) an array from scalar elements.
@@ -588,13 +514,13 @@ modelica_string* string_array_element_addr(const string_array_t * source,
  * k is one based
  */
 void cat_string_array(int k, string_array_t* dest, int n,
-                    string_array_t* first,...)
+                    const string_array_t* first,...)
 {
     va_list ap;
     int i, j, r, c;
     int n_sub = 1, n_super = 1;
     int new_k_dim_size = 0;
-    string_array_t **elts = (string_array_t**)malloc(sizeof(string_array_t *) * n);
+    const string_array_t **elts = (const string_array_t**)malloc(sizeof(string_array_t *) * n);
 
     assert(elts);
     /* collect all array ptrs to simplify traversal.*/
@@ -602,7 +528,7 @@ void cat_string_array(int k, string_array_t* dest, int n,
     elts[0] = first;
 
     for(i = 1; i < n; i++) {
-        elts[i] = va_arg(ap,string_array_t*);
+        elts[i] = va_arg(ap,const string_array_t*);
     }
     va_end(ap);
 
@@ -650,13 +576,13 @@ void cat_string_array(int k, string_array_t* dest, int n,
  * k is one based
  */
 void cat_alloc_string_array(int k, string_array_t* dest, int n,
-                          string_array_t* first,...)
+                          const string_array_t* first,...)
 {
     va_list ap;
     int i, j, r, c;
     int n_sub = 1, n_super = 1;
     int new_k_dim_size = 0;
-    string_array_t **elts = (string_array_t**)malloc(sizeof(string_array_t *) * n);
+    const string_array_t **elts = (const string_array_t**)malloc(sizeof(string_array_t *) * n);
 
     assert(elts);
     /* collect all array ptrs to simplify traversal.*/
@@ -664,7 +590,7 @@ void cat_alloc_string_array(int k, string_array_t* dest, int n,
     elts[0] = first;
 
     for(i = 1; i < n; i++) {
-        elts[i] = va_arg(ap,string_array_t*);
+        elts[i] = va_arg(ap,const string_array_t*);
     }
     va_end(ap);
 
@@ -944,7 +870,7 @@ const char** data_of_string_c89_array(const string_array_t *a)
 {
   long i;
   size_t sz = base_array_nr_of_elements(*a);
-  const char **res = (const char**) GC_malloc(sz*sizeof(const char*));
+  const char **res = (const char**) omc_alloc_interface.malloc(sz*sizeof(const char*));
   for (i=0; i<sz; i++) {
     res[i] = MMC_STRINGDATA(((void**)a->data)[i]);
   }
@@ -956,6 +882,6 @@ void unpack_string_array(const string_array_t *a, const char **data)
   size_t sz = base_array_nr_of_elements(*a);
   long i;
   for (i=0; i<sz; i++) {
-    data[i] = mmc_mk_scon(data[i]);
+    ((void**)a->data)[i] = mmc_mk_scon(data[i]);
   }
 }
