@@ -38,8 +38,9 @@
 
 /**
  * Modelica slice.
- * Defined by start:stop or start:step:stop, start = 0 or stop = 0 meaning end,
- * or by an index vector if step = 0.
+ * Defined by an index vector iset != NULL or by start:stop or start:step:stop,
+ * start == stop and step == 0 meaning reduction of dimension,
+ * start == 0 or stop == 0 meaning end.
  */
 class Slice {
  public:
@@ -51,10 +52,10 @@ class Slice {
     iset = NULL;
   }
 
-  // one index
+  // one index (reduction)
   Slice(int index) {
     start = index;
-    step = 1;
+    step = 0;
     stop = index;
     iset = NULL;
   }
@@ -73,6 +74,7 @@ class Slice {
     iset = NULL;
   }
 
+  // index set, reduction if size(indices) == 1
   Slice(const BaseArray<int> &indices) {
     start = 0;
     step = 0;
@@ -84,9 +86,9 @@ class Slice {
     iset = &indices;
   }
 
-  size_t start;
-  size_t step;
-  size_t stop;
+  int start;
+  int step;
+  int stop;
   const BaseArray<int> *iset;
 };
 
@@ -99,45 +101,55 @@ class ArraySliceConst: public BaseArray<T> {
   ArraySliceConst(const BaseArray<T> &baseArray, const vector<Slice> &slice)
     : BaseArray<T>(baseArray.isStatic(), false)
     , _baseArray(baseArray)
-    , _isets(slice.size())
-    , _idxs(slice.size())
-    , _baseIdx(slice.size())
+    , _isets(baseArray.getNumDims())
+    , _idxs(baseArray.getNumDims())
+    , _baseIdx(baseArray.getNumDims())
     , _tmp_data(NULL) {
 
-    if (baseArray.getNumDims() != slice.size())
+    if (baseArray.getNumDims() < slice.size())
       throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,
-                                    "Wrong dimensions for ArraySlice");
+                                    "Wrong slices exceeding array dimensions");
     // create an explicit index set per dimension,
     // except for all indices that are indicated with an empty index set
-    size_t dim;
+    size_t dim, size;
     vector<Slice>::const_iterator sit;
     vector< vector<size_t> >::iterator dit = _idxs.begin();
     for (dim = 1, sit = slice.begin(); sit != slice.end(); dim++, sit++) {
-      if (sit->step == 0)
+      if (sit->iset != NULL) {
         _isets[dim - 1] = sit->iset;
+        size = sit->iset->getNumElems();
+      }
       else {
         _isets[dim - 1] = NULL;
-        size_t maxIndex = baseArray.getDim(dim);
-        size_t start = sit->start > 0? sit->start: maxIndex;
-        size_t stop = sit->stop > 0? sit->stop: maxIndex;
+        int maxIndex = baseArray.getDim(dim);
+        int start = sit->start > 0? sit->start: maxIndex;
+        int stop = sit->stop > 0? sit->stop: maxIndex;
+        int step = sit->step;
         if (start > maxIndex || stop > maxIndex)
           throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,
                                         "Wrong slice exceeding array size");
-        if (start > 1 || sit->step > 1 || stop < maxIndex)
-          for (size_t i = start; i <= stop; i += sit->step)
-            dit->push_back(i);
+        if (start == 1 && step == 1 && stop == maxIndex)
+          // all indices; avoid trivial fill of _idxs
+          size = _baseArray.getDim(dim);
+        else {
+          size = step == 0? 1: std::max(0, (stop - start) / step + 1);
+          for (int i = 0; i < size; i++)
+            dit->push_back(start + i * step);
+        }
       }
-      if (dit->size() == 1)
-        // prefill constant _baseIdx in case of reduction
-        _baseIdx[dim - 1] = (*dit)[0];
+      if (size == 1 && sit->step == 0)
+        // preset constant _baseIdx in case of reduction
+        _baseIdx[dim - 1] = sit->iset != NULL? (*_isets[dim - 1])(1): (*dit)[0];
       else
-	// store dimension of array slice
-	if(_isets[dim-1] !=NULL && _isets[dim-1]->getDim(0) > 1)
-	   _dims.push_back(_isets[dim-1]->getData()[1]-_isets[dim-1]->getData()[0] +1);
-	else
-	   _dims.push_back(dit->size() != 0? dit->size(): _baseArray.getDim(dim));
+        // store dimension of array slice
+        _dims.push_back(size);
       dit++;
     }
+    // use all indices of remaining dims
+    for (; dim <= baseArray.getNumDims(); dim++) {
+      _isets[dim - 1] = NULL;
+      _dims.push_back(_baseArray.getDim(dim));
+    }
   }
 
   virtual ~ArraySliceConst() {
@@ -164,6 +176,11 @@ class ArraySliceConst: public BaseArray<T> {
                                   "Can't assign array to ArraySliceConst");
   }
 
+  virtual void assign(const T& value) {
+    throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,
+                                  "Can't assign value to ArraySliceConst");
+  }
+
   virtual std::vector<size_t> getDims() const {
     return _dims;
   }
@@ -181,7 +198,18 @@ class ArraySliceConst: public BaseArray<T> {
     if (n != getNumElems())
       throw ModelicaSimulationError(MODEL_ARRAY_FUNCTION,
                                     "Wrong number of elements in getDataCopy");
-    getDataDim(_idxs.size(), data);
+    if (n > 0) {
+      const T* base_data = _baseArray.getData();
+      if (base_data <= data && data < base_data + n) {
+        // in-situ access requires an internal copy to avoid side effects,
+        // e.g. v = v[n:-1:1]
+        const T* slice_data = getData();
+        std::copy(slice_data, slice_data + n, data);
+      }
+      else
+        // direct access
+        getDataDim(_idxs.size(), data);
+    }
   }
 
   virtual const T* getData() const {
@@ -293,6 +321,18 @@ class ArraySlice: public ArraySliceConst<T> {
     , _baseIdx(ArraySliceConst<T>::_baseIdx) {
   }
 
+  ArraySlice<T>& operator=(const ArraySlice<T>& b)
+  {
+    this->assign(b);
+    return *this;
+  }
+
+  ArraySlice<T>& operator=(const BaseArray<T>& b)
+  {
+    this->assign(b);
+    return *this;
+  }
+
   virtual T& operator()(const vector<size_t> &idx) {
     return _baseArray(ArraySliceConst<T>::baseIdx(idx.size(), &idx[0]));
   }
@@ -305,6 +345,10 @@ class ArraySlice: public ArraySliceConst<T> {
     setDataDim(_idxs.size(), otherArray.getData());
   }
 
+  virtual void assign(const T& value) {
+    setEachDim(_idxs.size(), value);
+  }
+
   virtual T& operator()(size_t i) {
     return _baseArray(ArraySliceConst<T>::baseIdx(1, &i));
   }
@@ -355,5 +399,25 @@ class ArraySlice: public ArraySliceConst<T> {
     }
     return processed;
   }
+
+  /**
+   * recursive method for muli-dimensional fill of each element
+   */
+  void setEachDim(size_t dim, const T& value) {
+    const BaseArray<int> *iset = ArraySliceConst<T>::_isets[dim - 1];
+    size_t size = iset? iset->getNumElems(): _idxs[dim - 1].size();
+    if (size == 0)
+      size = _baseArray.getDim(dim);
+    for (size_t i = 1; i <= size; i++) {
+      if (iset)
+        _baseIdx[dim - 1] = iset->getNumElems() > 0? (*iset)(i): i;
+      else
+        _baseIdx[dim - 1] = _idxs[dim - 1].size() > 0? _idxs[dim - 1][i - 1]: i;
+      if (dim > 1)
+        setEachDim(dim - 1, value);
+      else
+        _baseArray(_baseIdx) = value;
+    }
+  }
 };
 /** @} */ // end of math
