@@ -12,16 +12,16 @@ class Configuration : public ConfigurationPolicy
 public:
   Configuration(PATH libraries_path,PATH config_path,PATH modelicasystem_path);
   ~Configuration(void);
-  boost::shared_ptr<ISolver> createSelectedSolver(IMixedSystem* system);
-  IGlobalSettings* getGlobalSettings();
+  shared_ptr<ISolver> createSelectedSolver(IMixedSystem* system);
+  shared_ptr<IGlobalSettings> getGlobalSettings();
   ISolverSettings* getSolverSettings();
   ISimControllerSettings* getSimControllerSettings();
 
 private:
-   boost::shared_ptr<ISettingsFactory> _settings_factory;
-   boost::shared_ptr<ISolverSettings> _solver_settings;
-   boost::shared_ptr<IGlobalSettings> _global_settings;
-   boost::shared_ptr<ISimControllerSettings> _simcontroller_settings;
-   boost::shared_ptr<ISolver> _solver;
+   shared_ptr<ISettingsFactory> _settings_factory;
+   shared_ptr<ISolverSettings> _solver_settings;
+   shared_ptr<IGlobalSettings> _global_settings;
+   shared_ptr<ISimControllerSettings> _simcontroller_settings;
+   shared_ptr<ISolver> _solver;
 };
 /** @} */ // end of coreSimcontroller
\ No newline at end of file
