@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -71,7 +71,7 @@ Parameter::Parameter(Component *pComponent, bool showStartAttribute, QString tab
   // set the value type based on component type.
   OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
   if (mpComponent->getComponentInfo()->getClassName().compare("Boolean") == 0) {
-    if (mpComponent->getChoicesAnnotation().size() > 0 &&
+    if (mpComponent->getChoicesAnnotation().size() > 1 && /* Size should be 2. We always get choices(checkBox, __Dymola_checkBox) */
         (mpComponent->getChoicesAnnotation().at(0).compare("true") == 0 || mpComponent->getChoicesAnnotation().at(1).compare("true") == 0)) {
       mValueType = Parameter::CheckBox;
     } else {
@@ -166,6 +166,7 @@ void Parameter::setValueWidget(QString value, bool defaultValue, QString fromUni
   if (!fromUnit.isEmpty() && mpUnitComboBox->currentText().compare(fromUnit) != 0) {
     bool ok = true;
     qreal realValue = value.toDouble(&ok);
+    // if the modifier is a literal constant
     if (ok) {
       OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
       OMCInterface::convertUnits_res convertUnit = pOMCProxy->convertUnits(fromUnit, mpUnitComboBox->currentText());
@@ -173,6 +174,8 @@ void Parameter::setValueWidget(QString value, bool defaultValue, QString fromUni
         realValue = Utilities::convertUnit(realValue, convertUnit.offset, convertUnit.scaleFactor);
         value = QString::number(realValue);
       }
+    } else { // if expression
+      value = Utilities::arrayExpressionUnitConversion(MainWindow::instance()->getOMCProxy(), value, fromUnit, mpUnitComboBox->currentText());
     }
   }
   mDefaultValue = defaultValue;
@@ -773,7 +776,7 @@ void ComponentParameters::setUpDialog()
   mpOkButton = new QPushButton(Helper::ok);
   mpOkButton->setAutoDefault(true);
   connect(mpOkButton, SIGNAL(clicked()), this, SLOT(updateComponentParameters()));
-  if (mpComponent->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->isSystemLibrary()) {
+  if (mpComponent->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->isSystemLibrary() || mpComponent->getGraphicsView()->isVisualizationView()) {
     mpOkButton->setDisabled(true);
   }
   mpCancelButton = new QPushButton(Helper::cancel);
@@ -1170,12 +1173,17 @@ void ComponentParameters::updateComponentParameters()
     if (!pParameter->getUnit().isEmpty() && pParameter->getUnit().compare(pParameter->getUnitComboBox()->currentText()) != 0) {
       bool ok = true;
       qreal componentModifierRealValue = componentModifierValue.toDouble(&ok);
+      // if the modifier is a literal constant
       if (ok) {
         OMCInterface::convertUnits_res convertUnit = pOMCProxy->convertUnits(pParameter->getUnitComboBox()->currentText(), pParameter->getUnit());
         if (convertUnit.unitsCompatible) {
           componentModifierRealValue = Utilities::convertUnit(componentModifierRealValue, convertUnit.offset, convertUnit.scaleFactor);
           componentModifierValue = QString::number(componentModifierRealValue);
         }
+      } else { // if expression
+        componentModifierValue = Utilities::arrayExpressionUnitConversion(pOMCProxy, componentModifierValue,
+                                                                          pParameter->getUnitComboBox()->currentText(),
+                                                                          pParameter->getUnit());
       }
     }
     if (pParameter->isValueModified()) {
@@ -1299,7 +1307,7 @@ void ComponentAttributes::setUpDialog()
   // dimensions
   mpDimensionsLabel = new Label(tr("Dimensions:"));
   mpDimensionsTextBox = new QLineEdit;
-  mpDimensionsTextBox->setToolTip(tr("Array of dimensions e.g {1, 5, 2}"));
+  mpDimensionsTextBox->setToolTip(tr("Array of dimensions e.g [1, 5, 2]"));
   mpCommentLabel = new Label(Helper::comment);
   mpCommentTextBox = new QLineEdit;
   mpPathLabel = new Label(Helper::path);
@@ -1399,7 +1407,8 @@ void ComponentAttributes::initializeDialog()
   mpNameTextBox->setText(mpComponent->getComponentInfo()->getName());
   mpNameTextBox->setCursorPosition(0);
   // get dimensions
-  mpDimensionsTextBox->setText(mpComponent->getComponentInfo()->getArrayIndex());
+  QString dimensions = StringHandler::removeFirstLastCurlBrackets(mpComponent->getComponentInfo()->getArrayIndex());
+  mpDimensionsTextBox->setText(QString("[%1]").arg(dimensions));
   // get Comment
   mpCommentTextBox->setText(mpComponent->getComponentInfo()->getComment());
   mpCommentTextBox->setCursorPosition(0);
@@ -1505,7 +1514,8 @@ void ComponentAttributes::updateComponentAttributes()
   newComponentInfo.setInner(mpInnerCheckBox->isChecked());
   newComponentInfo.setOuter(mpOuterCheckBox->isChecked());
   newComponentInfo.setCausality(causality);
-  newComponentInfo.setArrayIndex(mpDimensionsTextBox->text());
+  QString dimensions = StringHandler::removeFirstLastSquareBrackets(mpDimensionsTextBox->text());
+  newComponentInfo.setArrayIndex(QString("{%1}").arg(dimensions));
   /* If user has really changed the Component's attributes then push that change on the stack.
    */
   if (oldComponentInfo != newComponentInfo) {
