@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -37,6 +37,12 @@
 
 #undef smooth
 
+extern "C" {
+#include "meta/meta_modelica.h"
+#include "omc_config.h"
+#include "gc.h"
+}
+
 #include <QtGlobal>
 #if (QT_VERSION < QT_VERSION_CHECK(4, 6, 0))
 #error "OMEdit requires Qt 4.6.0 or newer"
@@ -51,7 +57,7 @@
 #include <QToolButton>
 #include <QMdiSubWindow>
 #include <QMdiArea>
-
+#include <QShortcut>
 
 class OMCProxy;
 class TransformationsWidget;
@@ -70,6 +76,7 @@ class ThreeDViewer;
 class BreakpointsWidget;
 class SimulationDialog;
 class TLMCoSimulationDialog;
+class OMSSimulationDialog;
 class ModelWidgetContainer;
 class WelcomePageWidget;
 class InfoBar;
@@ -81,7 +88,7 @@ class GitCommands;
 class CommitChangesDialog;
 class TraceabilityInformationURI;
 class TraceabilityGraphViewWidget;
-
+class SearchWidget;
 
 class MainWindow : public QMainWindow
 {
@@ -93,11 +100,12 @@ private:
   static MainWindow *mpInstance;
 public:
   static MainWindow *instance(bool debug = false);
-  void setUpMainWindow();
+  void setUpMainWindow(threadData_t *threadData);
   bool isDebug() {return mDebug;}
   OMCProxy* getOMCProxy() {return mpOMCProxy;}
   void setExitApplicationStatus(bool status) {mExitApplicationStatus = status;}
   bool getExitApplicationStatus() {return mExitApplicationStatus;}
+  int getNumberOfProcessors() {return mNumberOfProcessors;}
   LibraryWidget* getLibraryWidget() {return mpLibraryWidget;}
   StackFramesWidget* getStackFramesWidget() {return mpStackFramesWidget;}
   BreakpointsWidget* getBreakpointsWidget() {return mpBreakpointsWidget;}
@@ -110,6 +118,7 @@ public:
   PlotWindowContainer* getPlotWindowContainer() {return mpPlotWindowContainer;}
   VariablesWidget* getVariablesWidget() {return mpVariablesWidget;}
   QDockWidget* getVariablesDockWidget() {return mpVariablesDockWidget;}
+  SearchWidget* getSearchWidget() {return mpSearchWidget;}
 
 #if !defined(WITHOUT_OSG)
   bool isThreeDViewerInitialized();
@@ -118,6 +127,7 @@ public:
 #endif
   SimulationDialog* getSimulationDialog() {return mpSimulationDialog;}
   TLMCoSimulationDialog* getTLMCoSimulationDialog() {return mpTLMCoSimulationDialog;}
+  OMSSimulationDialog* getOMSSimulationDialog() {return mpOMSSimulationDialog;}
   ModelWidgetContainer* getModelWidgetContainer() {return mpModelWidgetContainer;}
   WelcomePageWidget* getWelcomePageWidget() {return mpWelcomePageWidget;}
   GitCommands* getGitCommands() {return mpGitCommands;}
@@ -154,6 +164,7 @@ public:
   QAction* getCheckModelAction() {return mpCheckModelAction;}
   QAction* getCheckAllModelsAction() {return mpCheckAllModelsAction;}
   QAction* getExportFMUAction() {return mpExportFMUAction;}
+  QAction* getExportEncryptedPackageAction() {return mpExportEncryptedPackageAction;}
   QAction* getExportXMLAction() {return mpExportXMLAction;}
   QAction* getExportFigaroAction() {return mpExportFigaroAction;}
   QAction* getLineShapeAction() {return mpLineShapeAction;}
@@ -175,13 +186,21 @@ public:
   QAction* getFetchInterfaceDataAction() {return mpFetchInterfaceDataAction;}
   QAction* getAlignInterfacesAction() {return mpAlignInterfacesAction;}
   QAction* getTLMSimulationAction() {return mpTLMCoSimulationAction;}
+  QAction* getAddSystemAction() {return mpAddSystemAction;}
+  QAction* getAddOrEditIconAction() {return mpAddOrEditIconAction;}
+  QAction* getDeleteIconAction() {return mpDeleteIconAction;}
+  QAction* getAddConnectorAction() {return mpAddConnectorAction;}
+  QAction* getAddBusAction() {return mpAddBusAction;}
+  QAction* getAddTLMBusAction() {return mpAddTLMBusAction;}
+  QAction* getAddSubModelAction() {return mpAddSubModelAction;}
+  QAction* getOMSInstantiateModelAction() {return mpOMSInstantiateModelAction;}
+  QAction* getOMSSimulationSetupAction() {return mpOMSSimulateAction;}
   QAction* getLogCurrentFileAction() {return mpLogCurrentFileAction;}
   QAction* getStageCurrentFileForCommitAction() {return mpStageCurrentFileForCommitAction;}
   QAction* getUnstageCurrentFileFromCommitAction() {return mpUnstageCurrentFileFromCommitAction;}
   QAction* getCommitFilesAction() {return mpCommitFilesAction;}
   QAction* getRevertCommitAction() {return mpRevertCommitAction;}
   QAction* getCleanWorkingDirectoryAction() {return mpCleanWorkingDirectoryAction;}
-  QAction* getTraceabilityQueryAction() {return mpTraceabilityQueryAction;}
   void addRecentFile(const QString &fileName, const QString &encoding);
   void updateRecentFileActions();
   void closeEvent(QCloseEvent *event);
@@ -196,10 +215,13 @@ public:
   void simulateWithAnimation(LibraryTreeItem *pLibraryTreeItem);
 #endif
   void simulationSetup(LibraryTreeItem *pLibraryTreeItem);
+  void instantiateOMSModel(LibraryTreeItem *pLibraryTreeItem, bool checked);
+  void simulateOMSModel(LibraryTreeItem *pLibraryTreeItem);
   void instantiateModel(LibraryTreeItem *pLibraryTreeItem);
   void checkModel(LibraryTreeItem *pLibraryTreeItem);
   void checkAllModels(LibraryTreeItem *pLibraryTreeItem);
   void exportModelFMU(LibraryTreeItem *pLibraryTreeItem);
+  void exportEncryptedPackage(LibraryTreeItem *pLibraryTreeItem);
   void exportModelXML(LibraryTreeItem *pLibraryTreeItem);
   void exportModelFigaro(LibraryTreeItem *pLibraryTreeItem);
   void fetchInterfaceData(LibraryTreeItem *pLibraryTreeItem, QString singleModel=QString());
@@ -215,13 +237,16 @@ public:
                                    const char* x1, const char* x2, const char* y1, const char* y2, const char* curveWidth,
                                    const char* curveStyle, const char* legendPosition, const char* footer, const char* autoScale,
                                    const char* variables);
+
+  QList<QString> mFMUDirectoriesList;
 private:
   bool mDebug;
   OMCProxy *mpOMCProxy;
   bool mExitApplicationStatus;
+  int mNumberOfProcessors;
+  SearchWidget *mpSearchWidget;
+  QDockWidget *mpSearchDockWidget;
   QDockWidget *mpMessagesDockWidget;
-  FileDataNotifier *mpOutputFileDataNotifier;
-  FileDataNotifier *mpErrorFileDataNotifier;
   LibraryWidget *mpLibraryWidget;
   QDockWidget *mpLibraryDockWidget;
   GDBAdapter *mpGDBAdapter;
@@ -247,6 +272,7 @@ private:
 #endif
   SimulationDialog *mpSimulationDialog;
   TLMCoSimulationDialog *mpTLMCoSimulationDialog;
+  OMSSimulationDialog *mpOMSSimulationDialog;
   ModelWidgetContainer *mpModelWidgetContainer;
   WelcomePageWidget *mpWelcomePageWidget;
   GitCommands *mpGitCommands;
@@ -258,18 +284,23 @@ private:
   QTabBar *mpPerspectiveTabbar;
   QStatusBar *mpStatusBar;
   QTimer *mpAutoSaveTimer;
+  QShortcut *mpSearchBrowserShortcut;
   // File Menu
   // Modelica File Actions
   QAction *mpNewModelicaClassAction;
   QAction *mpOpenModelicaFileAction;
   QAction *mpOpenModelicaFileWithEncodingAction;
   QAction *mpLoadModelicaLibraryAction;
+  QAction *mpLoadEncryptedLibraryAction;
   QAction *mpOpenResultFileAction;
   QAction *mpOpenTransformationFileAction;
   // CompositeModel File Actions
   QAction *mpNewCompositeModelFileAction;
   QAction *mpOpenCompositeModelFileAction;
   QAction *mpLoadExternModelAction;
+  // OMSimulator File Actions
+  QAction *mpNewOMSimulatorModelAction;
+  QAction *mpOpenOMSModelFileAction;
   QAction *mpOpenDirectoryAction;
   QAction *mpSaveAction;
   QAction *mpSaveAsAction;
@@ -314,6 +345,7 @@ private:
   QAction *mpImportFMUAction;
   QAction *mpImportFMUModelDescriptionAction;
   // Export Menu
+  QAction *mpExportEncryptedPackageAction;
   QAction *mpExportXMLAction;
   QAction *mpExportFigaroAction;
   // Debug Menu
@@ -327,7 +359,6 @@ private:
   QAction *mpCommitFilesAction;
   QAction *mpRevertCommitAction;
   QAction *mpCleanWorkingDirectoryAction;
-  QAction *mpTraceabilityQueryAction;
   // Tools Menu
   QAction *mpShowOMCLoggerWidgetAction;
   QAction *mpShowOpenModelicaCommandPromptAction;
@@ -346,6 +377,7 @@ private:
   QAction *mpModelicaDocumentationAction;
   QAction *mpModelicaByExampleAction;
   QAction *mpModelicaWebReferenceAction;
+  QAction *mpOMSimulatorUsersGuideAction;
   QAction *mpOpenModelicaTLMSimulatorDocumentationAction;
   QAction *mpAboutOMEditAction;
   // Toolbar Actions
@@ -371,6 +403,7 @@ private:
 #if !defined(WITHOUT_OSG)
   QAction *mpNewAnimationWindowAction;
 #endif
+  QAction *mpDiagramWindowAction;
   QAction *mpClearPlotWindowAction;
   QAction *mpExportVariablesAction;
   // Other Actions
@@ -381,6 +414,17 @@ private:
   QAction *mpFetchInterfaceDataAction;
   QAction *mpAlignInterfacesAction;
   QAction *mpTLMCoSimulationAction;
+  // OMSimulator Actions
+  QAction *mpAddSystemAction;
+  QAction *mpAddOrEditIconAction;
+  QAction *mpDeleteIconAction;
+  QAction *mpAddConnectorAction;
+  QAction *mpAddBusAction;
+  QAction *mpAddTLMBusAction;
+  QAction *mpAddSubModelAction;
+  QAction *mpOMSInstantiateModelAction;
+  QAction *mpOMSSimulateAction;
+  QAction *mpOMSArchivedSimulationsAction;
   // Toolbars
   QMenu *mpRecentFilesMenu;
   QMenu *mpLibrariesMenu;
@@ -389,24 +433,33 @@ private:
   QToolBar *mpViewToolBar;
   QToolBar *mpShapesToolBar;
   QToolBar *mpModelSwitcherToolBar;
-  QToolButton *mpModelSwitcherToolButton;
   QMenu *mpModelSwitcherMenu;
+  QToolButton *mpModelSwitcherToolButton;
   QToolBar *mpCheckToolBar;
   QToolBar *mpSimulationToolBar;
   QToolBar *mpReSimulationToolBar;
   QToolBar *mpPlotToolBar;
+  QToolBar *mpDebuggerToolBar;
+  QMenu *mpDebugConfigurationMenu;
+  QToolButton *mpDebugConfigurationToolButton;
   QToolBar *mpTLMSimulationToolbar;
+  QToolBar *mpOMSimulatorToobar;
   QHash<QString, TransformationsWidget*> mTransformationsWidgetHash;
 public slots:
+  void showMessagesBrowser();
+  void showSearchBrowser();
   void createNewModelicaClass();
   void openModelicaFile();
   void showOpenModelicaFileDialog();
   void loadModelicaLibrary();
+  void loadEncryptedLibrary();
   void showOpenResultFileDialog();
   void showOpenTransformationFileDialog();
   void createNewCompositeModelFile();
   void openCompositeModelFile();
   void loadExternalModels();
+  void createNewOMSModel();
+  void openOMSModelFile();
   void openDirectory();
   void loadSystemLibrary();
   void writeOutputFileData(QString data);
@@ -438,6 +491,7 @@ public slots:
   void exportModelFMU();
   void importModelFMU();
   void importFMUModelDescription();
+  void exportEncryptedPackage();
   void exportModelXML();
   void exportModelFigaro();
   void showOpenModelicaCommandPrompt();
@@ -448,6 +502,9 @@ public slots:
   void exportToClipboard();
   void fetchInterfaceData();
   void TLMSimulate();
+  void instantiateOMSModel(bool checked);
+  void simulateOMSModel();
+  void showOMSArchivedSimulations();
   void openWorkingDirectory();
   void openTerminal();
   void openConfigurationOptions();
@@ -459,11 +516,14 @@ public slots:
   void openModelicaDocumentation();
   void openModelicaByExample();
   void openModelicaWebReference();
-  void openOpenModelicaTLMSimulatorDocumentationAction();
+  void openOMSimulatorUsersGuide();
+  void openOpenModelicaTLMSimulatorDocumentation();
   void openAboutOMEdit();
   void toggleShapesButton();
   void openRecentModelWidget();
   void updateModelSwitcherMenu(QMdiSubWindow *pSubWindow);
+  void runDebugConfiguration();
+  void updateDebuggerToolBarMenu();
   void toggleAutoSave();
   void readInterfaceData(LibraryTreeItem *pLibraryTreeItem);
   void enableReSimulationToolbar(bool visible);
@@ -476,7 +536,7 @@ private slots:
   void switchToModelingPerspectiveSlot();
   void switchToPlottingPerspectiveSlot();
   void switchToAlgorithmicDebuggingPerspectiveSlot();
-  void showConfigureDialog();
+  void showDebugConfigurationsDialog();
   void showAttachToProcessDialog();
   void createGitRepository();
   void logCurrentFile();
@@ -485,7 +545,6 @@ private slots:
   void commitFiles();
   void revertCommit();
   void cleanWorkingDirectory();
-  void queryTraceabilityInformation();
 private:
   void createActions();
   void createToolbars();
