@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -37,8 +37,14 @@
 
 #include <QWidget>
 #include <QToolButton>
+#include <QTabBar>
 #include <QFile>
 #include <QWebView>
+#include <QToolBar>
+#include <QComboBox>
+#include <QFontComboBox>
+#include <QSpinBox>
+#include <QColorDialog>
 
 class LibraryTreeItem;
 class DocumentationHistory
@@ -46,6 +52,10 @@ class DocumentationHistory
 public:
   LibraryTreeItem *mpLibraryTreeItem;
   DocumentationHistory(LibraryTreeItem *pLibraryTreeItem) {mpLibraryTreeItem = pLibraryTreeItem;}
+  bool operator==(const DocumentationHistory &documentationHistory) const
+  {
+    return (documentationHistory.mpLibraryTreeItem == this->mpLibraryTreeItem);
+  }
 };
 
 class DocumentationViewer;
@@ -62,26 +72,66 @@ public:
   };
   DocumentationWidget(QWidget *pParent = 0);
   ~DocumentationWidget();
-  QToolButton* getPreviousToolButton() {return mpPreviousToolButton;}
-  QToolButton* getNextToolButton() {return mpNextToolButton;}
+  QAction* getPreviousAction() {return mpPreviousAction;}
+  QAction* getNextAction() {return mpNextAction;}
   DocumentationViewer* getDocumentationViewer() {return mpDocumentationViewer;}
   void showDocumentation(LibraryTreeItem *pLibraryTreeItem);
+  void execCommand(const QString &commandName);
+  void execCommand(const QString &commandName, const QString &valueArgument);
+  bool queryCommandState(const QString &commandName);
+  QString queryCommandValue(const QString &commandName);
 private:
   QFile mDocumentationFile;
-  QToolButton *mpPreviousToolButton;
-  QToolButton *mpNextToolButton;
-  QToolButton *mpEditInfoToolButton;
-  QToolButton *mpEditRevisionsToolButton;
-  QToolButton *mpEditInfoHeaderToolButton;
-  QToolButton *mpSaveToolButton;
-  QToolButton *mpCancelToolButton;
+  QAction *mpPreviousAction;
+  QAction *mpNextAction;
+  QAction *mpEditInfoAction;
+  QAction *mpEditRevisionsAction;
+  QAction *mpEditInfoHeaderAction;
+  QAction *mpSaveAction;
+  QAction *mpCancelAction;
   DocumentationViewer *mpDocumentationViewer;
-  HTMLEditor *mpHTMLEditor;
+  QFrame *mpDocumentationViewerFrame;
+  QWidget *mpEditorsWidget;
+  QTabBar *mpTabBar;
+  QWidget *mpHTMLEditorWidget;
+  QToolBar *mpEditorToolBar;
+  DocumentationViewer *mpHTMLEditor;
+  QFrame *mpHTMLEditorFrame;
+  QComboBox *mpStyleComboBox;
+  QFontComboBox *mpFontComboBox;
+  QSpinBox *mpFontSizeSpinBox;
+  QAction *mpBoldAction;
+  QAction *mpItalicAction;
+  QAction *mpUnderlineAction;
+  QAction *mpStrikethroughAction;
+  QAction *mpSubscriptAction;
+  QAction *mpSuperscriptAction;
+  QColor mTextColor;
+  QColorDialog *mpTextColorDialog;
+  QToolButton *mpTextColorToolButton;
+  QColor mBackgroundColor;
+  QColorDialog *mpBackgroundColorDialog;
+  QToolButton *mpBackgroundColorToolButton;
+  QToolButton *mpAlignLeftToolButton;
+  QToolButton *mpAlignCenterToolButton;
+  QToolButton *mpAlignRightToolButton;
+  QToolButton *mpJustifyToolButton;
+  QAction *mpDecreaseIndentAction;
+  QAction *mpIncreaseIndentAction;
+  QAction *mpBulletListAction;
+  QAction *mpNumberedListAction;
+  QAction *mpLinkAction;
+  QAction *mpUnLinkAction;
+  HTMLEditor *mpHTMLSourceEditor;
   EditType mEditType;
   QList<DocumentationHistory> *mpDocumentationHistoryList;
   int mDocumentationHistoryPos;
 
+  QPixmap createPixmapForToolButton(QColor color, QIcon icon);
   void updatePreviousNextButtons();
+  void writeDocumentationFile(QString documentation);
+  bool isLinkSelected();
+  void updateDocumentationHistory(LibraryTreeItem *pLibraryTreeItem);
 public slots:
   void previousDocumentation();
   void nextDocumentation();
@@ -90,6 +140,25 @@ public slots:
   void editInfoHeaderDocumentation();
   void saveDocumentation(LibraryTreeItem *pNextLibraryTreeItem = 0);
   void cancelDocumentation();
+  void toggleEditor(int tabIndex);
+  void updateActions();
+  void formatBlock(int index);
+  void fontName(QFont font);
+  void fontSize(int size);
+  void applyTextColor();
+  void applyTextColor(QColor color);
+  void applyBackgroundColor();
+  void applyBackgroundColor(QColor color);
+  void alignLeft();
+  void alignCenter();
+  void alignRight();
+  void justify();
+  void bulletList();
+  void numberedList();
+  void createLink();
+  void removeLink();
+  void updateHTMLSourceEditor();
+  void updateDocumentationHistory();
 };
 
 class DocumentationViewer : public QWebView
@@ -98,7 +167,8 @@ class DocumentationViewer : public QWebView
 private:
   DocumentationWidget *mpDocumentationWidget;
 public:
-  DocumentationViewer(DocumentationWidget *pParent);
+  DocumentationViewer(DocumentationWidget *pDocumentationWidget, bool isContentEditable = false);
+  void setFocusInternal();
 private:
   void createActions();
   void resetZoom();
