@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -165,10 +165,14 @@ private:
   Label *mpPathLabel;
   QLineEdit *mpPathTextBox;
   QPushButton *mpPathBrowseButton;
+  QCheckBox *mpSaveContentsInOneFileCheckBox;
   QPushButton *mpOkButton;
   QPushButton *mpCancelButton;
   QDialogButtonBox *mpButtonBox;
+
+  void setSaveContentsTypeAsFolderStructure(LibraryTreeItem *pLibraryTreeItem);
 public slots:
+  int exec();
   void browsePath();
   void duplicateClass();
 };
@@ -237,9 +241,9 @@ private:
   QToolButton *mpAddUsesAnnotationButton;
   QToolButton *mpRemoveUsesAnnotationButton;
   QDialogButtonBox *mpUsesButtonBox;
-  Label *mpOMCFlagsLabel;
-  QString mOMCFlags;
-  QPlainTextEdit *mpOMCFlagsTextBox;
+  Label *mpOMCCommandLineOptionsLabel;
+  QString mOMCCommandLineOptions;
+  QPlainTextEdit *mpOMCCommandLineOptionsTextBox;
   QPushButton *mpOkButton;
   QPushButton *mpCancelButton;
   QDialogButtonBox *mpButtonBox;
