@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -32,18 +32,17 @@
  * @author Adeel Asghar <adeel.asghar@liu.se>
  */
 
+extern "C" {
 #include "meta/meta_modelica.h"
 #include "omc_config.h"
 #include "gc.h"
 
-extern "C" {
 void (*omc_assert)(threadData_t*,FILE_INFO info,const char *msg,...) __attribute__((noreturn)) = omc_assert_function;
 void (*omc_assert_warning)(FILE_INFO info,const char *msg,...) = omc_assert_warning_function;
 void (*omc_terminate)(FILE_INFO info,const char *msg,...) = omc_terminate_function;
 void (*omc_throw)(threadData_t*) __attribute__ ((noreturn)) = omc_throw_function;
-int omc_Main_handleCommand(void *threadData, void *imsg, void *ist, void **omsg, void **ost);
+int omc_Main_handleCommand(void *threadData, void *imsg, void **omsg);
 void* omc_Main_init(void *threadData, void *args);
-void* omc_Main_readSettings(void *threadData, void *args);
 void omc_System_initGarbageCollector(void *threadData);
 #ifdef WIN32
 void omc_Main_setWindowsPaths(threadData_t *threadData, void* _inOMHome);
@@ -55,6 +54,7 @@ void omc_Main_setWindowsPaths(threadData_t *threadData, void* _inOMHome);
 
 #include "OMCProxy.h"
 #include "MainWindow.h"
+#include "Component/Component.h"
 #include "Options/OptionsDialog.h"
 #include "Modeling/MessagesWidget.h"
 #include "simulation_options.h"
@@ -64,13 +64,14 @@ void omc_Main_setWindowsPaths(threadData_t *threadData, void* _inOMHome);
 
 /*!
  * \class OMCProxy
- * \brief It contains the reference of the CORBA object used to communicate with the OpenModelica Compiler.
+ * \brief Interface to send commands to OpenModelica Compiler.
  */
 /*!
  * \brief OMCProxy::OMCProxy
+ * \param threadData
  * \param pParent
  */
-OMCProxy::OMCProxy(QWidget *pParent)
+OMCProxy::OMCProxy(threadData_t* threadData, QWidget *pParent)
   : QObject(pParent), mHasInitialized(false), mResult(""), mTotalOMCCallsTime(0.0)
 {
   mCurrentCommandIndex = -1;
@@ -126,7 +127,7 @@ OMCProxy::OMCProxy(QWidget *pParent)
   mDerivedUnitsMap.clear();
   setLoggingEnabled(true);
   //start the server
-  if(!initializeOMC()) {  // if we are unable to start OMC. Exit the application.
+  if(!initializeOMC(threadData)) {  // if we are unable to start OMC. Exit the application.
     MainWindow::instance()->setExitApplicationStatus(true);
     return;
   }
@@ -181,7 +182,7 @@ void OMCProxy::getNextCommand()
   Creates the omeditcommunication.log & omeditcommands.mos files.
   \return status - returns true if initialization is successful otherwise false.
   */
-bool OMCProxy::initializeOMC()
+bool OMCProxy::initializeOMC(threadData_t *threadData)
 {
   /* create the tmp path */
   QString& tmpPath = Utilities::tempDirectory();
@@ -198,19 +199,16 @@ bool OMCProxy::initializeOMC()
   void *args = mmc_mk_nil();
   QString locale = "+locale=" + settingsLocale.name();
   args = mmc_mk_cons(mmc_mk_scon(locale.toStdString().c_str()), args);
-  // initialize threadData
+  // initialize garbage collector
   omc_System_initGarbageCollector(NULL);
-  threadData_t *threadData = (threadData_t *) GC_malloc(sizeof(threadData_t));
-  void *st = 0;
   MMC_TRY_TOP_INTERNAL()
   omc_Main_init(threadData, args);
-  st = omc_Main_readSettings(threadData, mmc_mk_nil());
   threadData->plotClassPointer = MainWindow::instance();
   threadData->plotCB = MainWindow::PlotCallbackFunction;
   MMC_CATCH_TOP(return false;)
-  mpOMCInterface = new OMCInterface(threadData, st);
+  mpOMCInterface = new OMCInterface(threadData);
   connect(mpOMCInterface, SIGNAL(logCommand(QString,QTime*)), this, SLOT(logCommand(QString,QTime*)));
-  connect(mpOMCInterface, SIGNAL(logResponse(QString,QTime*)), this, SLOT(logResponse(QString,QTime*)));
+  connect(mpOMCInterface, SIGNAL(logResponse(QString,QString,QTime*)), this, SLOT(logResponse(QString,QString,QTime*)));
   connect(mpOMCInterface, SIGNAL(throwException(QString)), SLOT(showException(QString)));
   mHasInitialized = true;
   // get OpenModelica version
@@ -256,13 +254,6 @@ void OMCProxy::quitOMC()
  */
 void OMCProxy::sendCommand(const QString expression)
 {
-  if (!mHasInitialized) {
-    // if we are unable to start OMC. Exit the application.
-    if(!initializeOMC()) {
-      MainWindow::instance()->setExitApplicationStatus(true);
-      return;
-    }
-  }
   // write command to the commands log.
   QTime commandTime;
   commandTime.start();
@@ -275,14 +266,14 @@ void OMCProxy::sendCommand(const QString expression)
 
   MMC_TRY_STACK()
 
-  if (!omc_Main_handleCommand(threadData, mmc_mk_scon(expression.toStdString().c_str()), mpOMCInterface->st, &reply_str, &mpOMCInterface->st)) {
+  if (!omc_Main_handleCommand(threadData, mmc_mk_scon(expression.toStdString().c_str()), &reply_str)) {
     if (expression == "quit()") {
       return;
     }
     exitApplication();
   }
   mResult = MMC_STRINGDATA(reply_str);
-  logResponse(mResult.trimmed(), &commandTime);
+  logResponse(expression, mResult.trimmed(), &commandTime);
 
   MMC_ELSE()
     mResult = "";
@@ -355,8 +346,17 @@ void OMCProxy::logCommand(QString command, QTime *commandTime)
  * \param response - the response to write
  * \param responseTime - the response end time
  */
-void OMCProxy::logResponse(QString response, QTime *responseTime)
+void OMCProxy::logResponse(QString command, QString response, QTime *responseTime)
 {
+  QString firstLine("");
+  for (int i = 0; i < command.length(); i++)
+    if (command[i] != '\n')
+    {
+      firstLine.append(command[i]);
+    }
+    else
+      break;
+
   if (isLoggingEnabled()) {
     // insert the response to the logger window.
     QFont font(Helper::monospacedFontInfo.family(), Helper::monospacedFontInfo.pointSize() - 2, QFont::Normal, false);
@@ -367,7 +367,7 @@ void OMCProxy::logResponse(QString response, QTime *responseTime)
     if (mpCommunicationLogFile) {
       fputs(QString("%1 %2\n").arg(response).arg(responseTime->currentTime().toString("hh:mm:ss:zzz")).toStdString().c_str(), mpCommunicationLogFile);
       mTotalOMCCallsTime += (double)responseTime->elapsed() / 1000;
-      fputs(QString("%1 secs (%2 secs)\n\n").arg(QString::number((double)responseTime->elapsed() / 1000)).arg(QString::number(mTotalOMCCallsTime)).toStdString().c_str(), mpCommunicationLogFile);
+      fputs(QString("#s#; %1; %2; \'%3\'\n\n").arg(QString::number((double)responseTime->elapsed() / 1000)).arg(QString::number(mTotalOMCCallsTime)).arg(firstLine).toStdString().c_str(),  mpCommunicationLogFile);
     }
   }
 }
@@ -458,13 +458,44 @@ QString OMCProxy::getErrorString(bool warningsAsErrors)
 }
 
 /*!
-  Gets the errors by using the getMessagesStringInternal API.
-  Reads all the errors and add them to the Messages Window.
-  \see MessagesWidget::addGUIMessage
-  \return true if there are any errors otherwise false.'
-  */
+ * \brief OMCProxy::printMessagesStringInternal
+ * Gets the errors by using the getMessagesStringInternal API.
+ * Reads all the errors and add them to the Messages Browser.
+ * Reads the omeditoutput.txt and omediterror.txt files and add the data to Messages Browser if there is any.
+ * \see MessagesWidget::addGUIMessage
+ * \return true if there are any errors otherwise false.
+ */
 bool OMCProxy::printMessagesStringInternal()
 {
+  // read stdout file
+  QFile outputFile(Utilities::tempDirectory() + "/omeditoutput.txt");
+  if (outputFile.open(QIODevice::ReadOnly)) {
+    static qint64 outputFilePosition = 0;
+    if (outputFile.seek(outputFilePosition)) {
+      QString outputFileData = outputFile.readAll();
+      if (!outputFileData.isEmpty()) {
+        outputFilePosition = outputFile.pos();
+        MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0, outputFileData,
+                                                              Helper::scriptingKind, Helper::notificationLevel));
+      }
+    }
+    outputFile.close();
+  }
+  // read stderr file
+  QFile errorFile(Utilities::tempDirectory() + "/omediterror.txt");
+  if (errorFile.open(QIODevice::ReadOnly)) {
+    static qint64 errorFilePosition = 0;
+    if (errorFile.seek(errorFilePosition)) {
+      QString errorFileData = errorFile.readAll();
+      if (!errorFileData.isEmpty()) {
+        errorFilePosition = errorFile.pos();
+        MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0, errorFileData,
+                                                              Helper::scriptingKind, Helper::errorLevel));
+      }
+    }
+    errorFile.close();
+  }
+  // read errors
   int errorsSize = getMessagesStringInternal();
   bool returnValue = errorsSize > 0 ? true : false;
 
@@ -960,7 +991,7 @@ bool OMCProxy::setComponentModifierValue(QString className, QString modifierName
   QString expression;
   if (modifierValue.isEmpty()) {
     expression = QString("setComponentModifierValue(%1, %2, $Code(()))").arg(className).arg(modifierName);
-  } else if (modifierValue.startsWith("(")) {
+  } else if (modifierValue.startsWith("(") && modifierValue.contains("=")) {
     expression = QString("setComponentModifierValue(%1, %2, $Code(%3))").arg(className).arg(modifierName).arg(modifierValue);
   } else {
     expression = QString("setComponentModifierValue(%1, %2, $Code(=%3))").arg(className).arg(modifierName).arg(modifierValue);
@@ -997,7 +1028,12 @@ bool OMCProxy::removeComponentModifiers(QString className, QString name)
  */
 QString OMCProxy::getComponentModifierValues(QString className, QString name)
 {
-  return mpOMCInterface->getComponentModifierValues(className, name);
+  QString values = mpOMCInterface->getComponentModifierValues(className, name);
+  if (values.startsWith(" = ")) {
+    return values.mid(3);
+  } else {
+    return values;
+  }
 }
 
 QStringList OMCProxy::getExtendsModifierNames(QString className, QString extendsClassName)
@@ -1293,21 +1329,21 @@ QString OMCProxy::getDocumentationAnnotation(LibraryTreeItem *pLibraryTreeItem)
      * Otherwise we use monospaced font and put the text inside a div with special style.
      */
     int startPos = docElement.indexOf("<html>", 0, Qt::CaseInsensitive);
-    int endPos = docElement.indexOf("</html>", startPos, Qt::CaseInsensitive);
-    QString startNonHtml, endNonHtml = "";
-    if (startPos > -1 || endPos > -1) {
-      if (endPos > -1) {
-        endPos += 7;  // include </html>
-      }
+    int endPos = -1;
+    if (startPos > -1) {
+      endPos = docElement.indexOf("</html>", startPos + 6, Qt::CaseInsensitive);
+    }
+    if (startPos > -1 && endPos > -1) {
+      QString startNonHtml = "", endNonHtml = "";
       if (startPos < docElement.length()) {
-        startNonHtml = Qt::convertFromPlainText(docElement.left(startPos));
+        startNonHtml = Qt::convertFromPlainText(docElement.left(startPos)); // First startPos number of characters
       }
       if (endPos < docElement.length()) {
-        endNonHtml = Qt::convertFromPlainText(docElement.right(endPos));
+        endNonHtml = Qt::convertFromPlainText(docElement.mid(endPos+7)); // All characters after the position of </html>
       }
       docElement = QString("<div class=\"textDoc\">%1</div><div class=\"htmlDoc\">%2</div><div class=\"textDoc\">%3</div>")
           .arg(startNonHtml)
-          .arg(docElement.mid(startPos, endPos - startPos))
+          .arg(docElement.mid(startPos, endPos - startPos + strlen("</html>")))
           .arg(endNonHtml);
     } else {  // if we have just plain text
       docElement = QString("<div class=\"textDoc\">%1</div>").arg(Qt::convertFromPlainText(docElement));
@@ -1379,7 +1415,11 @@ QString OMCProxy::getClassComment(QString className)
   */
 QString OMCProxy::changeDirectory(QString directory)
 {
-  return mpOMCInterface->cd(directory);
+  QString result = mpOMCInterface->cd(directory);
+  if (result.isEmpty()) {
+    printMessagesStringInternal();
+  }
+  return result;
 }
 
 /*!
@@ -1474,12 +1514,19 @@ QList<QString> OMCProxy::parseString(QString value, QString fileName, bool print
  */
 bool OMCProxy::createClass(QString type, QString className, LibraryTreeItem *pExtendsLibraryTreeItem)
 {
-  QString expression;
-  if (!pExtendsLibraryTreeItem) {
-    expression = QString("%1 %2 end %3;").arg(type).arg(className).arg(className);
+  QString expression, equationOrAlgorithm;
+  if (type.compare("model") == 0) {
+    equationOrAlgorithm = "equation";
+  } else if (type.compare("function") == 0) {
+    equationOrAlgorithm = "algorithm";
   } else {
-    expression = QString("%1 %2 extends %3; end %4;").arg(type).arg(className).arg(pExtendsLibraryTreeItem->getNameStructure())
-        .arg(className);
+    equationOrAlgorithm = "";
+  }
+  if (!pExtendsLibraryTreeItem) {
+    expression = QString("%1 %2 %3 end %4;").arg(type).arg(className).arg(equationOrAlgorithm).arg(className);
+  } else {
+    expression = QString("%1 %2 extends %3; %4 end %5;").arg(type).arg(className).arg(pExtendsLibraryTreeItem->getNameStructure())
+                 .arg(equationOrAlgorithm).arg(className);
   }
   return loadString(expression, className, Helper::utf8, false, false);
 }
@@ -1496,12 +1543,20 @@ bool OMCProxy::createClass(QString type, QString className, LibraryTreeItem *pEx
 bool OMCProxy::createSubClass(QString type, QString className, LibraryTreeItem *pParentLibraryTreeItem,
                               LibraryTreeItem *pExtendsLibraryTreeItem)
 {
-  QString expression;
-  if (!pExtendsLibraryTreeItem) {
-    expression = QString("within %1; %2 %3 end %4;").arg(pParentLibraryTreeItem->getNameStructure()).arg(type).arg(className).arg(className);
+  QString expression, equationOrAlgorithm;
+  if (type.compare("model") == 0) {
+    equationOrAlgorithm = "equation";
+  } else if (type.compare("function") == 0) {
+    equationOrAlgorithm = "algorithm";
   } else {
-    expression = QString("within %1; %2 %3 extends %4; end %5;").arg(pParentLibraryTreeItem->getNameStructure()).arg(type).arg(className)
-        .arg(pExtendsLibraryTreeItem->getNameStructure()).arg(className);
+    equationOrAlgorithm = "";
+  }
+  if (!pExtendsLibraryTreeItem) {
+    expression = QString("within %1; %2 %3 %4 end %5;").arg(pParentLibraryTreeItem->getNameStructure()).arg(type).arg(className)
+                 .arg(equationOrAlgorithm).arg(className);
+  } else {
+    expression = QString("within %1; %2 %3 extends %4; %5 end %6;").arg(pParentLibraryTreeItem->getNameStructure()).arg(type).arg(className)
+                 .arg(pExtendsLibraryTreeItem->getNameStructure()).arg(equationOrAlgorithm).arg(className);
   }
   QString fileName;
   if (pParentLibraryTreeItem->getSaveContentsType() == LibraryTreeItem::SaveInOneFile) {
@@ -1617,7 +1672,7 @@ bool OMCProxy::saveModifiedModel(QString modelText)
  */
 bool OMCProxy::saveTotalModel(QString fileName, QString className)
 {
-  bool result = mpOMCInterface->saveTotalModel(fileName, className);
+  bool result = mpOMCInterface->saveTotalModel(fileName, className, false, false);
   if (!result) {
     printMessagesStringInternal();
   }
@@ -2125,6 +2180,19 @@ bool OMCProxy::translateModel(QString className, QString simualtionParameters)
   return res;
 }
 
+/*!
+ * \brief OMCProxy::readSimulationResultSize
+ * Reads the simulation result size.
+ * \param fileName
+ * \return
+ */
+int OMCProxy::readSimulationResultSize(QString fileName)
+{
+  int size = mpOMCInterface->readSimulationResultSize(fileName);
+  getErrorString();
+  return size;
+}
+
 /*!
  * \brief OMCProxy::readSimulationResultVars
  * Reads the simulation result variables from the result file.
@@ -2243,10 +2311,10 @@ OMCInterface::getSimulationOptions_res OMCProxy::getSimulationOptions(QString cl
  * \param platforms
  * \return
  */
-QString OMCProxy::buildModelFMU(QString className, double version, QString type, QString fileNamePrefix, QList<QString> platforms, bool includeResources)
+QString OMCProxy::buildModelFMU(QString className, QString version, QString type, QString fileNamePrefix, QList<QString> platforms, bool includeResources)
 {
   fileNamePrefix = fileNamePrefix.isEmpty() ? "<default>" : fileNamePrefix;
-  QString fmuFileName = mpOMCInterface->buildModelFMU(className, QString::number(version), type, fileNamePrefix, platforms, includeResources);
+  QString fmuFileName = mpOMCInterface->buildModelFMU(className, version, type, fileNamePrefix, platforms, includeResources);
   if (!fmuFileName.isEmpty()) {
     MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel()->loadDependentLibraries(getClassNames());
   }
@@ -2387,7 +2455,7 @@ bool OMCProxy::setIndexReductionMethod(QString method)
 
 /*!
  * \brief OMCProxy::setCommandLineOptions
- * Sets the OMC flags.
+ * Sets the OMC CommandLineOptions.
  * \param options - a space separated list fo OMC command line options e.g. -d=initialization --cheapmatchingAlgorithm=3
  * \return true on success
  */
@@ -2402,7 +2470,7 @@ bool OMCProxy::setCommandLineOptions(QString options)
 
 /*!
  * \brief OMCProxy::clearCommandLineOptions
- * Clears the OMC flags.
+ * Clears the OMC CommandLineOptions.
  * \return true on success
  */
 bool OMCProxy::clearCommandLineOptions()
@@ -2455,7 +2523,8 @@ QString OMCProxy::makeDocumentationUriToFileName(QString documentation)
   }
   // go through the list of links and convert them if needed.
   foreach (QString attribute, attributeMatches) {
-    if (attribute.startsWith("modelica://")) {
+    // ticket:4923 Modelica specification allows both modelica:// and Modelica://
+    if (attribute.startsWith("modelica://") || attribute.startsWith("Modelica://")) {
       QString fileName = uriToFilename(attribute);
 #ifdef WIN32
       documentation = documentation.replace(attribute, "file:///" + fileName);
@@ -2477,7 +2546,7 @@ QString OMCProxy::makeDocumentationUriToFileName(QString documentation)
 QString OMCProxy::uriToFilename(QString uri)
 {
   sendCommand("uriToFilename(\"" + uri + "\")");
-  QString result = StringHandler::removeFirstLastBrackets(getResult());
+  QString result = StringHandler::removeFirstLastParentheses(getResult());
   result = result.prepend("{").append("}");
   QStringList results = StringHandler::unparseStrings(result);
   /* the second argument of uriToFilename result is error string. */
@@ -2911,6 +2980,37 @@ QList<QList<QString > > OMCProxy::getUses(QString className)
   return result;
 }
 
+/*!
+ * \brief OMCProxy::buildEncryptedPackage
+ * Builds the encrypted package.
+ * \param className
+ * \return
+ */
+bool OMCProxy::buildEncryptedPackage(QString className)
+{
+  OMCInterface::buildEncryptedPackage_res result = mpOMCInterface->buildEncryptedPackage(className);
+  printMessagesStringInternal();
+  if (!result.success && !result.commandOutput.isEmpty()) {
+    MessageItem messageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0, result.commandOutput, Helper::scriptingKind, Helper::errorLevel);
+    MessagesWidget::instance()->addGUIMessage(messageItem);
+  }
+  return result.success;
+}
+
+/*!
+ * \brief OMCProxy::loadEncryptedPackage
+ * Loads the encrypted package.
+ * \param fileName
+ * \param workingDirectory
+ * \return
+ */
+bool OMCProxy::loadEncryptedPackage(QString fileName, QString workingDirectory)
+{
+  bool result = mpOMCInterface->loadEncryptedPackage(fileName, workingDirectory);
+  printMessagesStringInternal();
+  return result;
+}
+
 /*!
   \class CustomExpressionBox
   \brief A text box for executing OMC commands.
