@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -29,33 +29,28 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef OMCPROXY_H
 #define OMCPROXY_H
 
 #include "OpenModelicaScriptingAPIQt.h"
-#include "MainWindow.h"
-#include "Component.h"
-#include "StringHandler.h"
-#include "Utilities.h"
+#include "Util/StringHandler.h"
+#include "Util/Helper.h"
+#include "Util/Utilities.h"
 
-class MainWindow;
 class CustomExpressionBox;
 class ComponentInfo;
 class StringHandler;
 class OMCInterface;
 class LibraryTreeItem;
 
-struct cachedOMCCommand
-{
-  QString mOMCCommand;
-  QString mOMCCommandResult;
-};
+typedef struct {
+  QString mFromUnit;
+  QString mToUnit;
+  OMCInterface::convertUnits_res mConvertUnits;
+} UnitConverion;
 
 class OMCProxy : public QObject
 {
@@ -67,7 +62,7 @@ private:
   CustomExpressionBox *mpExpressionTextBox;
   QPushButton *mpOMCLoggerSendButton;
   QPlainTextEdit *mpOMCLoggerTextBox;
-  Label *mpOMCLoggerEnableHintLabel;
+  QTextCursor mOMCLoggerTextCursor;
   QWidget *mpOMCDiffWidget;
   Label *mpOMCDiffBeforeLabel;
   QPlainTextEdit *mpOMCDiffBeforeTextBox;
@@ -78,26 +73,27 @@ private:
   QString mObjectRefFile;
   QList<QString> mCommandsList;
   int mCurrentCommandIndex;
-  QFile mCommunicationLogFile;
-  QTextStream mCommunicationLogFileTextStream;
-  QFile mCommandsMosFile;
-  QTextStream mCommandsLogFileTextStream;
-  MainWindow *mpMainWindow;
-  int mAnnotationVersion;
+  FILE *mpCommunicationLogFile;
+  FILE *mpCommandsLogFile;
+  double mTotalOMCCallsTime;
+  QList<UnitConverion> mUnitConversionList;
+  QMap<QString, QList<QString> > mDerivedUnitsMap;
   OMCInterface *mpOMCInterface;
+  bool mIsLoggingEnabled;
 public:
-  OMCProxy(MainWindow *pMainWindow);
+  OMCProxy(threadData_t *threadData, QWidget *pParent = 0);
   ~OMCProxy();
-  void enableCustomExpression(bool enable);
   void getPreviousCommand();
   void getNextCommand();
-  bool initializeOMC();
+  bool initializeOMC(threadData_t *threadData);
   void quitOMC();
   void sendCommand(const QString expression);
   void setResult(QString value);
   QString getResult();
   void exitApplication();
   void removeObjectRefFile();
+  void setLoggingEnabled(bool enable) {mIsLoggingEnabled = enable;}
+  bool isLoggingEnabled() {return mIsLoggingEnabled;}
   QString getErrorString(bool warningsAsErrors = false);
   bool printMessagesStringInternal();
   int getMessagesStringInternal();
@@ -115,7 +111,7 @@ public:
   void loadSystemLibraries();
   void loadUserLibraries();
   QStringList getClassNames(QString className = QString("AllLoadedClasses"), bool recursive = false, bool qualified = false,
-                            bool sort = false, bool builtin = false, bool showProtected = true);
+                            bool sort = false, bool builtin = false, bool showProtected = true, bool includeConstants = false);
   QStringList searchClassNames(QString searchText, bool findInText = false);
   OMCInterface::getClassInformation_res getClassInformation(QString className);
   bool isPackage(QString className);
@@ -131,6 +127,7 @@ public:
   QString getComponentModifierValue(QString className, QString name);
   bool setComponentModifierValue(QString className, QString name, QString modifierValue);
   bool removeComponentModifiers(QString className, QString name);
+  QString getComponentModifierValues(QString className, QString name);
   QStringList getExtendsModifierNames(QString className, QString extendsClassName);
   QString getExtendsModifierValue(QString className, QString extendsClassName, QString modifierName);
   bool setExtendsModifierValue(QString className, QString extendsClassName, QString modifierName, QString modifierValue);
@@ -139,8 +136,10 @@ public:
   QString getIconAnnotation(QString className);
   QString getDiagramAnnotation(QString className);
   int getConnectionCount(QString className);
-  QString getNthConnection(QString className, int num);
+  QList<QString> getNthConnection(QString className, int index);
   QString getNthConnectionAnnotation(QString className, int num);
+  QList<QList<QString> > getTransitions(QString className);
+  QList<QList<QString> > getInitialStates(QString className);
   int getInheritanceCount(QString className);
   QString getNthInheritedClass(QString className, int num);
   QList<QString> getInheritedClasses(QString className);
@@ -148,6 +147,7 @@ public:
   QStringList getComponentAnnotations(QString className);
   QString getDocumentationAnnotationInfoHeader(LibraryTreeItem *pLibraryTreeItem, QString infoHeader);
   QString getDocumentationAnnotation(LibraryTreeItem *pLibraryTreeItem);
+  QList<QString> getDocumentationAnnotationInClass(LibraryTreeItem *pLibraryTreeItem);
   QString getClassComment(QString className);
   QString changeDirectory(QString directory = QString(""));
   bool loadModel(QString className, QString priorityVersion = QString("default"), bool notify = false, QString languageStandard = QString(""),
@@ -155,7 +155,7 @@ public:
   bool loadFile(QString fileName, QString encoding = Helper::utf8, bool uses = true);
   bool loadString(QString value, QString fileName, QString encoding = Helper::utf8, bool merge = false, bool checkError = true);
   QList<QString> parseFile(QString fileName, QString encoding = Helper::utf8);
-  QList<QString> parseString(QString value, QString fileName);
+  QList<QString> parseString(QString value, QString fileName, bool printErrors = true);
   bool createClass(QString type, QString className, LibraryTreeItem *pExtendsLibraryTreeItem);
   bool createSubClass(QString type, QString className, LibraryTreeItem *pParentLibraryTreeItem, LibraryTreeItem *pExtendsLibraryTreeItem);
   bool existClass(QString className);
@@ -185,19 +185,31 @@ public:
   bool setComponentDimensions(QString className, QString componentName, QString dimensions);
   bool addConnection(QString from, QString to, QString className, QString annotation);
   bool deleteConnection(QString from, QString to, QString className);
+  bool addTransition(QString className, QString from, QString to, QString condition, bool immediate, bool reset, bool synchronize,
+                     int priority, QString annotation);
+  bool deleteTransition(QString className, QString from, QString to, QString condition, bool immediate, bool reset, bool synchronize,
+                        int priority);
+  bool updateTransition(QString className, QString from, QString to, QString oldCondition, bool oldImmediate, bool oldReset,
+                        bool oldSynchronize, int oldPriority, QString condition, bool immediate, bool reset, bool synchronize, int priority,
+                        QString annotation);
+  bool addInitialState(QString className, QString state, QString annotation);
+  bool deleteInitialState(QString className, QString state);
+  bool updateInitialState(QString className, QString state, QString annotation);
   bool simulate(QString className, QString simualtionParameters);
   bool buildModel(QString className, QString simualtionParameters);
   bool translateModel(QString className, QString simualtionParameters);
+  int readSimulationResultSize(QString fileName);
   QStringList readSimulationResultVars(QString fileName);
   bool closeSimulationResultFile();
   QString checkModel(QString className);
   bool ngspicetoModelica(QString fileName);
   QString checkAllModelsRecursive(QString className);
   bool isExperiment(QString className);
-  OMCInterface::getSimulationOptions_res getSimulationOptions(QString className, double defaultTolerance = 1e-4);
-  bool translateModelFMU(QString className, double version, QString type, QString fileNamePrefix);
-  bool translateModelXML(QString className);
+  OMCInterface::getSimulationOptions_res getSimulationOptions(QString className, double defaultTolerance = 1e-6);
+  QString buildModelFMU(QString className, QString version, QString type, QString fileNamePrefix, QList<QString> platforms, bool includeResources = true);
+  QString translateModelXML(QString className);
   QString importFMU(QString fmuName, QString outputDirectory, int logLevel, bool debugLogging, bool generateInputConnectors, bool generateOutputConnectors);
+  QString importFMUModelDescription(QString fmuModelDescriptionName, QString outputDirectory, int logLevel, bool debugLogging, bool generateInputConnectors, bool generateOutputConnectors);
   QString getMatchingAlgorithm();
   OMCInterface::getAvailableMatchingAlgorithms_res getAvailableMatchingAlgorithms();
   bool setMatchingAlgorithm(QString matchingAlgorithm);
@@ -211,7 +223,15 @@ public:
   QString getModelicaPath();
   QStringList getAvailableLibraries();
   QString getDerivedClassModifierValue(QString className, QString modifierName);
+  OMCInterface::convertUnits_res convertUnits(QString from, QString to);
+  QList<QString> getDerivedUnits(QString baseUnit);
+  QString getVersionDateAnnotation(QString className);
+  QString getVersionBuildAnnotation(QString className);
   bool getDocumentationClassAnnotation(QString className);
+  QString getCommandLineOptionsAnnotation(QString className);
+  QList<QString> getAnnotationNamedModifiers(QString className, QString annotation);
+  QString getAnnotationModifierValue(QString className, QString annotation, QString modifier);
+  QString getSimulationFlagsAnnotation(QString className);
   int numProcessors();
   QString help(QString topic);
   OMCInterface::getConfigFlagValidOptions_res getConfigFlagValidOptions(QString topic);
@@ -220,19 +240,25 @@ public:
   bool copyClass(QString className, QString newClassName, QString withIn);
   QStringList getEnumerationLiterals(QString className);
   void getSolverMethods(QStringList *methods, QStringList *descriptions);
+  void getJacobianMethods(QStringList *methods, QStringList *descriptions);
+  QString getJacobianFlagDetailedDescription();
   void getInitializationMethods(QStringList *methods, QStringList *descriptions);
   void getLinearSolvers(QStringList *methods, QStringList *descriptions);
   void getNonLinearSolvers(QStringList *methods, QStringList *descriptions);
+  void getLogStreams(QStringList *names, QStringList *descriptions);
   bool moveClass(QString className, int offset);
   bool moveClassToTop(QString className);
   bool moveClassToBottom(QString className);
   bool inferBindings(QString className);
+  bool generateVerificationScenarios(QString className);
   QList<QList<QString > > getUses(QString className);
+  bool buildEncryptedPackage(QString className);
+  bool loadEncryptedPackage(QString fileName, QString workingDirectory);
 signals:
   void commandFinished();
 public slots:
   void logCommand(QString command, QTime *commandTime);
-  void logResponse(QString response, QTime *responseTime);
+  void logResponse(QString command, QString response, QTime *responseTime);
   void showException(QString exception);
   void openOMCLoggerWidget();
   void sendCustomExpression();
