@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -29,24 +29,32 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef OPTIONSDIALOG_H
 #define OPTIONSDIALOG_H
 
-#include "MainWindow.h"
-#include "Helper.h"
-#include "Utilities.h"
+#include "Util/Helper.h"
+#include "Util/Utilities.h"
+
+#include <QFontComboBox>
+#include <QStackedWidget>
+#include <QDialogButtonBox>
+#include <QRadioButton>
+#include <QTreeWidget>
+#include <QDialog>
+#include <QLineEdit>
 
-class MainWindow;
 class GeneralSettingsPage;
 class LibrariesPage;
-class ModelicaTextHighlighter;
-class ModelicaTextEditorPage;
+class TextEditorPage;
+class ModelicaEditorPage;
+class MetaModelicaEditorPage;
+class CompositeModelEditorPage;
+class OMSimulatorEditorPage;
+class CEditorPage;
+class HTMLEditorPage;
 class GraphicalViewsPage;
 class SimulationPage;
 class MessagesPage;
@@ -58,18 +66,35 @@ class FigaroPage;
 class DebuggerPage;
 class FMIPage;
 class TLMPage;
-class TLMEditorPage;
+class OMSimulatorPage;
+class TraceabilityPage;
 class TabSettings;
+class StackFramesWidget;
 
 class OptionsDialog : public QDialog
 {
   Q_OBJECT
+private:
+  // the only class that is allowed to create and destroy
+  friend class MainWindow;
+
+  static void create();
+  static void destroy();
+  OptionsDialog(QWidget *pParent = 0);
+
+  static OptionsDialog *mpInstance;
 public:
-  OptionsDialog(MainWindow *pMainWindow);
+  static OptionsDialog* instance() {return mpInstance;}
   void readSettings();
   void readGeneralSettings();
   void readLibrariesSettings();
-  void readModelicaTextSettings();
+  void readTextEditorSettings();
+  void readModelicaEditorSettings();
+  void readMetaModelicaEditorSettings();
+  void readCompositeModelEditorSettings();
+  void readOMSimulatorEditorSettings();
+  void readCEditorSettings();
+  void readHTMLEditorSettings();
   void readGraphicalViewsSettings();
   void readSimulationSettings();
   void readMessagesSettings();
@@ -81,12 +106,20 @@ public:
   void readDebuggerSettings();
   void readFMISettings();
   void readTLMSettings();
-  void readTLMEditorSettings();
+  void readOMSimulatorSettings();
+  void readTraceabilitySettings();
   void saveGeneralSettings();
   void saveLibrariesSettings();
-  void saveModelicaTextSettings();
+  void saveTextEditorSettings();
+  void saveModelicaEditorSettings();
+  void saveMetaModelicaEditorSettings();
+  void saveCompositeModelEditorSettings();
+  void saveOMSimulatorEditorSettings();
+  void saveCEditorSettings();
+  void saveHTMLEditorSettings();
   void saveTLMSettings();
-  void saveTLMEditorSettings();
+  void saveOMSimulatorSettings();
+  void saveTraceabilitySettings();
   void saveGraphicalViewsSettings();
   void saveSimulationSettings();
   void saveMessagesSettings();
@@ -100,9 +133,15 @@ public:
   void setUpDialog();
   void addListItems();
   void createPages();
-  MainWindow* getMainWindow() {return mpMainWindow;}
   GeneralSettingsPage* getGeneralSettingsPage() {return mpGeneralSettingsPage;}
-  ModelicaTextEditorPage* getModelicaTextEditorPage() {return mpModelicaTextEditorPage;}
+  LibrariesPage* getLibrariesPage() {return mpLibrariesPage;}
+  TextEditorPage* getTextEditorPage() {return mpTextEditorPage;}
+  ModelicaEditorPage* getModelicaEditorPage() {return mpModelicaEditorPage;}
+  MetaModelicaEditorPage* getMetaModelicaEditorPage() {return mpMetaModelicaEditorPage;}
+  CompositeModelEditorPage* getCompositeModelEditorPage() {return mpCompositeModelEditorPage;}
+  OMSimulatorEditorPage* getOMSimulatorEditorPage() {return mpOMSimulatorEditorPage;}
+  CEditorPage* getCEditorPage() {return mpCEditorPage;}
+  HTMLEditorPage* getHTMLEditorPage() {return mpHTMLEditorPage;}
   GraphicalViewsPage* getGraphicalViewsPage() {return mpGraphicalViewsPage;}
   SimulationPage* getSimulationPage() {return mpSimulationPage;}
   MessagesPage* getMessagesPage() {return mpMessagesPage;}
@@ -114,24 +153,34 @@ public:
   DebuggerPage* getDebuggerPage() {return mpDebuggerPage;}
   FMIPage* getFMIPage() {return mpFMIPage;}
   TLMPage* getTLMPage() {return mpTLMPage;}
-  TLMEditorPage* getTLMEditorPage() {return mpTLMEditorPage;}
+  OMSimulatorPage* getOMSimulatorPage() {return mpOMSimulatorPage;}
+  TraceabilityPage* getTraceabilityPage() {return mpTraceabilityPage;}
+  void emitModelicaEditorSettingsChanged() {emit modelicaEditorSettingsChanged();}
   void saveDialogGeometry();
   void show();
-  TabSettings getModelicaTabSettings();
-  TabSettings getTLMTabSettings();
+  TabSettings getTabSettings();
 signals:
-  void modelicaTextSettingsChanged();
-  void TLMEditorSettingsChanged();
-  void updateLineWrapping();
+  void textSettingsChanged();
+  void modelicaEditorSettingsChanged();
+  void metaModelicaEditorSettingsChanged();
+  void compositeModelEditorSettingsChanged();
+  void omsimulatorEditorSettingsChanged();
+  void cEditorSettingsChanged();
+  void HTMLEditorSettingsChanged();
 public slots:
   void changePage(QListWidgetItem *current, QListWidgetItem *previous);
   void reject();
   void saveSettings();
 private:
-  MainWindow *mpMainWindow;
   GeneralSettingsPage *mpGeneralSettingsPage;
   LibrariesPage *mpLibrariesPage;
-  ModelicaTextEditorPage *mpModelicaTextEditorPage;
+  TextEditorPage *mpTextEditorPage;
+  ModelicaEditorPage *mpModelicaEditorPage;
+  MetaModelicaEditorPage *mpMetaModelicaEditorPage;
+  CompositeModelEditorPage *mpCompositeModelEditorPage;
+  OMSimulatorEditorPage *mpOMSimulatorEditorPage;
+  CEditorPage *mpCEditorPage;
+  HTMLEditorPage *mpHTMLEditorPage;
   GraphicalViewsPage *mpGraphicalViewsPage;
   SimulationPage *mpSimulationPage;
   MessagesPage *mpMessagesPage;
@@ -143,10 +192,12 @@ private:
   DebuggerPage *mpDebuggerPage;
   FMIPage *mpFMIPage;
   TLMPage *mpTLMPage;
-  TLMEditorPage *mpTLMEditorPage;
+  OMSimulatorPage *mpOMSimulatorPage;
+  TraceabilityPage *mpTraceabilityPage;
   QSettings *mpSettings;
   QListWidget *mpOptionsList;
   QStackedWidget *mpPagesWidget;
+  QScrollArea *mpPagesWidgetScrollArea;
   Label *mpChangesEffectLabel;
   QPushButton *mpCancelButton;
   QPushButton *mpOkButton;
@@ -158,30 +209,32 @@ class GeneralSettingsPage : public QWidget
   Q_OBJECT
 public:
   GeneralSettingsPage(OptionsDialog *pOptionsDialog);
-  QComboBox* getLanguageComboBox();
-  void setWorkingDirectory(QString value);
-  QString getWorkingDirectory();
+  QComboBox* getLanguageComboBox() {return mpLanguageComboBox;}
+  void setWorkingDirectory(QString value) {mpWorkingDirectoryTextBox->setText(value);}
+  QString getWorkingDirectory() {return mpWorkingDirectoryTextBox->text();}
   QSpinBox* getToolbarIconSizeSpinBox() {return mpToolbarIconSizeSpinBox;}
-  void setPreserveUserCustomizations(bool value);
-  bool getPreserveUserCustomizations();
+  void setPreserveUserCustomizations(bool value) {mpPreserveUserCustomizations->setChecked(value);}
+  bool getPreserveUserCustomizations() {return mpPreserveUserCustomizations->isChecked();}
   void setTerminalCommand(QString value) {mpTerminalCommandTextBox->setText(value);}
   QString getTerminalCommand() {return mpTerminalCommandTextBox->text();}
   void setTerminalCommandArguments(QString value) {mpTerminalCommandArgumentsTextBox->setText(value);}
   QString getTerminalCommandArguments() {return mpTerminalCommandArgumentsTextBox->text();}
+  QCheckBox* getHideVariablesBrowserCheckBox() {return mpHideVariablesBrowserCheckBox;}
+  QCheckBox* getActivateAccessAnnotationsCheckBox() {return mpActivateAccessAnnotationsCheckBox;}
   QSpinBox* getLibraryIconSizeSpinBox() {return mpLibraryIconSizeSpinBox;}
-  void setShowProtectedClasses(bool value);
-  bool getShowProtectedClasses();
+  void setShowProtectedClasses(bool value) {mpShowProtectedClasses->setChecked(value);}
+  bool getShowProtectedClasses() {return mpShowProtectedClasses->isChecked();}
+  void setShowHiddenClasses(bool value) {mpShowHiddenClasses->setChecked(value);}
+  bool getShowHiddenClasses() {return mpShowHiddenClasses->isChecked();}
   void setModelingViewMode(QString value);
   QString getModelingViewMode();
   void setDefaultView(QString value);
   QString getDefaultView();
-  QGroupBox* getEnableAutoSaveGroupBox();
-  QSpinBox* getAutoSaveIntervalSpinBox();
-  QCheckBox* getEnableAutoSaveForSingleClassesCheckBox();
-  QCheckBox* getEnableAutoSaveForOneFilePackagesCheckBox();
+  QGroupBox* getEnableAutoSaveGroupBox() {return mpEnableAutoSaveGroupBox;}
+  QSpinBox* getAutoSaveIntervalSpinBox() {return mpAutoSaveIntervalSpinBox;}
   int getWelcomePageView();
   void setWelcomePageView(int view);
-  QCheckBox* getShowLatestNewsCheckBox();
+  QCheckBox* getShowLatestNewsCheckBox() {return mpShowLatestNewsCheckBox;}
 private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpGeneralSettingsGroupBox;
@@ -198,10 +251,13 @@ private:
   QPushButton *mpTerminalCommandBrowseButton;
   Label *mpTerminalCommandArgumentsLabel;
   QLineEdit *mpTerminalCommandArgumentsTextBox;
+  QCheckBox *mpHideVariablesBrowserCheckBox;
+  QCheckBox *mpActivateAccessAnnotationsCheckBox;
   QGroupBox *mpLibrariesBrowserGroupBox;
   Label *mpLibraryIconSizeLabel;
   QSpinBox *mpLibraryIconSizeSpinBox;
   QCheckBox *mpShowProtectedClasses;
+  QCheckBox *mpShowHiddenClasses;
   QGroupBox *mpModelingViewModeGroupBox;
   QRadioButton *mpModelingTabbedViewRadioButton;
   QRadioButton *mpModelingSubWindowViewRadioButton;
@@ -214,8 +270,6 @@ private:
   Label *mpAutoSaveIntervalLabel;
   QSpinBox *mpAutoSaveIntervalSpinBox;
   Label *mpAutoSaveSecondsLabel;
-  QCheckBox *mpEnableAutoSaveForSingleClassesCheckBox;
-  QCheckBox *mpEnableAutoSaveForOneFilePackagesCheckBox;
   QGroupBox *mpWelcomePageGroupBox;
   QRadioButton *mpHorizontalViewRadioButton;
   QRadioButton *mpVerticalViewRadioButton;
@@ -231,9 +285,10 @@ class LibrariesPage : public QWidget
   Q_OBJECT
 public:
   LibrariesPage(OptionsDialog *pOptionsDialog);
-  QTreeWidget* getSystemLibrariesTree();
-  QCheckBox* getForceModelicaLoadCheckBox();
-  QTreeWidget* getUserLibrariesTree();
+  QTreeWidget* getSystemLibrariesTree() {return mpSystemLibrariesTree;}
+  QCheckBox* getForceModelicaLoadCheckBox() {return mpForceModelicaLoadCheckBox;}
+  QCheckBox* getLoadOpenModelicaLibraryCheckBox() {return mpLoadOpenModelicaOnStartupCheckBox;}
+  QTreeWidget* getUserLibrariesTree() {return mpUserLibrariesTree;}
   OptionsDialog *mpOptionsDialog;
 private:
   QGroupBox *mpSystemLibrariesGroupBox;
@@ -244,6 +299,7 @@ private:
   QPushButton *mpEditSystemLibraryButton;
   QDialogButtonBox *mpSystemLibrariesButtonBox;
   QCheckBox *mpForceModelicaLoadCheckBox;
+  QCheckBox *mpLoadOpenModelicaOnStartupCheckBox;
   QGroupBox *mpUserLibrariesGroupBox;
   QTreeWidget *mpUserLibrariesTree;
   QPushButton *mpAddUserLibraryButton;
@@ -298,37 +354,30 @@ private slots:
   void addUserLibrary();
 };
 
-class ModelicaTextEditorPage : public QWidget
+class TextEditorPage : public QWidget
 {
   Q_OBJECT
 public:
-  ModelicaTextEditorPage(OptionsDialog *pOptionsDialog);
+  TextEditorPage(OptionsDialog *pOptionsDialog);
+  QComboBox *getLineEndingComboBox() {return mpLineEndingComboBox;}
+  QComboBox *getBOMComboBox() {return mpBOMComboBox;}
   QComboBox *getTabPolicyComboBox() {return mpTabPolicyComboBox;}
   QSpinBox *getTabSizeSpinBox() {return mpTabSizeSpinBox;}
   QSpinBox *getIndentSpinBox() {return mpIndentSpinBox;}
-  QCheckBox *getPreserveTextIndentationCheckBox() {return mpPreserveTextIndentationCheckBox;}
-  QCheckBox* getSyntaxHighlightingCheckbox() {return mpSyntaxHighlightingCheckbox;}
+  QGroupBox* getSyntaxHighlightingGroupBox() {return mpSyntaxHighlightingGroupBox;}
+  QCheckBox* getAutoCompleteCheckBox() {return mpAutoCompleteCheckBox;}
+  QCheckBox* getCodeFoldingCheckBox() {return mpCodeFoldingCheckBox;}
   QCheckBox* getMatchParenthesesCommentsQuotesCheckBox() {return mpMatchParenthesesCommentsQuotesCheckBox;}
   QCheckBox* getLineWrappingCheckbox() {return mpLineWrappingCheckbox;}
   QFontComboBox* getFontFamilyComboBox() {return mpFontFamilyComboBox;}
   DoubleSpinBox* getFontSizeSpinBox() {return mpFontSizeSpinBox;}
-  void addListItems();
-  void setTextRuleColor(QColor color);
-  QColor getTextRuleColor() {return mTextColor;}
-  void setNumberRuleColor(QColor color);
-  QColor getNumberRuleColor() {return mNumberColor;}
-  void setKeywordRuleColor(QColor color);
-  QColor getKeywordRuleColor() {return mKeywordColor;}
-  void setTypeRuleColor(QColor color);
-  QColor getTypeRuleColor() {return mTypeColor;}
-  void setFunctionRuleColor(QColor color);
-  QColor getFunctionRuleColor() {return mFunctionColor;}
-  void setQuotesRuleColor(QColor color);
-  QColor getQuotesRuleColor() {return mQuotesColor;}
-  void setCommentRuleColor(QColor color);
-  QColor getCommentRuleColor() {return mCommentColor;}
 private:
   OptionsDialog *mpOptionsDialog;
+  QGroupBox *mpFormatGroupBox;
+  Label *mpLineEndingLabel;
+  QComboBox *mpLineEndingComboBox;
+  Label *mpBOMLabel;
+  QComboBox *mpBOMComboBox;
   QGroupBox *mpTabsAndIndentation;
   Label *mpTabPolicyLabel;
   QComboBox *mpTabPolicyComboBox;
@@ -336,41 +385,128 @@ private:
   QSpinBox *mpTabSizeSpinBox;
   Label *mpIndentSizeLabel;
   QSpinBox *mpIndentSpinBox;
-  QCheckBox *mpPreserveTextIndentationCheckBox;
   QGroupBox *mpSyntaxHighlightAndTextWrappingGroupBox;
-  QCheckBox *mpSyntaxHighlightingCheckbox;
+  QGroupBox *mpSyntaxHighlightingGroupBox;
+  QGroupBox *mpAutoCompleteGroupBox;
+  QCheckBox *mpAutoCompleteCheckBox;
+  QCheckBox *mpCodeFoldingCheckBox;
   QCheckBox *mpMatchParenthesesCommentsQuotesCheckBox;
   QCheckBox *mpLineWrappingCheckbox;
-  QGroupBox *mpFontColorsGroupBox;
+  QGroupBox *mpFontGroupBox;
   Label *mpFontFamilyLabel;
   QFontComboBox *mpFontFamilyComboBox;
   Label *mpFontSizeLabel;
   DoubleSpinBox *mpFontSizeSpinBox;
-  Label *mpItemsLabel;
-  QListWidget *mpItemsList;
-  Label *mpItemColorLabel;
-  QPushButton *mpItemColorPickButton;
-  Label *mpPreviewLabel;
-  QPlainTextEdit *mpPreviewPlainTextBox;
-  QColor mTextColor;
-  QListWidgetItem *mpTextItem;
-  QColor mNumberColor;
-  QListWidgetItem *mpNumberItem;
-  QColor mKeywordColor;
-  QListWidgetItem *mpKeywordItem;
-  QColor mTypeColor;
-  QListWidgetItem *mpTypeItem;
-  QColor mFunctionColor;
-  QListWidgetItem *mpFunctionItem;
-  QColor mQuotesColor;
-  QListWidgetItem *mpQuotesItem;
-  QColor mCommentColor;
-  QListWidgetItem *mpCommentItem;
+};
+
+class ModelicaEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  ModelicaEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  QCheckBox *getPreserveTextIndentationCheckBox() {return mpPreserveTextIndentationCheckBox;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  QCheckBox *mpPreserveTextIndentationCheckBox;
+  CodeColorsWidget *mpCodeColorsWidget;
 signals:
   void updatePreview();
 public slots:
-  void setLineWrapping();
-  void pickColor();
+  void setLineWrapping(bool enabled);
+};
+
+class MetaModelicaEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  MetaModelicaEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  CodeColorsWidget *mpCodeColorsWidget;
+signals:
+  void updatePreview();
+public slots:
+  void setLineWrapping(bool enabled);
+};
+
+class CompositeModelEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  CompositeModelEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  CodeColorsWidget *mpCodeColorsWidget;
+signals:
+  void updatePreview();
+public slots:
+  void setLineWrapping(bool enabled);
+};
+
+class OMSimulatorEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  OMSimulatorEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  CodeColorsWidget *mpCodeColorsWidget;
+signals:
+  void updatePreview();
+public slots:
+  void setLineWrapping(bool enabled);
+};
+
+class CEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  CEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  CodeColorsWidget *mpCodeColorsWidget;
+signals:
+  void updatePreview();
+public slots:
+  void setLineWrapping(bool enabled);
+};
+
+class HTMLEditorPage : public QWidget
+{
+  Q_OBJECT
+public:
+  HTMLEditorPage(OptionsDialog *pOptionsDialog);
+  OptionsDialog* getOptionsDialog() {return mpOptionsDialog;}
+  void setColor(QString item, QColor color);
+  QColor getColor(QString item);
+  void emitUpdatePreview() {emit updatePreview();}
+private:
+  OptionsDialog *mpOptionsDialog;
+  CodeColorsWidget *mpCodeColorsWidget;
+signals:
+  void updatePreview();
+public slots:
+  void setLineWrapping(bool enabled);
 };
 
 class GraphicalViewsPage : public QWidget
@@ -462,8 +598,14 @@ public:
   QComboBox* getIndexReductionMethodComboBox() {return mpIndexReductionMethodComboBox;}
   QComboBox* getTargetLanguageComboBox() {return mpTargetLanguageComboBox;}
   QComboBox* getTargetCompilerComboBox() {return mpTargetCompilerComboBox;}
-  QLineEdit* getOMCFlagsTextBox() {return mpOMCFlagsTextBox;}
-  QCheckBox *getSaveClassBeforeSimulationCheckBox() {return mpSaveClassBeforeSimulationCheckBox;}
+  QLineEdit* getOMCCommandLineOptionsTextBox() {return mpOMCCommandLineOptionsTextBox;}
+  QCheckBox* getIgnoreCommandLineOptionsAnnotationCheckBox() {return mpIgnoreCommandLineOptionsAnnotationCheckBox;}
+  QCheckBox* getIgnoreSimulationFlagsAnnotationCheckBox() {return mpIgnoreSimulationFlagsAnnotationCheckBox;}
+  QCheckBox* getSaveClassBeforeSimulationCheckBox() {return mpSaveClassBeforeSimulationCheckBox;}
+  QCheckBox* getSwitchToPlottingPerspectiveCheckBox() {return mpSwitchToPlottingPerspectiveCheckBox;}
+  QCheckBox* getCloseSimulationOutputWidgetsBeforeSimulationCheckBox() {return mpCloseSimulationOutputWidgetsBeforeSimulationCheckBox;}
+  QCheckBox* getDeleteIntermediateCompilationFilesCheckBox() {return mpDeleteIntermediateCompilationFilesCheckBox;}
+  QCheckBox* getDeleteEntireSimulationDirectoryCheckBox() {return mpDeleteEntireSimulationDirectoryCheckBox;}
   void setOutputMode(QString value);
   QString getOutputMode();
 private:
@@ -477,15 +619,23 @@ private:
   QComboBox *mpTargetLanguageComboBox;
   Label *mpCompilerLabel;
   QComboBox *mpTargetCompilerComboBox;
-  Label *mpOMCFlagsLabel;
-  QLineEdit *mpOMCFlagsTextBox;
+  Label *mpOMCCommandLineOptionsLabel;
+  QLineEdit *mpOMCCommandLineOptionsTextBox;
+  QToolButton *mpOMCCommandLineOptionsHelpButton;
+  QCheckBox *mpIgnoreCommandLineOptionsAnnotationCheckBox;
+  QCheckBox *mpIgnoreSimulationFlagsAnnotationCheckBox;
   QCheckBox *mpSaveClassBeforeSimulationCheckBox;
+  QCheckBox *mpSwitchToPlottingPerspectiveCheckBox;
+  QCheckBox *mpCloseSimulationOutputWidgetsBeforeSimulationCheckBox;
+  QCheckBox *mpDeleteIntermediateCompilationFilesCheckBox;
+  QCheckBox *mpDeleteEntireSimulationDirectoryCheckBox;
   QGroupBox *mpOutputGroupBox;
   QRadioButton *mpStructuredRadioButton;
   QRadioButton *mpFormattedTextRadioButton;
 public slots:
   void updateMatchingAlgorithmToolTip(int index);
   void updateIndexReductionToolTip(int index);
+  void showOMCCommandLineOptionsHelp();
 };
 
 class MessagesPage : public QWidget
@@ -495,6 +645,7 @@ public:
   MessagesPage(OptionsDialog *pOptionsDialog);
   QSpinBox* getOutputSizeSpinBox() {return mpOutputSizeSpinBox;}
   QCheckBox* getResetMessagesNumberBeforeSimulationCheckBox() {return mpResetMessagesNumberBeforeSimulationCheckBox;}
+  QCheckBox* getClearMessagesBrowserBeforeSimulationCheckBox() {return mpClearMessagesBrowserBeforeSimulationCheckBox;}
   QFontComboBox* getFontFamilyComboBox() {return mpFontFamilyComboBox;}
   DoubleSpinBox* getFontSizeSpinBox() {return mpFontSizeSpinBox;}
   void setNotificationColor(QColor color) {mNotificaitonColor = color;}
@@ -512,6 +663,7 @@ private:
   Label *mpOutputSizeLabel;
   QSpinBox *mpOutputSizeSpinBox;
   QCheckBox *mpResetMessagesNumberBeforeSimulationCheckBox;
+  QCheckBox *mpClearMessagesBrowserBeforeSimulationCheckBox;
   QGroupBox *mpFontColorsGroupBox;
   Label *mpFontFamilyLabel;
   QFontComboBox *mpFontFamilyComboBox;
@@ -537,11 +689,13 @@ class NotificationsPage : public QWidget
   Q_OBJECT
 public:
   NotificationsPage(OptionsDialog *pOptionsDialog);
-  QCheckBox* getQuitApplicationCheckBox();
-  QCheckBox* getItemDroppedOnItselfCheckBox();
-  QCheckBox* getReplaceableIfPartialCheckBox();
-  QCheckBox* getInnerModelNameChangedCheckBox();
-  QCheckBox* getSaveModelForBitmapInsertionCheckBox();
+  QCheckBox* getQuitApplicationCheckBox() {return mpQuitApplicationCheckBox;}
+  QCheckBox* getItemDroppedOnItselfCheckBox() {return mpItemDroppedOnItselfCheckBox;}
+  QCheckBox* getReplaceableIfPartialCheckBox() {return mpReplaceableIfPartialCheckBox;}
+  QCheckBox* getInnerModelNameChangedCheckBox() {return mpInnerModelNameChangedCheckBox;}
+  QCheckBox* getSaveModelForBitmapInsertionCheckBox() {return mpSaveModelForBitmapInsertionCheckBox;}
+  QCheckBox* getAlwaysAskForDraggedComponentName() {return mpAlwaysAskForDraggedComponentName;}
+  QCheckBox* getAlwaysAskForTextEditorErrorCheckBox() {return mpAlwaysAskForTextEditorErrorCheckBox;}
 private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpNotificationsGroupBox;
@@ -550,6 +704,8 @@ private:
   QCheckBox *mpReplaceableIfPartialCheckBox;
   QCheckBox *mpInnerModelNameChangedCheckBox;
   QCheckBox *mpSaveModelForBitmapInsertionCheckBox;
+  QCheckBox *mpAlwaysAskForDraggedComponentName;
+  QCheckBox *mpAlwaysAskForTextEditorErrorCheckBox;
 };
 
 class LineStylePage : public QWidget
@@ -660,12 +816,15 @@ private:
   QLineEdit *mpFigaroOptionsFileTextBox;
   QPushButton *mpBrowseFigaroOptionsFileButton;
   Label *mpFigaroProcessLabel;
+  QString mFigaroProcessPath;
   QLineEdit *mpFigaroProcessTextBox;
   QPushButton *mpBrowseFigaroProcessButton;
+  QPushButton *mpResetFigaroProcessButton;
 private slots:
   void browseFigaroLibraryFile();
   void browseFigaroOptionsFile();
   void browseFigaroProcessFile();
+  void resetFigaroProcessPath();
 };
 
 class DebuggerPage : public QWidget
@@ -675,6 +834,7 @@ public:
   DebuggerPage(OptionsDialog *pOptionsDialog);
   void setGDBPath(QString path);
   QString getGDBPath();
+  QString getGDBPathForSettings() {return mpGDBPathTextBox->text();}
   QSpinBox* getGDBCommandTimeoutSpinBox() {return mpGDBCommandTimeoutSpinBox;}
   QSpinBox* getGDBOutputLimitSpinBox() {return mpGDBOutputLimitSpinBox;}
   QCheckBox* getDisplayCFramesCheckBox() {return mpDisplayCFramesCheckBox;}
@@ -709,11 +869,14 @@ class FMIPage : public QWidget
   Q_OBJECT
 public:
   FMIPage(OptionsDialog *pOptionsDialog);
-  void setFMIExportVersion(double version);
-  double getFMIExportVersion();
+  void setFMIExportVersion(QString version);
+  QString getFMIExportVersion();
   void setFMIExportType(QString type);
   QString getFMIExportType();
   QLineEdit* getFMUNameTextBox() {return mpFMUNameTextBox;}
+  QGroupBox* getPlatformsGroupBox() {return mpPlatformsGroupBox;}
+  QComboBox* getLinkingComboBox() {return mpLinkingComboBox;}
+  QCheckBox* getDeleteFMUDirectoryAndModelCheckBox() {return mpDeleteFMUDirectoryAndModelCheckBox;}
 private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpExportGroupBox;
@@ -726,6 +889,10 @@ private:
   QRadioButton *mpModelExchangeCoSimulationRadioButton;
   Label *mpFMUNameLabel;
   QLineEdit *mpFMUNameTextBox;
+  QGroupBox *mpPlatformsGroupBox;
+  QComboBox *mpLinkingComboBox;
+  QGroupBox *mpImportGroupBox;
+  QCheckBox *mpDeleteFMUDirectoryAndModelCheckBox;
 };
 
 class TLMPage : public QWidget
@@ -733,6 +900,9 @@ class TLMPage : public QWidget
   Q_OBJECT
 public:
   TLMPage(OptionsDialog *pOptionsDialog);
+  QString getOMTLMSimulatorPath();
+  QString getOMTLMSimulatorManagerPath();
+  QString getOMTLMSimulatorMonitorPath();
   QLineEdit* getTLMPluginPathTextBox() {return mpTLMPluginPathTextBox;}
   QLineEdit* getTLMManagerProcessTextBox() {return mpTLMManagerProcessTextBox;}
   QLineEdit* getTLMMonitorProcessTextBox() {return mpTLMMonitorProcessTextBox;}
@@ -754,67 +924,59 @@ private slots:
   void browseTLMMonitorProcess();
 };
 
-class TLMEditorPage : public QWidget
+class OMSimulatorPage : public QWidget
 {
   Q_OBJECT
 public:
-  TLMEditorPage(OptionsDialog *pOptionsDialog);
-  QComboBox *getTabPolicyComboBox() {return mpTabPolicyComboBox;}
-  QSpinBox *getTabSizeSpinBox() {return mpTabSizeSpinBox;}
-  QSpinBox *getIndentSpinBox() {return mpIndentSpinBox;}
-  QCheckBox* getSyntaxHighlightingCheckbox() {return mpSyntaxHighlightingCheckbox;}
-  QCheckBox* getLineWrappingCheckbox() {return mpLineWrappingCheckbox;}
-  QFontComboBox* getFontFamilyComboBox() {return mpFontFamilyComboBox;}
-  DoubleSpinBox* getFontSizeSpinBox() {return mpFontSizeSpinBox;}
-  void addListItems();
-  void setTextRuleColor(QColor color);
-  QColor getTextRuleColor(){return mTextColor;}
-  void setQuotesRuleColor(QColor color);
-  QColor getQuotesRuleColor(){return mQuotesColor;}
-  void setCommentRuleColor(QColor color);
-  QColor getCommentRuleColor(){return mCommentColor;}
-  void setTagRuleColor(QColor color);
-  QColor getTagRuleColor(){return mTagColor;}
-  void setElementRuleColor(QColor color);
-  QColor getElementRuleColor(){return mElementColor;}
+  OMSimulatorPage(OptionsDialog *pOptionsDialog);
+  void setWorkingDirectory(QString value) {mpWorkingDirectoryTextBox->setText(value);}
+  QString getWorkingDirectory() {return mpWorkingDirectoryTextBox->text();}
+  QComboBox* getLoggingLevelComboBox() {return mpLoggingLevelComboBox;}
 private:
   OptionsDialog *mpOptionsDialog;
-  QGroupBox *mpTabsAndIndentation;
-  Label *mpTabPolicyLabel;
-  QComboBox *mpTabPolicyComboBox;
-  Label *mpTabSizeLabel;
-  QSpinBox *mpTabSizeSpinBox;
-  Label *mpIndentSizeLabel;
-  QSpinBox *mpIndentSpinBox;
-  QGroupBox *mpSyntaxHighlightAndTextWrappingGroupBox;
-  QCheckBox *mpSyntaxHighlightingCheckbox;
-  QCheckBox *mpLineWrappingCheckbox;
-  QGroupBox *mpFontColorsGroupBox;
-  Label *mpFontFamilyLabel;
-  QFontComboBox *mpFontFamilyComboBox;
-  Label *mpFontSizeLabel;
-  DoubleSpinBox *mpFontSizeSpinBox;
-  Label *mpItemsLabel;
-  QListWidget *mpItemsList;
-  Label *mpItemColorLabel;
-  QPushButton *mpItemColorPickButton;
-  Label *mpPreviewLabel;
-  QPlainTextEdit *mpPreviewPlainTextBox;
-  QColor mTextColor;
-  QListWidgetItem *mpTextItem;
-  QColor mQuotesColor;
-  QListWidgetItem *mpQuotesItem;
-  QColor mCommentColor;
-  QListWidgetItem *mpCommentItem;
-  QColor mTagColor;
-  QListWidgetItem *mpTagItem;
-  QColor mElementColor;
-  QListWidgetItem *mpElementItem;
-signals:
-  void updatePreview();
-public slots:
-  void setLineWrapping();
-  void pickColor();
+  QGroupBox *mpGeneralGroupBox;
+  Label *mpWorkingDirectoryLabel;
+  QLineEdit *mpWorkingDirectoryTextBox;
+  QPushButton *mpBrowseWorkingDirectoryButton;
+  Label *mpLoggingLevelLabel;
+  QComboBox *mpLoggingLevelComboBox;
+private slots:
+  void browseWorkingDirectory();
+};
+
+class TraceabilityPage : public QWidget
+{
+  Q_OBJECT
+public:
+  TraceabilityPage(OptionsDialog *pOptionsDialog);
+  QGroupBox* getTraceabilityGroupBox() {return mpTraceabilityGroupBox;}
+//  QLineEdit* getFMUOutputDirectory() {return mpFMUOutputDirectoryTextBox;}
+//  QPushButton *mpBrowseFMUOutputDirectoryButton;
+  QPushButton *mpBrowseGitRepositoryButton;
+  QLineEdit* getTraceabilityDaemonIpAdress() {return mpTraceabilityDaemonIpAdressTextBox;}
+  QLineEdit* getTraceabilityDaemonPort() {return mpTraceabilityDaemonPortTextBox;}
+  QLineEdit* getUserName() {return mpUserNameTextBox;}
+  QLineEdit* getEmail() {return mpEmailTextBox;}
+  QLineEdit* getGitRepository() {return mpGitRepositoryTextBox;}
+private:
+  OptionsDialog *mpOptionsDialog;
+  QGroupBox *mpTraceabilityGroupBox;
+  Label *mpUserNameLabel;
+  QLineEdit *mpUserNameTextBox;
+  Label *mpEmailLabel;
+  QLineEdit *mpEmailTextBox;
+  Label *mpGitRepositoryLabel;
+  QLineEdit *mpGitRepositoryTextBox;
+//  Label *mpFMUOutputDirectoryLabel;
+//  QLineEdit *mpFMUOutputDirectoryTextBox;
+  Label *mpTraceabilityDaemonIpAdressLabel;
+  QLineEdit *mpTraceabilityDaemonIpAdressTextBox;
+  Label *mpTraceabilityDaemonPortLabel;
+  QLineEdit *mpTraceabilityDaemonPortTextBox;
+private slots:
+//  void browseFMUOutputDirectory();
+  void browseGitRepository();
+
 };
 
 #endif // OPTIONSDIALOG_H
