@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -43,6 +43,8 @@
 #include "OMPlot.h"
 
 class AnimationWindow;
+class DiagramWindow;
+class VariablesTreeItem;
 
 class PlotWindowContainer : public QMdiArea
 {
@@ -51,15 +53,30 @@ public:
   PlotWindowContainer(QWidget *pParent = 0);
   QString getUniqueName(QString name = QString("Plot"), int number = 1);
   OMPlot::PlotWindow* getCurrentWindow();
+  OMPlot::PlotWindow* getInteractiveWindow(QString targetWindow);
+  OMPlot::PlotWindow* getTopPlotWindow();
+  void setTopPlotWindowActive();
 #if !defined(WITHOUT_OSG)
   AnimationWindow* getCurrentAnimationWindow();
 #endif
+  QMdiSubWindow* getDiagramSubWindowFromMdi();
+  DiagramWindow* getDiagramWindow() {return mpDiagramWindow;}
+  bool isPlotWindow(QObject *pObject);
+  bool isAnimationWindow(QObject *pObject);
+  bool isDiagramWindow(QObject *pObject);
   bool eventFilter(QObject *pObject, QEvent *pEvent);
+private:
+  DiagramWindow *mpDiagramWindow;
 public slots:
   void addPlotWindow(bool maximized = false);
   void addParametricPlotWindow();
+  void addArrayPlotWindow(bool maximized = false);
+  void addArrayParametricPlotWindow();
+  OMPlot::PlotWindow* addInteractivePlotWindow(bool maximized = false, QString owner = QString(), int port = 0);
   void addAnimationWindow(bool maximized = false);
+  void addDiagramWindow(bool maximized = false);
   void clearPlotWindow();
+  void removeInteractivePlotWindow();
   void exportVariables();
   void updatePlotWindows(QString variable);
 };
