@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -39,6 +39,7 @@
 
 #include "Simulation/SimulationOptions.h"
 #include "PlotWindow.h"
+#include "Animation/TimeManager.h"
 
 class OMCProxy;
 class TreeSearchFilters;
@@ -64,9 +65,13 @@ public:
   void setChecked(bool set) {mChecked = set;}
   bool isEditable() const {return mEditable;}
   void setEditable(bool set) {mEditable = set;}
+  void setVariability(QString variability) {mVariability = variability;}
+  bool isParameter() const {return mVariability.compare("parameter") == 0;}
   bool isMainArray() const {return mIsMainArray;}
   SimulationOptions getSimulationOptions() {return mSimulationOptions;}
   void setSimulationOptions(SimulationOptions simulationOptions) {mSimulationOptions = simulationOptions;}
+  bool isActive() const {return mActive;}
+  void setActive();
   QIcon getVariableTreeItemIcon(QString name) const;
   void insertChild(int position, VariablesTreeItem *pVariablesTreeItem);
   VariablesTreeItem* child(int row);
@@ -98,8 +103,11 @@ private:
   QString mToolTip;
   bool mChecked;
   bool mEditable;
+  QString mVariability;
   bool mIsMainArray;
   SimulationOptions mSimulationOptions;
+protected:
+  bool mActive;
 };
 
 class VariablesTreeView;
@@ -109,6 +117,7 @@ class VariablesTreeModel : public QAbstractItemModel
 public:
   VariablesTreeModel(VariablesTreeView *pVariablesTreeView = 0);
   VariablesTreeItem* getRootVariablesTreeItem() {return mpRootVariablesTreeItem;}
+  VariablesTreeItem* getActiveVariablesTreeItem() {return mpActiveVariablesTreeItem;}
   int columnCount(const QModelIndex &parent = QModelIndex()) const;
   int rowCount(const QModelIndex &parent = QModelIndex()) const;
   QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
@@ -130,15 +139,18 @@ public:
 private:
   VariablesTreeView *mpVariablesTreeView;
   VariablesTreeItem *mpRootVariablesTreeItem;
+  VariablesTreeItem *mpActiveVariablesTreeItem;
   QHash<QString, QHash<QString,QString> > mScalarVariablesList;
-  void getVariableInformation(ModelicaMatReader *pMatReader, QString variableToFind, QString *value, bool *changeAble, QString *unit,
-                              QString *displayUnit, QString *description);
+  void getVariableInformation(ModelicaMatReader *pMatReader, QString variableToFind, QString *value, bool *changeAble, QString *variability,
+                              QString *unit, QString *displayUnit, QString *description);
 signals:
   void itemChecked(const QModelIndex &index, qreal curveThickness, int curveStyle);
   void unitChanged(const QModelIndex &index);
+  void valueEntered(const QModelIndex &index);
   void variableTreeItemRemoved(QString variable);
 public slots:
   void removeVariableTreeItem();
+  void setVariableTreeItemActive();
 };
 
 class VariableTreeProxyModel : public QSortFilterProxyModel
@@ -162,6 +174,7 @@ private:
   VariablesWidget *mpVariablesWidget;
 protected:
   virtual void mouseReleaseEvent(QMouseEvent *event);
+  virtual void keyPressEvent(QKeyEvent *event);
 };
 
 class VariablesWidget : public QWidget
@@ -174,35 +187,66 @@ public:
   VariablesTreeModel* getVariablesTreeModel() {return mpVariablesTreeModel;}
   VariablesTreeView* getVariablesTreeView() {return mpVariablesTreeView;}
   void insertVariablesItemsToTree(QString fileName, QString filePath, QStringList variablesList, SimulationOptions simulationOptions);
+  void addSelectedInteractiveVariables(const QString &modelName, const QList<QString> &selectedVariables);
   void variablesUpdated();
   void updateVariablesTreeHelper(QMdiSubWindow *pSubWindow);
   void readVariablesAndUpdateXML(VariablesTreeItem *pVariablesTreeItem, QString outputFileName,
                                  QHash<QString, QHash<QString, QString> > *variables);
   void findVariableAndUpdateValue(QDomDocument xmlDocument, QHash<QString, QHash<QString, QString> > variables);
   void reSimulate(bool showSetup);
+  void interactiveReSimulation(QString modelName);
   void updateInitXmlFile(SimulationOptions simulationOptions);
+  void initializeVisualization(SimulationOptions simulationOptions);
+  double readVariableValue(QString variable, double time);
 private:
   TreeSearchFilters *mpTreeSearchFilters;
   Label *mpSimulationTimeLabel;
   QComboBox *mpSimulationTimeComboBox;
   QSlider *mpSimulationTimeSlider;
+  QToolBar *mpToolBar;
+  QAction *mpRewindAction;
+  QAction *mpPlayAction;
+  QAction *mpPauseAction;
+  Label *mpTimeLabel;
+  QLineEdit *mpTimeTextBox;
+  Label *mpSpeedLabel;
+  QComboBox *mpSpeedComboBox;
+  TimeManager *mpTimeManager;
   VariableTreeProxyModel *mpVariableTreeProxyModel;
   VariablesTreeModel *mpVariablesTreeModel;
   VariablesTreeView *mpVariablesTreeView;
   QList<QStringList> mPlotParametricVariables;
+  QHash<QString, QList<QString>> mSelectedInteractiveVariables;
   QString mFileName;
   QMdiSubWindow *mpLastActiveSubWindow;
+  ModelicaMatReader mModelicaMatReader;
+  csv_data *mpCSVData;
+  QFile mPlotFileReader;
+  void selectInteractivePlotWindow(VariablesTreeItem *pVariablesTreeItem);
+  void closeResultFile();
+  void openResultFile();
+  void updateVisualization();
 public slots:
-  void plotVariables(const QModelIndex &index, qreal curveThickness, int curveStyle, OMPlot::PlotCurve *pPlotCurve = 0,
-                     OMPlot::PlotWindow *pPlotWindow = 0);
+  void plotVariables(const QModelIndex &index, qreal curveThickness, int curveStyle,
+                     OMPlot::PlotCurve *pPlotCurve = 0, OMPlot::PlotWindow *pPlotWindow = 0);
   void unitChanged(const QModelIndex &index);
   void simulationTimeChanged(int timePercent);
+  void valueEntered(const QModelIndex &index);
   void timeUnitChanged(QString unit);
   void updateVariablesTree(QMdiSubWindow *pSubWindow);
   void showContextMenu(QPoint point);
   void findVariables();
   void directReSimulate();
   void showReSimulateSetup();
+  void rewindVisualization();
+private slots:
+  void playVisualization();
+  void pauseVisualization();
+  void visulizationTimeChanged();
+  void visualizationSpeedChanged();
+  void incrementVisualization();
+signals:
+  void updateDynamicSelect(double time);
 };
 
 #endif // VARIABLESWIDGET_H
