@@ -187,15 +187,15 @@ void TLMCoSimulationDialog::show(LibraryTreeItem *pLibraryTreeItem)
   mpHeadingLabel->setText(QString("%1 - %2").arg(Helper::tlmCoSimulationSetup).arg(mpLibraryTreeItem->getName()));
   // if user has nothing in TLM plugin path then read from OptionsDialog
   if (mpTLMPluginPathTextBox->text().isEmpty()) {
-    mpTLMPluginPathTextBox->setText(OptionsDialog::instance()->getTLMPage()->getTLMPluginPathTextBox()->text());
+    mpTLMPluginPathTextBox->setText(OptionsDialog::instance()->getTLMPage()->getOMTLMSimulatorPath());
   }
   // if user has nothing in manager process box then read from OptionsDialog
   if (mpManagerProcessTextBox->text().isEmpty()) {
-    mpManagerProcessTextBox->setText(OptionsDialog::instance()->getTLMPage()->getTLMManagerProcessTextBox()->text());
+    mpManagerProcessTextBox->setText(OptionsDialog::instance()->getTLMPage()->getOMTLMSimulatorManagerPath());
   }
   // if user has nothing in monitor process box then read from OptionsDialog
   if (mpMonitorProcessTextBox->text().isEmpty()) {
-    mpMonitorProcessTextBox->setText(OptionsDialog::instance()->getTLMPage()->getTLMMonitorProcessTextBox()->text());
+    mpMonitorProcessTextBox->setText(OptionsDialog::instance()->getTLMPage()->getOMTLMSimulatorMonitorPath());
   }
   setVisible(true);
 }
@@ -314,7 +314,7 @@ TLMCoSimulationOptions TLMCoSimulationDialog::createTLMCoSimulationOptions()
     gethostname(myname, MAXHOSTNAME);
     hp = gethostbyname((const char*) myname);
     if (hp == NULL) {
-      MessageItem messageItem(MessageItem::MetaModel, "", false, 0, 0, 0, 0,
+      MessageItem messageItem(MessageItem::CompositeModel, "", false, 0, 0, 0, 0,
                               tr("Failed to get my hostname, check that name resolves, e.g. /etc/hosts has %1")
                               .arg(QString(myname)), Helper::scriptingKind, Helper::errorLevel);
       MessagesWidget::instance()->addGUIMessage(messageItem);
@@ -404,7 +404,10 @@ void TLMCoSimulationDialog::runTLMCoSimulation()
         MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel()->showModelWidget(mpLibraryTreeItem, false);
       }
       mpLibraryTreeItem->getModelWidget()->createModelWidgetComponents();
-      mpLibraryTreeItem->getModelWidget()->writeVisualXMLFile();
+      // write the visual xml file
+      QFileInfo fileInfo(mpLibraryTreeItem->getFileName());
+      QString fileName = QString("%1/%2_visual.xml").arg(fileInfo.absolutePath()).arg(fileInfo.baseName());
+      mpLibraryTreeItem->getModelWidget()->writeVisualXMLFile(fileName);
       mpTLMCoSimulationOutputWidget->showTLMCoSimulationOutputWidget(tlmCoSimulationOptions);
       showTLMCoSimulationOutputWindow();
       accept();
@@ -413,10 +416,10 @@ void TLMCoSimulationDialog::runTLMCoSimulation()
 }
 
 /*!
- * \brief MetaModelSimulationParamsDialog::MetaModelSimulationParamsDialog
+ * \brief CompositeModelSimulationParamsDialog::CompositeModelSimulationParamsDialog
  * \param pGraphicsView
  */
-MetaModelSimulationParamsDialog::MetaModelSimulationParamsDialog(GraphicsView *pGraphicsView)
+CompositeModelSimulationParamsDialog::CompositeModelSimulationParamsDialog(GraphicsView *pGraphicsView)
   : QDialog(pGraphicsView)
 {
   setAttribute(Qt::WA_DeleteOnClose);
@@ -432,17 +435,17 @@ MetaModelSimulationParamsDialog::MetaModelSimulationParamsDialog(GraphicsView *p
   // Initialize simulation parameters
   mOldStartTime = "";
   mOldStopTime = "";
-  MetaModelEditor *pMetaModelEditor = dynamic_cast<MetaModelEditor*>(mpGraphicsView->getModelWidget()->getEditor());
-  if(pMetaModelEditor->isSimulationParams()){
-    mOldStartTime = pMetaModelEditor->getSimulationStartTime();
-    mOldStopTime = pMetaModelEditor->getSimulationStopTime();
+  CompositeModelEditor *pCompositeModelEditor = dynamic_cast<CompositeModelEditor*>(mpGraphicsView->getModelWidget()->getEditor());
+  if(pCompositeModelEditor->isSimulationParams()){
+    mOldStartTime = pCompositeModelEditor->getSimulationStartTime();
+    mOldStopTime = pCompositeModelEditor->getSimulationStopTime();
   }
   // CoSimulation Interval
-  mpStartTimeLabel = new Label(tr("Start Time:"));
+  mpStartTimeLabel = new Label(QString("%1:").arg(Helper::startTime));
   mpStartTimeTextBox = new QLineEdit(mOldStartTime);
-  mpStopTimeLabel = new Label(tr("Stop Time:"));
+  mpStopTimeLabel = new Label(QString("%1:").arg(Helper::stopTime));
   mpStopTimeTextBox = new QLineEdit(mOldStopTime);
-    // Add the validators
+  // Add the validators
   QDoubleValidator *pDoubleValidator = new QDoubleValidator(this);
   mpStartTimeTextBox->setValidator(pDoubleValidator);
   mpStopTimeTextBox->setValidator(pDoubleValidator);
@@ -470,11 +473,11 @@ MetaModelSimulationParamsDialog::MetaModelSimulationParamsDialog(GraphicsView *p
 }
 
 /*!
- * \brief MetaModelSimulationParamsDialog::saveSimulationParams
+ * \brief CompositeModelSimulationParamsDialog::saveSimulationParams
  * Saves the Simulation Parameters.
  * Slot activated when mpSave button clicked signal is raised.
  */
-void MetaModelSimulationParamsDialog::saveSimulationParams()
+void CompositeModelSimulationParamsDialog::saveSimulationParams()
 {
   if (validateSimulationParams()) {
     // If user has changed the simulation parameters then push the change on the stack.
@@ -492,7 +495,7 @@ void MetaModelSimulationParamsDialog::saveSimulationParams()
 /*!
   Validates the simulation params entered by the user.
   */
-bool MetaModelSimulationParamsDialog::validateSimulationParams()
+bool CompositeModelSimulationParamsDialog::validateSimulationParams()
 {
   if (mpStartTimeTextBox->text().isEmpty()) {
     mpStartTimeTextBox->setText("0");
