@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -67,6 +67,10 @@ public:
     Debug,
     OMEditInfo  /* used internally by OMEdit to mark message blue. */
   };
+  enum TLMCausality { TLMBidirectional, TLMInput, TLMOutput };
+  static QString getTLMCausality(int causality);
+  enum TLMDomain { Mechanical, Electric, Hydraulic, Pneumatic, Magnetic, Signal };
+  static QString getTLMDomain(int domain);
   enum SimulationTools {Adams, Beast, Dymola, OpenModelica, Simulink, WolframSystemModeler, Other};
   static QString getSimulationTool(int tool);
   static QString getSimulationToolStartCommand(QString tool, QString simulationToolStartCommand);
@@ -103,8 +107,10 @@ public:
   static QString getTextAlignmentString(StringHandler::TextAlignment alignment);
   static QString getTextStyleString(StringHandler::TextStyle textStyle);
   static QString removeFirstLastCurlBrackets(QString value);
-  static QString removeFirstLastBrackets(QString value);
+  static QString removeFirstLastParentheses(QString value);
+  static QString removeFirstLastSquareBrackets(QString value);
   static QString removeFirstLastQuotes(QString value);
+  static QString removeFirstLastSingleQuotes(QString value);
   static QStringList getStrings(QString value);
   static QStringList getStrings(QString value, char start, char end);
   /* Handles quoted identifiers A.B.'C.D' -> A.B, A.B.C.D -> A.B.C */
@@ -133,12 +139,13 @@ public:
   static QString getExistingDirectory(QWidget* parent = 0, const QString &caption = "", QString * dir = 0);
   static void setLastOpenDirectory(QString lastOpenDirectory);
   static QString getLastOpenDirectory();
-  static QStringList getDialogAnnotation(QString componentAnnotation);
+  static QStringList getAnnotation(QString componentAnnotation, QString annotationName);
   static QString getPlacementAnnotation(QString componentAnnotation);
   static qreal getNormalizedAngle(qreal angle);
   static QStringList splitStringWithSpaces(QString value);
   static void fillEncodingComboBox(QComboBox *pEncodingComboBox);
   static QStringList makeVariableParts(QString variable);
+  static QStringList makeVariablePartsWithInd(QString variable);
   static bool naturalSort(const QString &s1, const QString &s2);
 #ifdef WIN32
   static QProcessEnvironment simulationProcessEnvironment();
@@ -148,10 +155,11 @@ public:
   static QColor getSimulationMessageTypeColor(StringHandler::SimulationMessageType type);
   static QString makeClassNameRelative(QString draggedClassName, QString droppedClassName);
   static QString toCamelCase(QString str);
-  static QMap<int, int> getLeadingSpaces(QString contents);
+  static QMap<int, int> getLeadingSpaces(QString contents, int tabWidth);
   static int getLeadingSpacesSize(QString str);
   static bool isFileWritAble(QString filePath);
   static bool containsSpace(QString str);
+  static QString joinDerivativeAndPreviousVariable(QString fullVariableName, QString variableName, QString derivativeOrPrevious);
 protected:
   static QString mLastOpenDir;
 };
